/*
 *  Intel i960CA Dependent Source
 *
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#include "fatal.h"
#include "isr.h"

/*  _CPU_Initialize
 *
 *  This routine performs processor dependent initialization.
 *
 *  INPUT PARAMETERS:
 *    cpu_table - CPU table to initialize
 *
 *  OUTPUT PARAMETERS: NONE
 */

void _CPU_Initialize(
  CPU_Table *cpu_table
)
{

  if ( cpu_table == NULL ) 
    _Fatal_Error_occurred( STATUS_NOT_CONFIGURED );

  _CPU_Table = cpu_table;

}

/*  _CPU__ISR_Install_vector
 *
 *  Install the RTEMS vector wrapper in the CPU's interrupt table.
 *
 *  Input parameters:  
 *    vector      - interrupt vector number
 *    old_handler - former ISR for this vector number
 *    new_handler - replacement ISR for this vector number
 * 
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#define _Is_vector_caching_enabled( _prcb ) \
   ((_prcb)->control_tbl->icon & 0x2000)

void _CPU_ISR_install_vector( 
  unsigned32  vector,
  proc_ptr    new_handler,
  proc_ptr   *old_handler
)
{
  CPU_I80960ca_PRCB *prcb = _CPU_Table->Prcb;
  proc_ptr           *cached_intr_tbl = NULL;

/*  The i80960CA does not support vectors 0-7.  The first 9 entries 
 *  in the Interrupt Table are used to manage pending interrupts.
 *  Thus vector 8, the first valid vector number, is actually in
 *  slot 9 in the table.
 */

  *old_handler = _ISR_Vector_table[ vector ];

  _ISR_Vector_table[ vector ] = new_handler;

  prcb->intr_tbl[ vector + 1 ] = _ISR_Handler;
  if ( _Is_vector_caching_enabled( prcb ) )
    if ( (vector & 0xf) == 0x2 )       /* cacheable? */
      cached_intr_tbl[ vector >> 4 ] = _ISR_Handler;

}
