/*  cpu.h
 *
 *  This include file contains information pertaining to the Intel
 *  i80960CA processor.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __CPU_h
#define __CPU_h

#pragma align 4            /* for GNU C structure alignment */

/* conditional compilation parameters */

#define CPU_INLINE_ENABLE_DISPATCH    FALSE
#define CPU_UNROLL_ENQUEUE_PRIORITY   FALSE

/* constants */

#define CPU_MODES_INTERRUPT_LEVEL  0x0000001f  /* interrupt level in mode */
#define CPU_MODES_INTERRUPT_MASK   0x0000001f  /* interrupt level in mode */

#define CPU_HARDWARE_FP     FALSE              /* NO FP for i80960 CA */

#define CPU_CONTEXT_FP_SIZE 0                  /* NO FP for i80960 CA */


#define CPU_SYSTEM_INITIALIZATION_THREAD_EXTRA_STACK \
   (4096 - CPU_STACK_MINIMUM_SIZE)   /* yields 2K total */

#define CPU_STACK_MINIMUM_SIZE          1024

#define CPU_INTERRUPT_NUMBER_OF_VECTORS 256    /* number of INTR vectors  */

#define CPU_HEAP_PAGE_SIZE_ALIGNMENT    4      /* on 4 byte boundary */

/* macros */

/* ISR handler macros */

#define _CPU_ISR_Disable( _level ) \
  { \
    (_level) = 0x1f0000; \
    __asm__ volatile ( "modpc   0,%1,%1" : "=d" ((_level)) \
                                     : "0"  ((_level)) ); \
  }

#define _CPU_ISR_Enable( _level ) \
  { \
    unsigned32 _mask = 0x1f0000; \
    \
    __asm__ volatile ( "modpc   0,%0,%1" : "=d" (_mask), "=d" ((_level)) \
                                     : "0"  (_mask), "1"  ((_level)) ); \
  }

#define _CPU_ISR_Flash( _level ) \
  { \
    unsigned32 _mask = 0x1f0000; \
    \
    __asm__ volatile ( "modpc   0,%0,%1" : "=d" (_mask), "=d" ((_level)) \
                                     : "0"  (_mask), "1"  ((_level)) ); \
    __asm__ volatile ( "mov     %0,%1"   : "=d" (_mask), "=d" ((_level)) \
                                     : "0"  (_mask), "1"  ((_level)) ); \
    __asm__ volatile ( "modpc   0,%0,%1" : "=d" (_mask), "=d" ((_level)) \
                                     : "0"  (_mask), "1"  ((_level)) ); \
  }

#define _CPU_ISR_Set_level( newlevel ) \
  { \
    unsigned32 _mask, _level=(newlevel); \
    \
    __asm__ volatile ( "ldconst 0x1f0000,%0; \
                    modpc   0,%0,%1"     : "=d" (_mask), "=d" (_level) \
                                         : "0"  (_mask), "1" (_level) \
    ); \
  }

/* ISR handler section macros */

/* Context handler macros */

#define _CPU_Context_Initialize( _the_context, _stack, _isr, _entry ) \
 { CPU_Call_frame *_texit_frame; \
   unsigned32 _mask; \
   unsigned32 _base_pc; \
   \
   __asm__ volatile ( "flushreg" : : );   /* flush register cache */ \
   \
   (_the_context)->r0_pfp = (_stack); \
   (_the_context)->g15_fp = (_stack) + (1 * sizeof(CPU_Call_frame)); \
   (_the_context)->r1_sp  = (_stack) + (2 * sizeof(CPU_Call_frame)); \
   __asm__ volatile ( "ldconst 0x1f0000,%0 ; " \
                  "modpc   0,0,%1 ; " \
                  "andnot  %0,%1,%1 ; " \
                  : "=d" (_mask), "=d" (_base_pc) : ); \
   (_the_context)->pc     = _base_pc | ((_isr) << 16); \
   (_the_context)->g14    = 0; \
   \
   _texit_frame         = (CPU_Call_frame *)(_stack); \
   _texit_frame->r0_pfp = NULL; \
   _texit_frame->r1_sp  = (_the_context)->g15_fp; \
   _texit_frame->r2_rip = (_entry); \
 }

#define _CPU_Context_Restart_self( _the_context ) \
   _Context_Restore( (_the_context) );

#define _CPU_Context_Fp_start( _base, _offset )         NULL
  
#define _CPU_Context_Initialize_fp( _fp_area ) 

/* end of Context handler macros */

/* Fatal Error Manager macros */

#define _CPU_Fatal_halt( _errorcode ) \
  { unsigned32 _mask, _level; \
    unsigned32 _error = (_errorcode); \
    \
    __asm__ volatile ( "ldconst 0x1f0000,%0 ; \
                    mov     %0,%1 ; \
                    modpc   0,%0,%1 ; \
                    mov     %2,g0 ; \
            self:   b       self " \
                    : "=d" (_mask), "=d" (_level), "=d" (_error) : ); \
  }

/* end of Fatal Error Manager macros */

/* Bitfield handler macros */

#define _CPU_Bitfield_Find_first_bit( _value, _output ) \
  { unsigned32 _search = (_value); \
    \
    __asm__ volatile ( "scanbit   %0,%1  " \
                    : "=d" (_search), "=d" (_output) \
                    : "0"  (_search), "1"  (_output) ); \
  }

/* end of Bitfield handler macros */

/* Priority handler macros */

#define _CPU_Priority_Mask( _bit_number ) \
   ( 0x8000 >> (_bit_number) )

#define _CPU_Priority_Bits_index( _priority ) \
   ( 15 - (_priority) )

/* end of Priority handler macros */

/* Stack handler macros */

#define _CPU_Stack_Adjust_size( _size, _new_size ) \
  (_new_size) = (_size) + 16

#define _CPU_Stack_Get_initial_pointer( _low_address, _size ) \
  (void *)(((unsigned32)(_low_address) + 16) & 0xfffffff0)

/* end of Stack handler macros */

/* miscellaneous */

#define _CPU_not_aligned( _the_address ) \
  ((unsigned32)(_the_address) & 0x3)

/* structures */

/* processor dependent RTEMS thread structures */

typedef struct {
  void       *r0_pfp;                 /* (r0)  Previous Frame Pointer */
  void       *r1_sp;                  /* (r1)  Stack Pointer */
  unsigned32  pc;                     /* (pc)  Processor Control */
  void       *g8;                     /* (g8)  Global Register 8 */
  void       *g9;                     /* (g9)  Global Register 9 */
  void       *g10;                    /* (g10) Global Register 10 */
  void       *g11;                    /* (g11) Global Register 11 */
  void       *g12;                    /* (g12) Global Register 12 */
  void       *g13;                    /* (g13) Global Register 13 */
  unsigned32  g14;                    /* (g14) Global Register 14 */
  void       *g15_fp;                 /* (g15) Frame Pointer */
}   Context_Control;

typedef struct {
  void       *r0_pfp;                 /* (r0)  Previous Frame Pointer */
  void       *r1_sp;                  /* (r1)  Stack Pointer */
  void       *r2_rip;                 /* (r2)  Return Instruction Pointer */
  void       *r3;                     /* (r3)  Local Register 3 */
  void       *r4;                     /* (r4)  Local Register 4 */
  void       *r5;                     /* (r5)  Local Register 5 */
  void       *r6;                     /* (r6)  Local Register 6 */
  void       *r7;                     /* (r7)  Local Register 7 */
  void       *r8;                     /* (r8)  Local Register 8 */
  void       *r9;                     /* (r9)  Local Register 9 */
  void       *r10;                    /* (r10) Local Register 10 */
  void       *r11;                    /* (r11) Local Register 11 */
  void       *r12;                    /* (r12) Local Register 12 */
  void       *r13;                    /* (r13) Local Register 13 */
  void       *r14;                    /* (r14) Local Register 14 */
  void       *r15;                    /* (r15) Local Register 15 */
}   CPU_Call_frame;

/* i80960CA control structures */

/* Intel i80960CA Control Table */

typedef struct {
                            /* Control Group 0 */
  unsigned32 ipb0;              /* IP breakpoint 0 */
  unsigned32 ipb1;              /* IP breakpoint 1 */
  unsigned32 dab0;              /* data address breakpoint 0 */
  unsigned32 dab1;              /* data address breakpoint 1 */
                            /* Control Group 1 */
  unsigned32 imap0;             /* interrupt map 0 */
  unsigned32 imap1;             /* interrupt map 1 */
  unsigned32 imap2;             /* interrupt map 2 */
  unsigned32 icon;              /* interrupt control */
                            /* Control Group 2 */
  unsigned32 mcon0;             /* memory region 0 configuration */
  unsigned32 mcon1;             /* memory region 1 configuration */
  unsigned32 mcon2;             /* memory region 2 configuration */
  unsigned32 mcon3;             /* memory region 3 configuration */
                            /* Control Group 3 */
  unsigned32 mcon4;             /* memory region 4 configuration */
  unsigned32 mcon5;             /* memory region 5 configuration */
  unsigned32 mcon6;             /* memory region 6 configuration */
  unsigned32 mcon7;             /* memory region 7 configuration */
                            /* Control Group 4 */
  unsigned32 mcon8;             /* memory region 8 configuration */
  unsigned32 mcon9;             /* memory region 9 configuration */
  unsigned32 mcon10;            /* memory region 10 configuration */
  unsigned32 mcon11;            /* memory region 11 configuration */
                            /* Control Group 5 */
  unsigned32 mcon12;            /* memory region 12 configuration */
  unsigned32 mcon13;            /* memory region 13 configuration */
  unsigned32 mcon14;            /* memory region 14 configuration */
  unsigned32 mcon15;            /* memory region 15 configuration */
                            /* Control Group 6 */
  unsigned32 bpcon;             /* breakpoint control */
  unsigned32 tc;                /* trace control */
  unsigned32 bcon;              /* bus configuration control */
  unsigned32 reserved;          /* reserved */
}   CPU_I80960ca_control_table;

/* Intel i80960CA Processor Control Block */

typedef struct {
  unsigned32    *fault_tbl;     /* fault table base address */
  CPU_I80960ca_control_table
                *control_tbl;   /* control table base address */
  unsigned32     initial_ac;    /* AC register initial value */
  unsigned32     fault_config;  /* fault configuration word */
  proc_ptr      *intr_tbl;      /* interrupt table base address */
  proc_ptr      *sys_proc_tbl;  /* system procedure table
                                     base address */
  unsigned32     reserved;      /* reserved */
  unsigned32    *intr_stack;    /* interrupt stack pointer */
  unsigned32     ins_cache_cfg; /* instruction cache
                                     configuration word */
  unsigned32     reg_cache_cfg; /* register cache configuration word */
}   CPU_I80960ca_PRCB;

typedef struct {
  CPU_I80960ca_PRCB *Prcb;           
}   CPU_Table;

/* functions */

void _CPU_Context_restore(
  Context_Control  *heirthread
);

void _CPU_Context_restore_fp(
  void        **fp_context_ptr
);

void _CPU_Context_save_fp(
  void        **fp_context_ptr
);

void _CPU_Context_switch(
  Context_Control  *run,
  Context_Control  *heir
);

void _CPU_Initialize( 
  CPU_Table *cpu_table
);

void _CPU_ISR_install_vector(
  unsigned32  vector,
  proc_ptr    new_handler,
  proc_ptr   *old_handler
);

#endif
/* end of include file */
