/*  attr.h
 *
 *  This include file contains all information about the Object Attributes
 *  Handler.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __ATTRIBUTES_h
#define __ATTRIBUTES_h

/* types */

typedef unsigned32 Attributes_Control;

/* constants */

#define ATTRIBUTES_DEFAULTS            0x00000000

#define ATTRIBUTES_NO_FLOATING_POINT   0x00000000 /* don't use FP HW */
#if ( CPU_HARDWARE_FP == TRUE )
#define ATTRIBUTES_FLOATING_POINT      0x00000001 /* utilize coprocessor */
#else
#define ATTRIBUTES_FLOATING_POINT      0x00000000 /* no coprocessor */
#endif

#define ATTRIBUTES_LOCAL               0x00000000 /* local resource */
#define ATTRIBUTES_GLOBAL              0x00000002 /* global resource */

#define ATTRIBUTES_FIFO                0x00000000 /* process FIFO */
#define ATTRIBUTES_PRIORITY            0x00000004 /* process by priority */

#define ATTRIBUTES_NO_LIMIT            0x00000000 /* unlimited Q entries */
#define ATTRIBUTES_LIMIT               0x00000008 /* limit Q entries */

#define ATTRIBUTES_COUNTING_SEMAPHORE  0x00000000
#define ATTRIBUTES_BINARY_SEMAPHORE    0x00000010

#define ATTRIBUTES_NO_INHERIT_PRIORITY 0x00000000
#define ATTRIBUTES_INHERIT_PRIORITY    0x00000020

/*
 *  _Attributes_Is_floating_point
 *
 *  DESCRIPTION:
 *
 *  This function returns TRUE if the floating point attribute is
 *  enabled in the attribute_set and FALSE otherwise.
 */

STATIC INLINE boolean _Attributes_Is_floating_point(
  Attributes_Control attribute_set
);

/*
 *  _Attributes_Is_global
 *
 *  DESCRIPTION:
 *
 *  This function returns TRUE if the global object attribute is
 *  enabled in the attribute_set and FALSE otherwise.
 */

STATIC INLINE boolean _Attributes_Is_global(
  Attributes_Control attribute_set
);

/*
 *  _Attributes_Is_priority
 *
 *  DESCRIPTION:
 *
 *  This function returns TRUE if the priority attribute is
 *  enabled in the attribute_set and FALSE otherwise.
 */

STATIC INLINE boolean _Attributes_Is_priority(
  Attributes_Control attribute_set
);

/*
 *  _Attributes_Is_limit
 *
 *  DESCRIPTION:
 *
 *  This function returns TRUE if the limited attribute is
 *  enabled in the attribute_set and FALSE otherwise.
 */

STATIC INLINE boolean _Attributes_Is_limit(
  Attributes_Control attribute_set
);

/*
 *  _Attributes_Is_binary_semaphore
 *
 *  DESCRIPTION:
 *
 *  This function returns TRUE if the binary semaphore attribute is
 *  enabled in the attribute_set and FALSE otherwise.
 */

STATIC INLINE boolean _Attributes_Is_binary_semaphore(
  Attributes_Control attribute_set
);

/*
 *  _Attributes_Is_inherit_priority
 *
 *  DESCRIPTION:
 *
 *  This function returns TRUE if the priority inheritance attribute
 *  is enabled in the attribute_set and FALSE otherwise.
 */

STATIC INLINE boolean _Attributes_Is_inherit_priority(
  Attributes_Control attribute_set
);

#include "attr.inl"

#endif
/* end of include file */
