/*  event.h
 *
 *  This include file contains the information pertaining to the Event
 *  Manager.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __EVENT_h
#define __EVENT_h

#include "object.h"
#include "options.h"
#include "thread.h"
#include "watchdog.h" 
#include "eventset.h"

/*
 *  _Event_Manager_initialization
 *
 *  This routine performs the initialization necessary for this manager.
 */

STATIC INLINE void _Event_Manager_initialization( void );

/*
 *  _Event_Send
 *
 *  This routine implements the event_send directive.  It sends
 *  event_in to the task specified by ID.  If the task is blocked
 *  waiting to receive events and the posting of event_in satisfies
 *  the task's event condition, then it is unblocked.
 */

Status_Codes _Event_Send ( 
  Objects_Id         id, 
  Event_sets_Control event_in
);

/*
 *  _Event_Receive
 *
 *  This routine implements the event_receive directive.  This
 *  directive is invoked when the calling task wishes to receive
 *  the event_in event condition.  One of the fields in the option_set
 *  parameter determines whether the receive request is satisfied if
 *  any or all of the events are pending.   If the event condition
 *  is not satisfied immediately, then the task may block with an
 *  optional timeout of TICKS clock ticks or return immediately.
 *  This determination is based on another field in the option_set
 *  parameter.  This directive returns the events received in the
 *  event_out parameter.
 */

Status_Codes _Event_Receive ( 
  Event_sets_Control  event_in, 
  Options_Control     option_set, 
  Watchdog_Interval   ticks, 
  Event_sets_Control *event_out
);

/*
 *  _Event_Seize
 *
 *  This routine determines if the event condition event_in is
 *  satisfied.  If so or if the no_wait option is enabled in option_set,
 *  then the procedure returns immediately.  If neither of these
 *  conditions is true, then the calling task is blocked with an
 *  optional timeout of ticks clock ticks.
 */

void _Event_Seize (
  Event_sets_Control event_in, 
  Options_Control    option_set, 
  Watchdog_Interval  ticks
);

/*
 *  _Event_Surrender
 *
 *  This routine determines if the event condition of the_thread
 *  has been satisfied.  If so, it unblocks the_thread.
 */

void _Event_Surrender ( 
  Thread_Control *the_thread 
);

/*
 *  _Event_Timeout
 *
 *  This routine is invoked when a task's event receive request
 *  has not been satisfied after the specified timeout interval.
 *  The task represented by ID will be unblocked and its status
 *  code will be set in it's control block to indicate that a timeout
 *  has occurred.
 */

void _Event_Timeout ( 
  Objects_Id  id,
  void       *ignored
);

/*
 *  The following defines the synchronization flag used by the
 *  Event Manager to insure that events sent to the currently
 *  executing thread are received properly.
 */

EXTERN boolean _Event_Sync;    /* event manager sync flag */

#include "event.inl"
#include "eventmp.h"

#endif
/* end of include file */
