/*  eventset.h
 *
 *  This include file contains the information pertaining to the
 *  Event Sets Handler.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __EVENT_SET_h
#define __EVENT_SET_h

/*
 *  The following defines the type used to control event sets.
 */

typedef unsigned32 Event_sets_Control;

/*
 *  The following constants define the individual events which may
 *  be used to compose an event set.
 */

#define EVENT_0     0x00000001
#define EVENT_1     0x00000002
#define EVENT_2     0x00000004
#define EVENT_3     0x00000008
#define EVENT_4     0x00000010
#define EVENT_5     0x00000020
#define EVENT_6     0x00000040
#define EVENT_7     0x00000080
#define EVENT_8     0x00000100
#define EVENT_9     0x00000200
#define EVENT_10    0x00000400
#define EVENT_11    0x00000800
#define EVENT_12    0x00001000
#define EVENT_13    0x00002000
#define EVENT_14    0x00004000
#define EVENT_15    0x00008000
#define EVENT_16    0x00010000
#define EVENT_17    0x00020000
#define EVENT_18    0x00040000
#define EVENT_19    0x00080000
#define EVENT_20    0x00100000
#define EVENT_21    0x00200000
#define EVENT_22    0x00400000
#define EVENT_23    0x00800000
#define EVENT_24    0x01000000
#define EVENT_25    0x02000000
#define EVENT_26    0x04000000
#define EVENT_27    0x08000000
#define EVENT_28    0x10000000
#define EVENT_29    0x20000000
#define EVENT_30    0x40000000
#define EVENT_31    0x80000000

#define ALL_EVENTS  0x80000000

/*
 *  The following constant is the value of an event set which
 *  has no events pending.
 */

#define EVENT_SETS_NONE_PENDING 0

/*
 *  _Event_sets_Is_empty
 *
 *  This function returns TRUE if no events are posted in the event_set,
 *  and FALSE otherwise.
 */

STATIC INLINE boolean _Event_sets_Is_empty(
  Event_sets_Control the_event_set
);

/*
 *  _Event_sets_Post
 *
 *  This routine posts the given new_events into the event_set
 *  passed in.  The result is returned to the user in event_set.
 */
          
STATIC INLINE void _Event_sets_Post(
  Event_sets_Control  the_new_events,
  Event_sets_Control *the_event_set
);

/*
 *  _Event_sets_Get
 *  
 *  This function returns the events in event_condition which are
 *  currently pending in event_set.
 */
          
STATIC INLINE Event_sets_Control _Event_sets_Get(
  Event_sets_Control the_event_set,
  Event_sets_Control the_event_condition
);

/*
 *  _Event_sets_Clear
 *
 *  This function removes the events in mask from the event_set
 *  passed in.  The result is returned to the user in event_set.
 */
          
STATIC INLINE Event_sets_Control _Event_sets_Clear(
 Event_sets_Control the_event_set,
 Event_sets_Control the_mask
);

#include "eventset.inl"

#endif
/* end of include file */
