/*  userext.inl
 *
 *  This file contains the macro implementation of the inlined routines
 *  from the User Extension Handler
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __USER_EXTENSIONS_inl
#define __USER_EXTENSIONS_inl

/*
 *  _User_extensions_Handler_initialization
 *
 */

STATIC INLINE void _User_extensions_Handler_initialization (
  Configuration_Extension_table *the_extension_table
)
{
    if ( the_extension_table == NULL )
      _User_extensions_Table = _User_extensions_Default_table;
    else
      _User_extensions_Table = *the_extension_table;
}

/*
 *  _User_extensions_Task_create
 *
 */

STATIC INLINE void _User_extensions_Task_create (
  Thread_Control *the_thread
)
{
  if ( _User_extensions_Table.task_create != NULL )
    (*_User_extensions_Table.task_create)( _Thread_Executing, the_thread );
}

/*
 *  _User_extensions_Task_delete
 *
 */

STATIC INLINE void _User_extensions_Task_delete (
  Thread_Control *the_thread
)
{
  if ( _User_extensions_Table.task_delete != NULL )
    (*_User_extensions_Table.task_delete)( _Thread_Executing, the_thread );
}

/*
 *  _User_extensions_Task_start
 *
 */

STATIC INLINE void _User_extensions_Task_start (
  Thread_Control *the_thread
)
{
  if ( _User_extensions_Table.task_start != NULL )
    (*_User_extensions_Table.task_start)( _Thread_Executing, the_thread );
}

/*
 *  _User_extensions_Task_restart
 *
 */

STATIC INLINE void _User_extensions_Task_restart (
  Thread_Control *the_thread
)
{
  if ( _User_extensions_Table.task_restart != NULL )
    (*_User_extensions_Table.task_restart)( _Thread_Executing, the_thread );
}

/*
 *  _User_extensions_Task_switch
 *
 */

STATIC INLINE void _User_extensions_Task_switch (
  Thread_Control *executing,
  Thread_Control *heir
)
{
  if ( _User_extensions_Table.task_switch != NULL )
    (*_User_extensions_Table.task_switch)( executing, heir );
}

/*
 *  _User_extensions_Task_exitted
 *
 */

STATIC INLINE void _User_extensions_Task_exitted (
  Thread_Control *executing
)
{
  if ( _User_extensions_Table.task_exitted != NULL )
    (*_User_extensions_Table.task_exitted)( executing );
}

/*
 *  _User_extensions_Fatal
 *
 */

STATIC INLINE void _User_extensions_Fatal (
  unsigned32 the_error
)
{
  if ( _User_extensions_Table.fatal != NULL )
    (*_User_extensions_Table.fatal)( the_error );
}

#endif
/* end of include file */
