/*  
 *  Multiprocessing Support for the Internal Threads
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#include "config.h"
#include "fatal.h"
#include "intthrd.h"
#include "mpci.h"
#include "mppkt.h"
#include "object.h"
#include "options.h"
#include "states.h"
#include "thread.h"

/*PAGE
 *
 *  _Internal_threads_MP_Send_process_packet
 *
 */

void _Internal_threads_MP_Send_process_packet (
  Internal_threads_MP_Remote_operations operation
)
{
  Internal_threads_MP_Packet *the_packet;
 
  switch ( operation ) {
 
    case INTERNAL_THREADS_MP_SYSTEM_VERIFY:
 
      the_packet                    = _Internal_threads_MP_Get_packet();
      the_packet->Prefix.class      = MP_PACKET_INTERNAL_THREADS;
      the_packet->Prefix.length     = sizeof ( Internal_threads_MP_Packet );
      the_packet->Prefix.to_convert = sizeof ( Internal_threads_MP_Packet );
      the_packet->operation         = operation;

      the_packet->maximum_nodes = _Configuration_MP_table->maximum_nodes;

      the_packet->maximum_global_objects =
        _Configuration_MP_table->maximum_global_objects;

      _MPCI_Send_process_packet( MPCI_ALL_NODES, &the_packet->Prefix );
      break;
  }
}

/*PAGE
 *
 *  _Internal_threads_MP_Send_request_packet
 *
 *  This subprogram is not needed since there are no request
 *  packets to be sent by this manager.
 *
 */

/*PAGE
 *
 *  _Internal_threads_MP_Send_response_packet
 *
 *  This subprogram is not needed since there are no response
 *  packets to be sent by this manager.
 *
 */

/*PAGE
 *
 *
 *  _Internal_threads_MP_Process_packet
 *
 */

void _Internal_threads_MP_Process_packet (
  MP_packet_Prefix  *the_packet_prefix
)
{
  Internal_threads_MP_Packet *the_packet;

  the_packet = (Internal_threads_MP_Packet *) the_packet_prefix;

  switch ( the_packet->operation ) {

    case INTERNAL_THREADS_MP_SYSTEM_VERIFY:
 
      if ( ( the_packet->maximum_nodes != 
                  _Configuration_MP_table->maximum_nodes ) ||
           ( the_packet->maximum_global_objects !=
                  _Configuration_MP_table->maximum_global_objects ) ) {

        _MPCI_Return_packet( the_packet_prefix );

        _Fatal_Error_occurred( STATUS_INVALID_NUMBER );

      }

      _MPCI_Return_packet( the_packet_prefix );

      break;
  } 
}

/*PAGE    
 *
 *  _Internal_threads_MP_Send_object_was_deleted
 *
 *  This subprogram is not needed since there are no objects
 *  deleted by this manager.
 *
 */

/*PAGE   
 *
 *  _Internal_threads_MP_Send_extract_proxy
 *
 *  This subprogram is not needed since there are no objects
 *  deleted by this manager.
 *
 */     
          
/*PAGE
 *
 *  _Internal_threads_MP_Get_packet
 *
 */

Internal_threads_MP_Packet *_Internal_threads_MP_Get_packet ( void )
{
  return ( (Internal_threads_MP_Packet *) _MPCI_Get_packet() );
}

/* end of file */
