/*  inthrdmp.h
 * 
 *  This include file contains all the constants and structures associated
 *  with the multiprocessing support in the Internal Thread Handler.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __INTERNAL_THREADS_MP_h
#define __INTERNAL_THREADS_MP_h

#include "intthrd.h"
#include "mppkt.h"
#include "object.h"
#include "thread.h"
#include "watchdog.h"

/*
 *  The following enumerated type defines the list of
 *  remote internal thread operations.
 */

typedef enum {
  INTERNAL_THREADS_MP_SYSTEM_VERIFY  =  0,
}   Internal_threads_MP_Remote_operations;

/*      
 *  The following data structure defines the packet used to perform
 *  remote internal thread operations.
 */

typedef struct {
  MP_packet_Prefix                      Prefix;
  Internal_threads_MP_Remote_operations operation;
  unsigned32                            maximum_nodes;
  unsigned32                            maximum_global_objects;
}   Internal_threads_MP_Packet;

/*
 *  _Internal_threads_MP_Send_process_packet
 *
 *  This routine performs a remote procedure call so that a
 *  process operation can be performed on another node.
 */

void _Internal_threads_MP_Send_process_packet (
  Internal_threads_MP_Remote_operations operation
);

/*
 *  _Internal_threads_MP_Send_request_packet
 *
 *  This routine performs a remote procedure call so that a
 *  directive operation can be initiated on another node.
 *
 *  This routine is not needed since there are no request
 *  packets to be sent by this manager.
 */

/*
 *  _Internal_threads_MP_Send_response_packet
 *
 *  This routine performs a remote procedure call so that a
 *  directive can be performed on another node.
 *
 *  This routine is not needed since there are no response
 *  packets to be sent by this manager.
 */

/*
 *
 *  _Internal_threads_MP_Process_packet
 *
 *  This routine performs the actions specific to this package for
 *  the request from another node.
 */

void _Internal_threads_MP_Process_packet (
  MP_packet_Prefix *the_packet_prefix
);

/*    
 *  _Internal_threads_MP_Send_object_was_deleted
 *
 *  This routine is invoked indirectly by the thread queue
 *  when a proxy has been removed from the thread queue and
 *  the remote node must be informed of this.
 *
 *  This routine is not needed since there are no objects
 *  deleted by this manager.
 */

/*   
 *  _Internal_threads_MP_Send_extract_proxy
 *
 *  This routine is invoked when a task is deleted and it
 *  has a proxy which must be removed from a thread queue and
 *  the remote node must be informed of this.
 *
 *  This routine is not needed since there are no objects
 *  deleted by this manager.
 */     
          
/*
 *  _Internal_threads_MP_Get_packet
 *
 *  This routine is used to obtain a internal threads mp packet.
 */

Internal_threads_MP_Packet *_Internal_threads_MP_Get_packet ( void );

#endif
/* end of file */
