/*
 *  Interrupt Manager
 *
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#include "isr.h"
#include "intr.h"

/*  _Interrupt_Catch
 *
 *  This directive allows a thread to specify what action to take when
 *  catching signals.
 *
 *  Input parameters:
 *    new_isr_handler - address of interrupt service routine (isr)
 *    vector          - interrupt vector number
 *    old_isr_handler - address at which to store previous ISR address
 *
 *  Output parameters:
 *    STATUS_SUCCESSFUL - always succeeds
 *    *old_isr_handler  - previous ISR address
 */

Status_Codes _Interrupt_Catch(
  proc_ptr    new_isr_handler,
  unsigned32  vector,
  proc_ptr   *old_isr_handler
)
{
  if ( !_ISR_Is_vector_number_valid( vector ) )
    return( STATUS_INVALID_NUMBER );

  if ( !_ISR_Is_valid_user_handler( new_isr_handler ) )
    return( STATUS_INVALID_ADDRESS );

  _ISR_Install_vector( vector, new_isr_handler, old_isr_handler );
  return( STATUS_SUCCESSFUL );
}
