/*  inthrd.h
 * 
 *  This include file contains all the constants and structures associated
 *  with the RTEMS internal threads.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __INTERNAL_THREADS_h
#define __INTERNAL_THREADS_h

#include "stack.h"
#include "thread.h"
#include "wkspace.h"

/*
 *  The following constants define the stack size requirements for
 *  the RTEMS system threads.
 */

#define INTERNAL_THREADS_SYSTEM_INITIALIZATION_THREAD_STACK_SIZE \
  ( 512 + CPU_SYSTEM_INITIALIZATION_THREAD_EXTRA_STACK )

#define INTERNAL_THREADS_IDLE_THREAD_STACK_SIZE  STACK_MINIMUM_SIZE 

/*
 *  The following define the thread control pointers used to access
 *  and manipulate the RTEMS system threads.
 */

EXTERN Thread_Control *_Internal_threads_System_initialization_thread;
EXTERN Thread_Control *_Internal_threads_Idle_thread;

/*
 *  The following contains the address of the multiprocessing
 *  receive server which is invoked by the system intialization
 *  thread in a multiprocessing system.
 */

EXTERN proc_ptr _Internal_threads_MP_Receive_server_entry;

/*
 *  _Internal_thread_Initialization
 *
 *  This routine initializes the system threads.
 */

void _Internal_threads_Initialization( void );

/*
 *  _Internal_threads_Set_MP_receive_server
 *
 *  This routine informs the system initialization thread
 *  of the entry point of the multiprocessing server.
 */

void _Internal_threads_Set_MP_receive_server( 
  proc_ptr mp_server_entry
);

/*
 *  _Internal_threads_Initialize_user_tasks
 *
 *  This routine creates and starts all of the application tasks
 *  specified in the Initialization Tasks Table.
 */

void _Internal_threads_Initialize_user_tasks( void );

/*
 *  _Internal_threads_System_initialization_thread_body
 *
 *  This routine is the body of the system initialization thread.
 */

Thread _Internal_threads_System_initialization_thread_body(
  unsigned32 ignored
);

/*
 *  _Internal_threads_Idle_thread_body
 *
 *  This routine is the body of the system idle thread.
 */

Thread _Internal_threads_Idle_thread_body(
  unsigned32 ignored
);

/*
 *  _Internal_threads_Initialize_thread
 *
 *  This routine initializes the specified system thread.
 */

void _Internal_threads_Initialize_thread(
  Thread_Control     *the_thread,
  Objects_Name        its_name,
  Priority_Control    its_priority,
  Attributes_Control  its_attributes,
  Modes_Control       its_mode,
  proc_ptr            entry_point,
  unsigned32          stack_size
);

#include "inthrdmp.h"

#endif
/* end of include file */
