/*  mpci.h
 * 
 *  This include file contains all the constants and structures associated
 *  with the MPCI layer.  It provides mechanisms to utilize packets.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __MPCI_h
#define __MPCI_h

#include "mppkt.h"
#include "states.h"
#include "status.h"
#include "thread.h"
#include "threadq.h"
#include "tqdata.h"
#include "watchdog.h"

/*
 *  The following defines the node number used when a broadcast is desired.
 */

#define MPCI_ALL_NODES 0

/*
 *  _MPCI_Handler_initialization
 *
 *  This routine performs the initialization necessary for this handler.
 */

void _MPCI_Handler_initialization ( void );

/*
 *  _MPCI_Initialization
 *
 *  This routine initializes the MPCI driver by
 *  invoking the user provided MPCI initialization callout.
 */

void _MPCI_Initialization ( void );

/*
 *  _MPCI_Get_packet
 *
 *  This function obtains a packet by invoking the user provided
 *  MPCI get packet callout.
 */

MP_packet_Prefix *_MPCI_Get_packet ( void );

/*
 *  _MPCI_Return_packet
 *
 *  This routine returns a packet by invoking the user provided
 *  MPCI return packet callout.
 */

void _MPCI_Return_packet (
  MP_packet_Prefix *the_packet
);

/*
 *  _MPCI_Send_process_packet
 *
 *  This routine sends a process packet by invoking the user provided
 *  MPCI send callout.
 */

void _MPCI_Send_process_packet (
  unsigned32        destination,
  MP_packet_Prefix *the_packet
);

/*
 *  _MPCI_Send_request_packet
 *
 *  This routine sends a request packet by invoking the user provided
 *  MPCI send callout.
 */

Status_Codes _MPCI_Send_request_packet (
  unsigned32         destination,
  MP_packet_Prefix  *the_packet,
  States_Control     extra_state,
  Watchdog_Interval  timeout
);

/*
 *  _MPCI_Send_response_packet
 *
 *  This routine sends a response packet by invoking the user provided
 *  MPCI send callout.
 */

void _MPCI_Send_response_packet (
  unsigned32        destination,
  MP_packet_Prefix *the_packet
);

/*
 *  _MPCI_Receive_packet
 *
 *  This routine receives a packet by invoking the user provided
 *  MPCI receive callout.
 */

MP_packet_Prefix  *_MPCI_Receive_packet ( void );

/*
 *  _MPCI_Process_response
 *
 *  This routine obtains a packet by invoking the user provided
 *  MPCI get packet callout.
 */

Thread_Control *_MPCI_Process_response (
  MP_packet_Prefix *the_packet
);

/*
 *  The following thread queue is used to maintain a list of tasks
 *  which currently have outstanding remote requests.
 */
 
EXTERN Thread_queue_Control _MPCI_Remote_blocked_threads;

#endif
/* end of include file */
