/*  mppkt.h
 * 
 *  This package is the specification for the Packet Handler.
 *  This handler defines the basic RTEMS packet and provides
 *  mechanisms to utilize packets based on this prefix.
 * 
 * 
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 * 
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __MP_PACKET_h
#define __MP_PACKET_h

#include "object.h"
#include "priority.h"

/*
 * The following enumerated type defines the packet classes
 *  supported by RTEMS.
 * 
 *  NOTE:  In general, each class corresponds to a manager
 *         which supports global operations.  Each manager
 *         defines the set of supported operations.
 */

typedef enum {
  MP_PACKET_INTERNAL_THREADS = 0,
  MP_PACKET_RTEMS_TASKS      = 1,
  MP_PACKET_MESSAGE_QUEUE    = 2,
  MP_PACKET_SEMAPHORE        = 3,
  MP_PACKET_PARTITION        = 4,
  MP_PACKET_REGION           = 5,
  MP_PACKET_EVENT            = 6,
  MP_PACKET_SIGNAL           = 7
}   MP_packet_Classes;

#define MP_PACKET_CLASSES_FIRST  MP_PACKET_INTERNAL_THREADS
#define MP_PACKET_CLASSES_LAST   MP_PACKET_SIGNAL

/*
 *  The following record contains the prefix for every packet
 *  passed between RTEMS nodes.
 *
 *  NOTE: This structure is padded to insure that anything
 *        following it is on a 16 byte boundary.  This is
 *        the most stringent structure alignment rule
 *        the RTEMS project has encountered yet (i960CA).
 */

typedef struct {
  MP_packet_Classes class;
  Objects_Id        id;
  Objects_Id        source_tid;
  Priority_Control  source_priority;
  Status_Codes      return_code;
  unsigned32        length;
  unsigned32        to_convert;
  unsigned32        pad0;
}   MP_packet_Prefix;

/*
 *  _Mp_packet_Is_valid_packet_class  
 *
 *  This function returns TRUE if the the_packet_class is valid,
 *  and FALSE otherwise.
 */

STATIC INLINE boolean _Mp_packet_Is_valid_packet_class (
  MP_packet_Classes the_packet_class
);

#include "mppkt.inl"

#endif
/* end of include file */
