/*  msgmp.h
 * 
 *  This include file contains all the constants and structures associated
 *  with the Multiprocessing Support in the Message Manager.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __MESSAGE_QUEUE_MP_h
#define __MESSAGE_QUEUE_MP_h

#include "message.h"
#include "mppkt.h"
#include "object.h"
#include "options.h"
#include "thread.h"
#include "watchdog.h"

/*
 *  The following enumerated type defines the list of
 *  remote message queue operations.
 */

typedef enum {
  MESSAGE_QUEUE_MP_ANNOUNCE_CREATE     =  0,
  MESSAGE_QUEUE_MP_ANNOUNCE_DELETE     =  1,
  MESSAGE_QUEUE_MP_EXTRACT_PROXY       =  2,
  MESSAGE_QUEUE_MP_RECEIVE_REQUEST     =  3,
  MESSAGE_QUEUE_MP_RECEIVE_RESPONSE    =  4,
  MESSAGE_QUEUE_MP_SEND_REQUEST        =  5,
  MESSAGE_QUEUE_MP_SEND_RESPONSE       =  6,
  MESSAGE_QUEUE_MP_URGENT_REQUEST      =  7,
  MESSAGE_QUEUE_MP_URGENT_RESPONSE     =  8,
  MESSAGE_QUEUE_MP_BROADCAST_REQUEST   =  9,
  MESSAGE_QUEUE_MP_BROADCAST_RESPONSE  = 10,
  MESSAGE_QUEUE_MP_FLUSH_REQUEST       = 11,
  MESSAGE_QUEUE_MP_FLUSH_RESPONSE      = 12
}   Message_queue_MP_Remote_operations;

/*      
 *  The following data structure defines the packet used to perform
 *  remote message queue operations.
 */

typedef struct {
  MP_packet_Prefix                   Prefix;
  Message_queue_MP_Remote_operations operation;
  Objects_Name                       name;
  Options_Control                    option_set;
  Watchdog_Interval                  timeout;
  Objects_Id                         proxy_id;
  unsigned32                         count;
  unsigned32                         pad0;
  unsigned32                         pad1;
  Message_queue_Buffer               Buffer;
}   Message_queue_MP_Packet;

/*
 *  _Message_queue_MP_Send_process_packet
 *
 *  This routine performs a remote procedure call so that a
 *  process operation can be performed on another node.
 */

void _Message_queue_MP_Send_process_packet (
  Message_queue_MP_Remote_operations operation,
  Objects_Id                         message_queue_id,
  Objects_Name                       name,
  Objects_Id                         proxy_id
); 

/*
 *  _Message_queue_MP_Send_request_packet
 *
 *  This routine performs a remote procedure call so that a
 *  directive operation can be initiated on another node.
 */

Status_Codes _Message_queue_MP_Send_request_packet (
  Message_queue_MP_Remote_operations  operation,
  Objects_Id                          message_queue_id,
  Message_queue_Buffer               *buffer,
  Options_Control                     option_set,
  Watchdog_Interval                   timeout
); 

/*
 *  _Message_queue_MP_Send_response_packet
 *
 *  This routine performs a remote procedure call so that a
 *  directive can be performed on another node.
 */

void _Message_queue_MP_Send_response_packet (
  Message_queue_MP_Remote_operations  operation,
  Objects_Id                          message_queue_id,
  Thread_Control                     *the_thread
); 

/*
 *
 *  _Message_queue_MP_Process_packet
 *
 *  This routine performs the actions specific to this package for
 *  the request from another node.
 */

void _Message_queue_MP_Process_packet (
  MP_packet_Prefix *the_packet_prefix
);

/*    
 *  _Message_queue_MP_Send_object_was_deleted
 *
 *  This routine is invoked indirectly by the thread queue
 *  when a proxy has been removed from the thread queue and
 *  the remote node must be informed of this.
 */

void _Message_queue_MP_Send_object_was_deleted (
  Thread_Control *the_proxy
);

/*   
 *  _Message_queue_MP_Send_extract_proxy
 *
 *  This routine is invoked when a task is deleted and it
 *  has a proxy which must be removed from a thread queue and
 *  the remote node must be informed of this.
 */     
          
void _Message_queue_MP_Send_extract_proxy (
  Thread_Control *the_thread
);

/*
 *  _Message_queue_MP_Get_packet
 *
 *  This function is used to obtain a message queue mp packet.
 */

Message_queue_MP_Packet *_Message_queue_MP_Get_packet ( void );

#endif
/* end of file */
