/*  objectmp.h
 * 
 *  This include file contains all the constants and structures associated
 *  with the manipulation of Global RTEMS Objects.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __OBJECTS_MP_h
#define __OBJECTS_MP_h

/*
 *  This defines the Global Object Control Block used to manage
 *  objects resident on other nodes.
 */

typedef struct {
  Objects_Control Object;
  Objects_Name    name;
}   Objects_MP_Control;

/*
 *  _Objects_MP_Handler_initialization
 *
 *  This routine intializes the inactive global object chain
 *  based on the maximum number of global objects configured.
 */

void _Objects_MP_Handler_initialization (
  unsigned32 maximum_global_objects
);

/*
 *  _Objects_MP_Allocate_global_object
 *
 *  This function allocates a Global Object control block.
 */

STATIC INLINE Objects_MP_Control *_Objects_MP_Allocate_global_object (
  void 
);

/*
 *  _Objects_MP_Free_global_object
 *
 *  This routine deallocates a Global Object control block.
 */

STATIC INLINE void _Objects_MP_Free_global_object (
  Objects_MP_Control *the_object  
);

/*
 *  _Objects_MP_Is_null_global_object
 *
 *  This function returns whether the global object is NULL or not.
 */

STATIC INLINE boolean _Objects_MP_Is_null_global_object (
  Objects_MP_Control *the_object  
);

/*
 *  _Objects_MP_Open
 *
 *  This routine allocates a global object control block
 *  and places it in the specified information table.  If the
 *  allocation fails, then is_fatal_error determines the
 *  error processing actions taken.
 */

boolean _Objects_MP_Open (
  Objects_Information *information,
  Objects_Name         the_name,
  Objects_Id           the_id,
  boolean              is_fatal_error
);

/* 
 *  _Objects_MP_Close
 *
 *  This routine removes a global object from the specified
 *  information table and deallocates the global object control block.
 */

void _Objects_MP_Close (
  Objects_Information *information,
  Objects_Id           the_id
);

/*
 *  _Objects_MP_Global_name_search
 *
 *  This routine looks for the object with the_name in the global
 *  object tables indicated by information.  It returns the ID of the
 *  object with that name if one is found.
 */

Status_Codes _Objects_MP_Global_name_search (
  Objects_Information *information,
  Objects_Name         the_name,
  unsigned32           nodes_to_search,
  Objects_Id          *the_id
);

/*
 *  _Objects_MP_Is_remote
 *
 *  This function searches the Global Object Table managed
 *  by information for the object indicated by ID.  If the object
 *  is found, then location is set to objects_remote, otherwise
 *  location is set to objects_error.  In both cases, the_object
 *  is undefined.
 */

void _Objects_MP_Is_remote (
  Objects_Information  *information,
  Objects_Id            the_id,
  Objects_Locations    *location, 
  Objects_Control     **the_object
);

/*
 *  The following chain header is used to manage the set of
 *  inactive global object control blocks.
 */
 
EXTERN Chain_Control  _Objects_MP_Inactive_global_objects;

#include "objectmp.inl"

#endif
/* end of include file */
