/*  stack.h
 *
 *  This include file contains all information about the thread
 *  Stack Handler.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __STACK_h
#define __STACK_h

/*
 *  The following constant defines the minimum stack size which every
 *  thread must exceed.
 */

#define STACK_MINIMUM_SIZE  CPU_STACK_MINIMUM_SIZE

/*
 *  The following defines the control block used to manage each stack.
 */

typedef struct {
  unsigned32  size;        /* stack size */
  void       *area;        /* low memory addr of stack */
}   Stack_Control;

/*
 *  _Stack_Adjust_size
 *
 *  This function increases the stack size to insure that the thread
 *  has the desired amount of stack space after the initial stack
 *  pointer is determined based on alignment restrictions.
 *
 *  NOTE:
 *
 *  The implementation of this routine is CPU dependent.
 */

#define _Stack_Adjust_size( _size, _new_size ) \
  _CPU_Stack_Adjust_size( (_size), (_new_size) )

/*
 *  _Stack_Initialize
 *
 *  This routine initializes the_stack record to indicate that
 *  size bytes of memory starting at starting_address have been
 *  reserved for a stack.
 */

STATIC INLINE void _Stack_Initialize (
  Stack_Control    *the_stack,
  void             *starting_address,
  unsigned32        size
);

/*
 *  _Stack_Get_initial_pointer
 *
 *  This function returns the thread's initial stack pointer based on the
 *  information in the_stack.
 *
 *  NOTE:
 *
 *  This routine must invoke a processor dependent function because
 *  stacks grow from low to high addresses on some processors, and
 *  vice-versa on others.
 */

#define _Stack_Get_initial_pointer( _the_stack ) \
  _CPU_Stack_Get_initial_pointer( (_the_stack)->area, (_the_stack)->size )

/*
 *  _Stack_Is_enough
 *
 *  This function returns TRUE if size bytes is enough memory for
 *  a valid stack area on this processor, and FALSE otherwise.
 */

STATIC INLINE boolean _Stack_Is_enough (
  unsigned32 size
);

#include "stack.inl"

#endif
/* end of include file */
