/*  status.h
 *
 *  This include file contains the status codes returned from the 
 *  executive directives.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __STATUS_h
#define __STATUS_h

/* types */

/* enumerated constants */

typedef enum {
  STATUS_SUCCESSFUL               =  0, /* successful completion */
  STATUS_TASK_EXITTED             =  1, /* returned from a thread */
  STATUS_MP_NOT_CONFIGURED        =  2, /* multiprocessing not configured */
  STATUS_INVALID_NAME             =  3, /* invalid object name */
  STATUS_INVALID_ID               =  4, /* invalid object id */
  STATUS_TOO_MANY                 =  5, /* too many */
  STATUS_TIMEOUT                  =  6, /* timed out waiting */
  STATUS_OBJECT_WAS_DELETED       =  7, /* object deleted while waiting */
  STATUS_INVALID_SIZE             =  8, /* specified size was invalid */
  STATUS_INVALID_ADDRESS          =  9, /* address specified is invalid */
  STATUS_INVALID_NUMBER           = 10, /* number was invalid */
  STATUS_NOT_DEFINED              = 11, /* item has not been initialized */
  STATUS_RESOURCE_IN_USE          = 12, /* resources still outstanding */
  STATUS_UNSATISFIED              = 13, /* request not satisfied */
  STATUS_INCORRECT_STATE          = 14, /* thread is in wrong state */
  STATUS_ALREADY_SUSPENDED        = 15, /* thread already in state */
  STATUS_ILLEGAL_ON_SELF          = 16, /* illegal on calling thread */
  STATUS_ILLEGAL_ON_REMOTE_OBJECT = 17, /* illegal for remote object */
  STATUS_CALLED_FROM_ISR          = 18, /* called from wrong environment */
  STATUS_INVALID_PRIORITY         = 19, /* invalid thread priority */
  STATUS_INVALID_CLOCK            = 20, /* invalid date/time */
  STATUS_INVALID_NODE             = 21, /* invalid node id */
  STATUS_NOT_CONFIGURED           = 22, /* directive not configured */
  STATUS_NOT_OWNER_OF_RESOURCE    = 23, /* not owner of resource */
  STATUS_NOT_IMPLEMENTED          = 24, /* directive not implemented */
  STATUS_PROXY_BLOCKING           = 25  /* internal multiprocessing only */
} Status_Codes;

/*
 *  _Status_Is_successful
 *
 *  This function returns TRUE if the status code is equal to SUCCESSFUL,
 *  and FALSE otherwise.
 */

STATIC INLINE boolean _Status_Is_successful (
  Status_Codes code 
);

/*
 *  _Status_Is_equal
 *
 *  This function returns TRUE if the status code1 is equal to code2,
 *  and FALSE otherwise.
 */

STATIC INLINE boolean _Status_Is_equal ( 
  Status_Codes code1,
  Status_Codes code2
);

/*
 *  _Status_Is_proxy_blocking
 *
 *  This function returns TRUE if the status code is equal to the
 *  status which indicates that a proxy is blocking, and FALSE otherwise.
 */
 
STATIC INLINE boolean _Status_Is_proxy_blocking (
  Status_Codes code
);

#include "status.inl"

#endif
/* end of include file */
