/*  system.h
 *
 *  This include file contains information that is included in every
 *  function in the executive.  This must be the first include file
 *  included in all internal RTEMS files.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __SYSTEM_h
#define __SYSTEM_h

/*
 *  The following insures that all data is declared in the space
 *  of the Initialization Manager.  It is referenced as "external"
 *  in every other file.
 */

#ifdef INIT
#undef EXTERN
#define EXTERN
#else
#undef EXTERN
#define EXTERN  extern
#endif

/*
 *  The following (in conjunction with compiler arguments) are used
 *  to choose between the use of static inline functions and macro
 *  functions.   The static inline implementation allows better
 *  type checking with no cost in code size or execution speed.
 */

#ifdef USE_INLINES
#define STATIC static
#define INLINE __inline__
#else
/*
#error Only the GNU C compiler is currently supported!!!
*/
#define STATIC 
#define INLINE 
#endif

/*
 *  Include a base set of files.
 */

#include "types.h"           /* RTEMS data types */
#include "cpu.h"             /* processor specific information */
#include "status.h"          /* RTEMS status codes */

/*
 *  Define some common constants
 */

#define NULL      0          /* NULL value */
#define TRUE      1          /* boolean true value */
#define FALSE     0          /* boolean false value */
#define FOREVER   1          /* infinite loop */
#define RESERVED  0          /* reserved for RTEMS use */

/*
 *  The following is the extern for the RTEMS version string.
 *  The contents of this string are CPU specific.
 */

extern const char _RTEMS_version[];         /* RTEMS version string */

/*
 *  The following defines the CPU dependent information table.
 */
EXTERN CPU_Table *_CPU_Table;               /* CPU dependent info */

#endif
/* end of include file */
