/*  taskmp.h
 * 
 *  This include file contains all the constants and structures associated
 *  with the multiprocessing support in the task manager.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __RTEMS_TASKS_MP_h
#define __RTEMS_TASKS_MP_h

#include "mppkt.h"
#include "object.h"
#include "options.h"
#include "priority.h"
#include "tasks.h"
#include "thread.h"

/*
 *  The following enumerated type defines the list of
 *  remote task operations.
 */

typedef enum {
  RTEMS_TASKS_MP_ANNOUNCE_CREATE       =  0,
  RTEMS_TASKS_MP_ANNOUNCE_DELETE       =  1,
  RTEMS_TASKS_MP_SUSPEND_REQUEST       =  2,
  RTEMS_TASKS_MP_SUSPEND_RESPONSE      =  3,
  RTEMS_TASKS_MP_RESUME_REQUEST        =  4,
  RTEMS_TASKS_MP_RESUME_RESPONSE       =  5,
  RTEMS_TASKS_MP_SET_PRIORITY_REQUEST  =  6,
  RTEMS_TASKS_MP_SET_PRIORITY_RESPONSE =  7,
  RTEMS_TASKS_MP_GET_NOTE_REQUEST      =  8,
  RTEMS_TASKS_MP_GET_NOTE_RESPONSE     =  9,
  RTEMS_TASKS_MP_SET_NOTE_REQUEST      = 10,
  RTEMS_TASKS_MP_SET_NOTE_RESPONSE     = 11,
}   RTEMS_tasks_MP_Remote_operations;

/*      
 *  The following data structure defines the packet used to perform
 *  remote task operations.
 */

typedef struct {
  MP_packet_Prefix                 Prefix;
  RTEMS_tasks_MP_Remote_operations operation;
  Objects_Name                     name;
  Priority_Control                 the_priority;
  unsigned32                       notepad;
  unsigned32                       note;
}   RTEMS_tasks_MP_Packet;

/*
 *  _RTEMS_tasks_MP_Send_process_packet
 *
 *  This routine performs a remote procedure call so that a
 *  process operation can be performed on another node.
 */

void _RTEMS_tasks_MP_Send_process_packet (
  RTEMS_tasks_MP_Remote_operations operation,
  Objects_Id                       task_id,
  Objects_Name                     name
); 

/*
 *  _RTEMS_tasks_MP_Send_request_packet
 *
 *  This routine performs a remote procedure call so that a
 *  directive operation can be initiated on another node.
 */

Status_Codes _RTEMS_tasks_MP_Send_request_packet (
  RTEMS_tasks_MP_Remote_operations operation,
  Objects_Id                       task_id,
  Priority_Control                 the_priority,
  unsigned32                       notepad,
  unsigned32                       note
); 

/*
 *  _RTEMS_tasks_MP_Send_response_packet
 *
 *  This routine performs a remote procedure call so that a
 *  directive can be performed on another node.
 */

void _RTEMS_tasks_MP_Send_response_packet (
  RTEMS_tasks_MP_Remote_operations  operation,
  Thread_Control                   *the_thread
); 

/*
 *
 *  _RTEMS_tasks_MP_Process_packet
 *
 *  This routine performs the actions specific to this package for
 *  the request from another node.
 */

void _RTEMS_tasks_MP_Process_packet (
  MP_packet_Prefix *the_packet_prefix
);

/*    
 *  _RTEMS_tasks_MP_Send_object_was_deleted
 *
 *  This routine is invoked indirectly by the thread queue
 *  when a proxy has been removed from the thread queue and
 *  the remote node must be informed of this.
 *
 *  This routine is not needed by RTEMS_tasks since a task
 *  cannot be deleted when segments are in use.
 */

/*   
 *  _Region_MP_Send_extract_proxy
 *
 *  This routine is invoked when a task is deleted and it
 *  has a proxy which must be removed from a thread queue and
 *  the remote node must be informed of this.
 *
 *  This routine is not needed since there are no objects
 *  deleted by this manager.
 *
 */     
          
/*
 *  _RTEMS_tasks_MP_Get_packet
 *
 *  This function is used to obtain a task mp packet.
 */

RTEMS_tasks_MP_Packet *_RTEMS_tasks_MP_Get_packet ( void );

#endif
/* end of file */
