/*  userext.h
 *
 *  This include file contains all information about user extensions.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __USER_EXTENSIONS_h
#define __USER_EXTENSIONS_h

#include "config.h"
#include "thread.h"

/*
 *  The following defines the default Device User Extension Table.
 *  This table is used when no user extensions are configured.
 */

#ifdef INIT

const 
  Configuration_Extension_table _User_extensions_Default_table = {
  NULL,    /* task create user extension address */
  NULL,    /* task start user extension address */
  NULL,    /* task restart user extension address */
  NULL,    /* task delete user extension address */
  NULL,    /* task switch user extension address */
  NULL,    /* thread exitted user extension address */
  NULL     /* fatal user extension address */
};

#else

extern Configuration_Extension_table _User_extensions_Default_table;

#endif

/*
 *  The following defines the internal RTEMS copy of the
 *  Extension Table.
 */

EXTERN Configuration_Extension_table _User_extensions_Table;

/*
 *  _User_extensions_Handler_initialization
 *
 *  This routine performs the initialization necessary for this handler.
 */

STATIC INLINE void _User_extensions_Handler_initialization (
  Configuration_Extension_table *the_extension_table
);

/*
 *  _User_extensions_Task_create
 *
 *  This routine is used to invoke the user extension for
 *  the task_create directive.
 */

STATIC INLINE void _User_extensions_Task_create (
  Thread_Control *the_thread
);

/*
 *  _User_extensions_Task_delete
 *
 *  This routine is used to invoke the user extension for
 *  the task_delete directive.
 */

STATIC INLINE void _User_extensions_Task_delete (
  Thread_Control *the_thread
);

/*
 *  _User_extensions_Task_start
 *
 *  This routine is used to invoke the user extension for
 *  the task_start directive.
 */

STATIC INLINE void _User_extensions_Task_start (
  Thread_Control *the_thread
);

/*
 *  _User_extensions_Task_restart
 *
 *  This routine is used to invoke the user extension for
 *  the task_restart directive.
 */

STATIC INLINE void _User_extensions_Task_restart (
  Thread_Control *the_thread
);

/*
 *  _User_extensions_Task_switch
 *
 *  This routine is used to invoke the user extension which
 *  is invoked when a context switch occurs.
 */

STATIC INLINE void _User_extensions_Task_switch (
  Thread_Control *executing,
  Thread_Control *heir
);

/*
 *  _User_extensions_Task_exitted
 *
 *  This routine is used to invoke the user extension which
 *  is invoked when a task exits.

 */

STATIC INLINE void _User_extensions_Task_exitted (
  Thread_Control *executing
);

/*
 *  _User_extensions_Fatal
 *
 *  This routine is used to invoke the user extension for
 *  the fatal_error_occurred directive.
 */

STATIC INLINE void _User_extensions_Fatal (
  unsigned32 the_error
);

#include "userext.inl"

#endif
/* end of include file */
