/*
 * <memory.h> : memory handling routines for gcc960
 */

#ifndef __MEMORY_H__
#define  __MEMORY_H__

/*
 * bring in size_t.
 */
#include <stddef.h>

#undef memchr
#undef memcmp
#undef memcpy
#undef memmove
#undef memset

#if 0
#define memchr(p,q,n)		__builtin_memchr(p,q,n)
#define memcmp(p,q,n)		__builtin_memcmp(p,q,n)
#define memcpy(p,q,n)		__builtin_memcpy(p,q,n)
#define memmove(p,q,n)		__builtin_memmove(p,q,n)
#define memset(p,q,n)		__builtin_memset(p,q,n)
#endif

#if defined(__STDC__)
/*
 * Prototypes.
 */
void *		(memccpy)(void *, const void *, int, int);
void *		(memchr)(const void *, int, size_t);
int		(memcmp)(const void *, const void *, size_t);
void *		(memcpy)(void *, const void *, size_t);
void *		(memmove)(void *, const void *, size_t);
void *		(memset)(void *, int, size_t);

char *          (bcopy)(const char *, char *, size_t );
void            (bzero)(char *, size_t );

#else
/*
 * not STDC, just declare types.
 */
void *		(memccpy)();
void *		(memchr)();
int		(memcmp)();
void *		(memcpy)();
void *		(memmove)();
void *		(memset)();
#endif  /* __STDC__ */

static __inline__ void*
__builtin_memcpy_align_1 (p, q, n)
char* p;
char* q;
size_t n;
{
  char* ret = p;

  while (n)
  { register char reg;
    reg = *((char*)q);
    *((char *)p) = reg;
    p += 1;
    q += 1;
    n--;
  }

  return ret;
}

typedef struct { int a; int b; int d; int e; } __mem_block_16;

static __inline__ void*
__builtin_memcpy_size_16_align_16 (p, q, n)
__mem_block_16* p;
__mem_block_16* q;
size_t n;
{
  __mem_block_16* ret = p;
  int n1 = n;

  while (n1 >= 16)
  { __mem_block_16 reg;
    reg = *q;
    *p = reg;
    p++;
    q++;
    n1-=16;
  }

  return (ret);
}

static __inline__ int
__builtin_memcmp_align_1 (s1, s2, n)
unsigned char* s1;
unsigned char* s2;
size_t n;
{
  unsigned char* end = s1 + n;

  while (s1 < end)
  { if (*s1 != *s2)
      return *s1 - *s2;

    s1++;
    s2++;
  }

  return 0;
}

#endif	/* __MEMORY_H__ */

