/*
 * <stdlib.h> for gcc960.
 */

#ifndef __STDLIBH__
#define __STDLIBH__

/*
 * bring in size_t.
 */
#include <stddef.h>


/*
 * div_t, ldiv_t : "quotient and remainder in any order"
 *
 */

typedef struct {
  int rem;
  int quot; } div_t;

typedef struct {
  long rem;
  long quot; } ldiv_t;

/*
 * Be sure ERANGE is defined.
 */
#include <errno.h>

/*
 * other macros required by ANSI.
 */
#define EXIT_FAILURE 1
#define EXIT_SUCCESS 0
#define RAND_MAX     0x7ffffffe	/* 2**31-2 */

/*
 * Prototypes (if not -traditional, e.g. __STDC__ is defined).
 */
#ifdef __STDC__
#ifndef __INSTANTIATE_STDLIB__
#define STDLIB_INLINE __inline__
#define STDLIB_STATIC static
#else
#define STDLIB_INLINE
#define STDLIB_STATIC
#endif
void		abort(void);
int 		atexit(void (*)(void));
double		atof(const char *);
int		atoi(const char *);
long		atol(const char *);
void *		bsearch(const void *, const void *, size_t, size_t,
			int(*)(const void *, const void *));
void *		calloc(size_t, size_t);
void		exit(int);
void		free(void *);
char *		getenv(const char *);

void *		malloc(size_t);
void		qsort(void *,size_t,size_t,int(*)(const void *, const void *));
int		rand(void);
void *		realloc(void *, size_t);
void		srand(unsigned int);
double		strtod(const char *, char **);
long		strtol(const char *, char **, int);
unsigned long  	strtoul(const char *, char **, int);
int		system(const char *);

/* STDLIB_STATIC STDLIB_INLINE const int   abs(int i)      {  
return i<0 ? -i : i;
e */
/*
 * i960 inline abs with no branching, thanks to Konrad Lai
 *				;        x >= 0	;   x < 0
 *	shri	31,x,temp	; temp = 0      ;   -1
 *	xor	x,temp,y	; y    = x      ;   not(x)
 *	subi	temp,y,y	; y    = x	;   not(x)+1 or -x
 *				; the subi also works in the overflow case
 */

STDLIB_STATIC STDLIB_INLINE const int abs(int i) {
	register int t,r,y;

	t = i>>31; 
        y = t^i; 
        r = y-t;
	return r;
}

STDLIB_STATIC STDLIB_INLINE const long labs(long int x) {  
return x<0 ? -x : x;
}

#if 0
STDLIB_STATIC const div_t div (int n, int d){
  div_t val; val.quot = n / d; val.rem = n % d; return val;
}

STDLIB_STATIC const ldiv_t ldiv(long n, long d)
{
  ldiv_t val; val.quot = n / d; val.rem = n % d; return val;
}
#endif

#undef STDLIB_STATIC
#undef STDLIB_INLINE
#endif /* __STDC__ */

#endif /* __STDLIBH__ */



