/*  ttydrv.h
 *  
 *  This file describes the TTY Driver for all boards.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __TTY_h
#define __TTY_h

/* constants */

#define TTY_SUCCESSFUL SUCCESSFUL

/* tty read and write line length defintions */

#define MAX_R_LL         256            /* maximum read line length */
#define MAX_W_LL         256            /* maximum write line length */

/* tty control constants */

/* tty baud rates */

#define B0      (char)  0               /* hang up */
#define B300    (char)  7               /* 300 baud */
#define B1200   (char)  9               /* 1200 baud */
#define B9600   (char) 13               /* 9600 baud */

/* tty control flags */

#define ECHO       0x00000001           /* echo enable */

/* tty ioctl commands */

#define TCGETA      1                   /* get tty control info */
#define TCSETA      2                   /* set tty control info */
#define TCCHRDY     3                   /* character available? */

/* structures */

/* tty parameter block structures */

/* read parameter block */

struct tty_r_parms {
  char *r_buf;                   /* ptr to read string */
  int  r_num;                    /* max number to read */
                                 /* zero = read until CR */
};

/* write parameter block */

struct tty_w_parms {
  char *w_buf;                   /* ptr to write string */
  rtems_unsigned8 w_nl;                /* write a new line after output */
};

/* ioctl parameter block */

struct tty_c_parms {
  char request;                  /* ioctl request */
  char ch;                       /* used for return char for CHRDY */
  struct tty_ctl *Control;       /* pointer to tty control info */
};

/* tty control information */

struct tty_ctl {
  char ispeed;                   /* input baud rate */
  char ospeed;                   /* output baud rate */
  int flags;                     /* misc tty features */
};

/* variables */

extern rtems_unsigned32 _TTY_major;    /* TTY major number -- user provided */

/* functions */

/* driver entries */

rtems_device_driver Tty_initialize(
  rtems_device_major_number,
  rtems_device_minor_number, 
  void *, 
  rtems_id, 
  rtems_unsigned32 * 
);

rtems_device_driver Tty_close( 
  rtems_device_major_number, 
  rtems_device_minor_number, 
  void *, 
  rtems_id, 
  rtems_unsigned32 * 
);

rtems_device_driver Tty_read(  
  rtems_device_major_number, 
  rtems_device_minor_number, 
  void *, 
  rtems_id, 
  rtems_unsigned32 * 
);

rtems_device_driver Tty_write( 
  rtems_device_major_number, 
  rtems_device_minor_number, 
  void *, 
  rtems_id, 
  rtems_unsigned32 *
);

rtems_device_driver Tty_control( 
  rtems_device_major_number, 
  rtems_device_minor_number, 
  void *, 
  rtems_id, 
  rtems_unsigned32 * 
);

#endif
/* end of include file */


