/*  force386.h
 * 
 *  This include file definitions related to the Force CPU-386 board.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __FORCE386_h
#define __FORCE386_h

#include "rtems.h"
#include "cpu.h"
#include "iosupp.h"

/* Constants */

#define FOREVER       1                    /* infinite loop */

/* I/O addressing */

/* 
 *  The following determines whether Port B or the Console should
 *  be used for test I/O.  Setting ONE (and only ONE) of these to 1
 *  enables I/O on that port.
 *
 *  PORT A - DUSCC MC68562 Channel A
 *  PORT B - DUSCC MC68562 Channel B
 *  PORT C - MFP MC68901 Channel      (*** FORCEbug console ***)
 */

#define PORTB         1               /* use port b as test port */
#define PORTC         0               /* use console port as test port */

#if ( PORTB == 1 )
#define TX_STATUS     0x1b6           /* DUSCC General Status Register */
#define RX_STATUS     0x1b6           /* DUSCC General Status Register */
#define TX_BUFFER     0x1e0           /* DUSCC Transmitter Channel B */
#define RX_BUFFER     0x1e8           /* DUSCC Receiver Channel B */
#define Is_tx_ready( _status ) ( (_status) & 0x20 )
#define Is_rx_ready( _status ) ( (_status) & 0x10 )
#endif

#if ( PORTC == 1 )
#define TX_STATUS     0x12c           /* MFP Transmit Status Register */
#define RX_STATUS     0x12a           /* MFP Receive Status Register */
#define TX_BUFFER     0x12e           /* MFP Transmitter Channel  */
#define RX_BUFFER     0x12e           /* MFP Receiver Channel  */
#define Is_tx_ready( _status ) ( (_status) & 0x80 )
#define Is_rx_ready( _status ) ( (_status) & 0x80 )
#endif

/* Timer constants */

#define IERA   0x106     /* Interrupt Enable Register A */
#define IMRA   0x112     /* Interrupt Mask Register A */
#define TACR   0x118     /* Timer A Control Register */
#define TADR   0x11e     /* Timer A Data Register */

#define IERB   0x108     /* Interrupt Enable Register B */
#define TBCR   0x11a     /* Timer B Control Register */
#define TBDR   0x120     /* Timer B Data Register */

/* Structures */

#ifdef F386_INIT
#undef EXTERN
#define EXTERN
#else
#undef EXTERN
#define EXTERN extern
#endif

/* functions */

int _F386_chrdy();
int _F386_inch();
int _F386_instr();
int _F386_outch();
int _F386_outstr();

/* end of include file */
#endif
