/*  fptest.h
 * 
 *  This include file contains the CPU dependent implementation
 *  of the following routines needed to test RTEMS floating
 *  point support:
 *           FP_load( &context )
 *           FP_check( &context )
 *
 *  FP_load   - loads the specified floating point context
 *  FP_check  - checks the specified floating point context
 *
 *  NOTE:  These routines are VERY CPU dependent and are thus
 *         located in in the CPU dependent include file 
 *         fptest.h.  These routines form the core of the 
 *         floating point context switch test.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "memory.h"

#define LOAD_FP_CONTEXT( _source ) \
  { \
    register void *__base = (_source); \
    \
    asm volatile( "flds   (%0) ; "  /* [0] */ \
                  "flds  4(%0) ; "  /* [1] */ \
                  "flds  8(%0) ; "  /* [2] */ \
                  "flds 12(%0) ; "  /* [3] */ \
                  "flds 16(%0) ; "  /* [4] */ \
                  "flds 20(%0) ; "  /* [5] */ \
                  "flds 24(%0) ; "  /* [6] */ \
                  "flds 28(%0) ; "  /* [7] */ \
       : "=r" (__base) : "0" (__base) ); \
  }

#define STORE_FP_CONTEXT( _dest ) \
  { \
    register void *__base = (_dest); \
    \
    asm volatile( "fstps 28(%0) ; "  /* [7] */ \
                  "fstps 24(%0) ; "  /* [6] */ \
                  "fstps 20(%0) ; "  /* [5] */ \
                  "fstps 16(%0) ; "  /* [4] */ \
                  "fstps 12(%0) ; "  /* [3] */ \
                  "fstps  8(%0) ; "  /* [2] */ \
                  "fstps  4(%0) ; "  /* [1] */ \
                  "fstps   (%0) ; "  /* [0] */ \
       : "=r" (__base) : "0" (__base) ); \
  }

void FP_load( 
  float FP_context[]
)
{
  asm volatile( "finit"  : : );

  LOAD_FP_CONTEXT( FP_context );

  STORE_FP_CONTEXT( FP_context );

  LOAD_FP_CONTEXT( FP_context );

}

int FP_check( 
  float FP_context[]
)
{
  float Context_to_compare[ 8 ];
  int   match;

  STORE_FP_CONTEXT( Context_to_compare );

  LOAD_FP_CONTEXT( Context_to_compare );

  match = memcmp( FP_context, Context_to_compare, 8 * sizeof( float ) );
  
  if ( match ) return( 1 );
  else         return( 0 );
}

int FP_use( void )
{
  return( 0 );
}
