/*  rtemscpu.h
 *
 *  This include file contains all information about RTEMS that
 *  is particular to the i80386 implementation of RTEMS except
 *  for the definition of the basic types.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __RTEMS_CPU_h
#define __RTEMS_CPU_h

/*
 *  The following defines the minimum stack size for this processor.
 */

#define MINIMUM_STACK_SIZE  256         /* minimum task stack size  */

/*
 *  The following defines the number of bits actually used in the
 *  interrupt field of the task mode and how that maps to CPU 
 *  interrupt levels.
 */

#define INTERRUPT_MASK          0x00000001  /* interrupt mode bits      */
#define INTERRUPT_LEVEL(_level) ((_level > 0x1) ? 1 : 0 )

/*
 *  The following defines the size of the reserved area in the
 *  Task Control Block for this processor.
 */

#define RTEMS_TCB_RESERVED_AREA       144

/*
 *  I80386 registers
 */

struct registers {
  rtems_unsigned32  eflags; /* extended flags register                   */
  void       *esp;          /* extended stack pointer register           */
  void       *ebp;          /* extended base pointer register            */
  rtems_unsigned32  ebx;    /* extended bx register                      */
  rtems_unsigned32  esi;    /* extended source index register            */
  rtems_unsigned32  edi;    /* extended destination index flags register */
};

/*
 *  CPU information table
 */

struct cpu_configuration_table {
  rtems_unsigned32  interrupt_segment;
  void             *interrupt_vector_table;
};

#endif
/* end of include file */
