/* cpu.h
 *
 *  This file contains macros which are used to access i80960CA 
 *  registers which are not addressable by C.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "rtems.h"

typedef void ( *i960ca_isr )( void );

#define disable_intr( oldlevel ) \
  { (oldlevel) = 0x1f0000; \
    asm volatile ( "modpc   0,%1,%1" \
                       : "=d" ((oldlevel)) \
                       : "0"  ((oldlevel)) ); \
  }

#define enable_intr( oldlevel ) \
  { rtems_unsigned32 _mask = 0x1f0000; \
    asm volatile ( "modpc   0,%0,%1" \
                       : "=d" (_mask), "=d" ((oldlevel)) \
                       : "0"  (_mask), "1"  ((oldlevel)) ); \
  }

#define flash_intr( oldlevel ) \
  { rtems_unsigned32 _mask = 0x1f0000; \
    asm volatile ( "modpc   0,%0,%1 ; \
                    mov     %0,%1 ; \
                    modpc   0,%0,%1"  \
                       : "=d" (_mask), "=d" ((oldlevel)) \
                       : "0"  (_mask), "1"  ((oldlevel)) ); \
  }

#define atomic_modify( mask, addr, prev ) \
 { register rtems_unsigned32  _mask = (mask); \
   register rtems_unsigned32 *_addr = (rtems_unsigned32 *)(addr); \
   asm volatile( "atmod  %0,%1,%1" \
                  : "=d" (_addr), "=d" (_mask) \
                  : "0"  (_addr), "1"  (_mask) ); \
   (prev) = _mask; \
 }
   
#define delay( microseconds ) \
  { register rtems_unsigned32 _delay=(microseconds); \
    register rtems_unsigned32 _tmp; \
    asm volatile( "delay0: \
                     remo      3,31,%0 ; \
                     cmpo      0,%0 ; \
                     subo      1,%1,%1 ; \
                     cmpobne.t 0,%1,delay0 " \
                  : "=d" (_tmp), "=d" (_delay) \
                  : "0"  (_tmp), "1"  (_delay) ); \
  }

#define enable_tracing() \
 { register rtems_unsigned32 _pc = 0x1; \
   asm volatile( "modpc 0,%0,%0" : "=d" (_pc) : "0" (_pc) ); \
 }

#define unmask_intr( xint ) \
 { register rtems_unsigned32 _mask= (1<<(xint)); \
   asm volatile( "or sf1,%0,sf1" : "=d" (_mask) : "0" (_mask) ); \
 }

#define mask_intr( xint ) \
 { register rtems_unsigned32 _mask= (1<<(xint)); \
   asm volatile( "andnot %0,sf1,sf1" : "=d" (_mask) : "0" (_mask) ); \
 }

#define clear_intr( xint ) \
 { register rtems_unsigned32 _xint=(xint); \
   asm volatile( "loop_til_cleared:
                    clrbit %0,sf0,sf0 ; \
                    bbs    %0,sf0,loop_til_cleared" \
                  : "=d" (_xint) : "0" (_xint) ); \
 }

#define reload_ctl_group( group ) \
 { register int _cmd = ((group)|0x400) ; \
   asm volatile( "sysctl %0,%0,%0" : "=d" (_cmd) : "0" (_cmd) ); \
 }

#define cause_intr( intr ) \
 { register int _intr = (intr); \
   asm volatile( "sysctl %0,%0,%0" : "=d" (_intr) : "0" (_intr) ); \
 }

#define soft_reset( prcb ) \
 { register struct i80960ca_prcb *_prcb = (prcb); \
   register rtems_unsigned32         *_next=0; \
   register rtems_unsigned32          _cmd  = 0x30000; \
   asm volatile( "lda    next,%1; \
                  sysctl %0,%1,%2; \
            next: mov    g0,g0" \
                  : "=d" (_cmd), "=d" (_next), "=d" (_prcb) \
                  : "0"  (_cmd), "1"  (_next), "2"  (_prcb) ); \
 }

static inline rtems_unsigned32 pend_intrs() 
{ register rtems_unsigned32 _intr=0; 
  asm volatile( "mov sf0,%0" : "=d" (_intr) : "0" (_intr) ); 
  return ( _intr ); 
}

static inline rtems_unsigned32 mask_intrs() 
{ register rtems_unsigned32 _intr=0;
  asm volatile( "mov sf1,%0" : "=d" (_intr) : "0" (_intr) ); 
  return( _intr );
}

static inline rtems_unsigned32 get_fp() 
{ register rtems_unsigned32 _fp=0; 
  asm volatile( "mov fp,%0" : "=d" (_fp) : "0" (_fp) ); 
  return ( _fp ); 
}

struct i80960ca_prcb   *Prcb;     /* pointer to processor control block */
struct i80960ca_ctltbl *Ctl_tbl;  /* pointer to control table */

/* functions */

i960ca_isr set_vector( rtems_isr_entry, unsigned int, unsigned int );

/* end of include file */
