/*  rtemscpu.h
 *
 *  This include file contains all information about RTEMS that 
 *  is particular to the i960CA implementation of RTEMS except
 *  for the definition of the basic types.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __RTEMS_CPU_h
#define __RTEMS_CPU_h

/*
 *  The following defines the minimum stack size for this processor.
 */

#define MINIMUM_STACK_SIZE  1024        /* minimum task stack size  */

/*
 *  The following defines the number of bits actually used in the
 *  interrupt field of the task mode and how that maps to CPU 
 *  interrupt levels.
 */

#define INTERRUPT_MASK         0x0000001f   /* interrupt mode bits      */
#define INTERRUPT_LEVEL(level) (level&0x1f)

/*
 *  The following defines the size of the reserved area in the
 *  Task Control Block for this processor.
 */

#define RTEMS_TCB_RESERVED_AREA       152

/*
 *  i80960CA registers
 */

struct registers {
  rtems_unsigned32 r0_pfp;          /* (r0)  Previous Frame Pointer */
  rtems_unsigned32 r1_sp;           /* (r1)  Stack Pointer        */
  rtems_unsigned32 pc;              /* (pc)  Processor Control    */
  rtems_unsigned32 g8;              /* (g8)  Global Register 8    */
  rtems_unsigned32 g9;              /* (g9)  Global Register 9    */
  rtems_unsigned32 g10;             /* (g10) Global Register 10   */
  rtems_unsigned32 g11;             /* (g11) Global Register 11   */
  rtems_unsigned32 g12;             /* (g12) Global Register 12   */
  rtems_unsigned32 g13;             /* (g13) Global Register 13   */
  rtems_unsigned32 g14;             /* (g14) Global Register 14   */
  rtems_unsigned32 g15_fp;          /* (g15) Frame Pointer        */
};

/*
 *  CPU information
 */

struct cpu_configuration_table {
  struct i80960ca_prcb *Prcb; /* address of i80960CA Process*/
                              /*   Control Block (PRCB)     */
};

/*
 *  Intel i80960CA Processor Control Block
 */

struct i80960ca_prcb {
  rtems_unsigned32    *fault_tbl;     /* fault table base address     */
  struct i80960ca_ctltbl
                      *control_tbl;   /* control table base address   */
  rtems_unsigned32     initial_ac;    /* AC register initial value    */
  rtems_unsigned32     fault_config;  /* fault configuration word     */
  void                *intr_tbl;      /* interrupt table base address */
  void                *sys_proc_tbl;  /* system procedure table       */
                                      /*   base address               */
  rtems_unsigned32     reserved;      /* reserved                     */
  rtems_unsigned32    *intr_stack;    /* interrupt stack pointer      */
  rtems_unsigned32     ins_cache_cfg; /* instruction cache            */
                                      /*   configuration word         */
  rtems_unsigned32     reg_cache_cfg; /* register cache               */
                                      /*   configuration word         */
};

/*
 *  Intel i80960CA Control Table
 */

struct i80960ca_ctltbl {
                            /* Control Group 0 */
  rtems_unsigned32 ipb0;              /* IP breakpoint 0 */
  rtems_unsigned32 ipb1;              /* IP breakpoint 1 */
  rtems_unsigned32 dab0;              /* data address breakpoint 0 */
  rtems_unsigned32 dab1;              /* data address breakpoint 1 */
                            /* Control Group 1 */
  rtems_unsigned32 imap0;             /* interrupt map 0 */
  rtems_unsigned32 imap1;             /* interrupt map 1 */
  rtems_unsigned32 imap2;             /* interrupt map 2 */
  rtems_unsigned32 icon;              /* interrupt control */
                            /* Control Group 2 */
  rtems_unsigned32 mcon0;             /* memory region 0 configuration */
  rtems_unsigned32 mcon1;             /* memory region 1 configuration */
  rtems_unsigned32 mcon2;             /* memory region 2 configuration */
  rtems_unsigned32 mcon3;             /* memory region 3 configuration */
                            /* Control Group 3 */
  rtems_unsigned32 mcon4;             /* memory region 4 configuration */
  rtems_unsigned32 mcon5;             /* memory region 5 configuration */
  rtems_unsigned32 mcon6;             /* memory region 6 configuration */
  rtems_unsigned32 mcon7;             /* memory region 7 configuration */
                            /* Control Group 4 */
  rtems_unsigned32 mcon8;             /* memory region 8 configuration */
  rtems_unsigned32 mcon9;             /* memory region 9 configuration */
  rtems_unsigned32 mcon10;            /* memory region 10 configuration */
  rtems_unsigned32 mcon11;            /* memory region 11 configuration */
                            /* Control Group 5 */
  rtems_unsigned32 mcon12;            /* memory region 12 configuration */
  rtems_unsigned32 mcon13;            /* memory region 13 configuration */
  rtems_unsigned32 mcon14;            /* memory region 14 configuration */
  rtems_unsigned32 mcon15;            /* memory region 15 configuration */
                            /* Control Group 6 */
  rtems_unsigned32 bpcon;             /* breakpoint control */
  rtems_unsigned32 tc;                /* trace control */
  rtems_unsigned32 bcon;              /* bus configuration control */
  rtems_unsigned32 reserved;          /* reserved */
};

#endif
/* end of include file */
