/*  shmcpu.h
 *
 *  This file contains Shared Memory Driver support which is CPU 
 *  dependent but target independent. 
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __SHMCPU_h
#define __SHMCPU_h

/*  The following routine swaps the endian format of an rtems_unsigned32.
 *  It cannot be static because it is referenced indirectly.
 *
 *  This version is based on code presented in Vol. 4, No. 4 of
 *  Insight 960.  It is certainly something you wouldn't think
 *  of on your own.
 *
 *  NOTE:  This function is actually instantiated when its
 *         address is referenced.  Because of this, its address
 *         should only be taken in ONE place and that place
 *         is when the SHM configuration table is built.
 */

static inline rtems_unsigned32 Shm_swap_it( 
  rtems_unsigned32 value
)
{
  register rtems_unsigned32 to_swap = value;
  register rtems_unsigned32 temp    = 0xFF00FF00;
  register rtems_unsigned32 swapped = 0;

                                            /*  to_swap      swapped  */
  asm volatile ( "rotate  16,%0,%2 ;"       /* 0x12345678  0x56781234 */
                 "modify  %1,%0,%2 ;"       /* 0x12345678  0x12785634 */
                 "rotate  8,%2,%2"          /* 0x12345678  0x78563412 */
                 : "=r" (to_swap), "=r" (temp), "=r" (swapped)
                 : "0" (to_swap), "1" (temp), "2" (swapped)
               );
  return( swapped );
}

#endif
/* end of include file */
