/* m68020.h
 *
 *  This file contains macros which are used to access MC68020 
 *  registers which are not addressable by C.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __CPU_h__
#define __CPU_h__

typedef void ( *mc68020_isr )( void );

#define disable_intr( level ) \
  { (level) = 0; \
    asm volatile ( "movew   %%sr,%0 ; \
                    orw     #0x0700,%%sr" \
                    : "=d" ((level)) : "0" ((level)) ); \
  }

#define enable_intr( level ) \
  { asm volatile ( "movew   %0,%%sr " \
                       : "=d" ((level)) : "0" ((level)) ); \
  }

#define flash_intr( level ) \
  { asm volatile ( "movew   %0,%%sr ; \
                    orw     #0x0700,%%sr" \
                       : "=d" ((level)) : "0" ((level)) ); \
  }

#define get_vbr( vbr ) \
  { (vbr) = 0; \
    asm volatile ( "movec   %%vbr,%0 " \
                       : "=a" (vbr) : "0" (vbr) ); \
  }

#define set_vbr( vbr ) \
  { register mc68020_isr *_vbr= (mc68020_isr *)(vbr); \
    asm volatile ( "movec   %0,%%vbr " \
                       : "=a" (_vbr) : "0" (_vbr) ); \
  }

#define enable_caching() \
  { register rtems_unsigned32 _ctl=0x01; \
    asm volatile ( "movec   %0,%%cacr" \
                       : "=d" (_ctl) : "0" (_ctl) ); \
  }

#define delay( microseconds ) \
  { register rtems_unsigned32 _delay=(microseconds); \
    register rtems_unsigned32 _tmp=123; \
    asm volatile( "0: \
                     nbcd      %0 ; \
                     nbcd      %0 ; \
                     dbf       %1,0 " \
                  : "=d" (_tmp), "=d" (_delay) \
                  : "0"  (_tmp), "1"  (_delay) ); \
  }

#define enable_tracing() 
#define cause_intr( X ) 
#define clear_intr( X ) 

extern mc68020_isr     M68Kvec[];   /* vector table address */

mc68020_isr set_vector( 
  rtems_isr_entry     handler,
  rtems_vector_number vector,
  int                 type
);

#endif
/* end of include file */
