/*  rtemscpu.h
 *
 *  This include file contains all information about RTEMS that
 *  is particular to the MC68020 implementation of RTEMS except
 *  for the definition of the basic types.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __RTEMS_CPU_h
#define __RTEMS_CPU_h

/*
 *  The following defines the minimum stack size for this processor.
 */

#define MINIMUM_STACK_SIZE  256         /* minimum task stack size  */

/*
 *  The following defines the number of bits actually used in the
 *  interrupt field of the task mode and how that maps to CPU 
 *  interrupt levels.
 */

#define INTERRUPT_MASK         0x0000001f   /* interrupt mode bits      */
#define INTERRUPT_LEVEL(level) (level&0x1f)

/*
 *  The following defines the size of the reserved area in the
 *  Task Control Block for this processor.
 */

#define RTEMS_TCB_RESERVED_AREA       144

/*
 *  MC68020 registers
 */

struct registers {
  rtems_unsigned32 sr;              /* (sr) status register      */
  rtems_unsigned32 d2;              /* (d2) data register 2      */
  rtems_unsigned32 d3;              /* (d3) data register 3      */
  rtems_unsigned32 d4;              /* (d4) data register 4      */
  rtems_unsigned32 d5;              /* (d5) data register 5      */
  rtems_unsigned32 d6;              /* (d6) data register 6      */
  rtems_unsigned32 d7;              /* (d7) data register 7      */
  rtems_unsigned32 a2;              /* (a2) address register 2   */
  rtems_unsigned32 a3;              /* (a3) address register 3   */
  rtems_unsigned32 a4;              /* (a4) address register 4   */
  rtems_unsigned32 a5;              /* (a5) address register 5   */
  rtems_unsigned32 a6;              /* (a6) address register 6   */
  rtems_unsigned32 a7_msp;          /* (a7) master stack pointer */
};

/*
 *  CPU information table
 */

struct cpu_configuration_table {
  void   *interrupt_vector_table; /* interrupt vector table     */
};

#endif
/* end of include file */
