/*  shmcpu.h
 *
 *  This file contains Shared Memory Driver support which is CPU 
 *  dependent but target independent. 
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#ifndef __SHMCPU_h
#define __SHMCPU_h

/*  The following routine swaps the endian format of an rtems_unsigned32.
 *  It cannot be static because it is referenced indirectly.
 *
 *  This version will work on any processor, but is the best 
 *  version for the i80960ca because it has no eight or sixteen
 *  bit register operations.
 *
 *  NOTE:  This function is actually instantiated when its
 *         address is referenced.  Because of this, its address
 *         should only be taken in ONE place and that place
 *         is when the SHM configuration table is built.
 */

static inline rtems_unsigned32 Shm_swap_it( value )
rtems_unsigned32 value;
{
  rtems_unsigned32 byte1, byte2, byte3, byte4, swapped;

  byte4 = (value >> 24) & 0xff;
  byte3 = (value >> 16) & 0xff;
  byte2 = (value >> 8)  & 0xff; 
  byte1 =  value        & 0xff;

  swapped = (byte1 << 24) | (byte2 << 16) | (byte3 << 8) | byte4;
  return( swapped );
}

#endif
/* end of include file */
