/*  void Shm_get_config( localnode, &shmcfg )
 *
 *  This routine initializes, if necessary, and returns a pointer
 *  to the Shared Memory Configuration Table for the FORCE CPU-386
 *
 *  INPUT PARAMETERS:
 *    localnode - local node number
 *    shmcfg    - address of pointer to SHM Config Table
 *
 *  OUTPUT PARAMETERS:
 *    *shmcfg   - pointer to SHM Config Table
 *
 *  NOTES:  The FORCE CPU-386 does not have an interprocessor interrupt. 
 * 
 *          The following table illustrates the configuration limitations:
 *
 *                                   BUS     MAX
 *                          MODE    ENDIAN  NODES
 *                        ========= ====== =======
 *                         POLLED    BIG    2+
 *                        INTERRUPT **** NOT SUPPORTED ****
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "rtems.h"
#include "shm.h"
#include "cpu.h"

#define INTERRUPT 0 
#define POLLING   1        /* FORCE CPU-386 target is polling ONLY!!! */ 


shm_config_table F386_shm_cfgtbl;

void Shm_get_config( localnode, shmcfg )
rtems_unsigned32         localnode;
shm_config_table **shmcfg;
{
   set_segment( get_ds(), 0x00002000, 0xffffd000  );

   F386_shm_cfgtbl.base         = Physical_to_logical( 
                                    get_ds(),
                                    (void *) 0x20000000
                                  );

   F386_shm_cfgtbl.length       = 1 * MEGABYTE;
   F386_shm_cfgtbl.format       = SHM_BIG;

#ifdef NEUTRAL_BIG
   F386_shm_cfgtbl.convert      = NULL_CONVERT;
#else
   F386_shm_cfgtbl.convert      = Shm_swap_it;
#endif

   F386_shm_cfgtbl.poll_intr    = POLLED_MODE;
   F386_shm_cfgtbl.Intr.address = NO_INTERRUPT;
   F386_shm_cfgtbl.Intr.value   = NO_INTERRUPT;
   F386_shm_cfgtbl.Intr.length  = NO_INTERRUPT;

   *shmcfg = &F386_shm_cfgtbl;
}
