/*  bsp_start()
 *
 *  This routine starts the application.  It includes application,
 *  board, and monitor specific initialization and configuration.
 *  The generic CPU dependent initialization has been performed
 *  before this routine is invoked.
 * 
 *  INPUT:  NONE
 * 
 *  OUTPUT: NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "rtems.h"
#include "cvme961.h"
#include "cpu.h"

extern rtems_configuration_table Configuration;

rtems_cpu_table Cpu_table;

void bsp_start()
{
  /* set node number in SQSIO4 CTL REG */

  *((rtems_unsigned32 *)0xc00000b0) = 
       (Configuration.User_multiprocessing_table) ? 
          Configuration.User_multiprocessing_table->node : 0;

  Prcb    = get_prcb();
  Ctl_tbl = Prcb->control_tbl;

  /* following configures the data breakpoint (which must be set 
   *   before this is executed) to break on writes only.
   */

  Ctl_tbl->bpcon &= ~0x00cc0000; 
  reload_ctl_group( 6 );

   /*  bit 31 of the Register Cache Control can be set to
    *  enable an alternative caching algorithm.  It does
    *  not appear to help RTEMS.
    */

   /* Configure Number of Register Caches */

  Prcb->reg_cache_cfg = 8;
  soft_reset( Prcb );

  Cpu_table.Prcb = Prcb;

  Configuration.work_space_start = (void *)
     (0x100000 - Configuration.work_space_size);

  initialize_executive( &Configuration, &Cpu_table ); /* does not return */
}
