/*  Clock_isr
 * 
 *  This routine is the clock tick for a MVME136 VME board running
 *  RTEMS for the onboard Z8036.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "rtems.h"
#include "clockdrv.h"
#include "dmv152.h"
#include "cpu.h"
#include "z8536.h"

extern rtems_unsigned32 Clock_isrs;
extern rtems_configuration_table Configuration;

#define TIMER 0x0c000000

/* standard version has incorrect offset!!! Must address this!!! */
#undef Z8x36_STATE0
#undef Z8x36_WRITE
#undef Z8x36_READ

#define Z8x36_STATE0 ( z8536 ) \
  { char *garbage; \
    (garbage) = *(VOL8(z8536+0x7)) \
  }

#define Z8x36_WRITE( z8536, reg, data ) \
   *(VOL8(z8536+0x7)) = (reg); \
   *(VOL8(z8536+0x7)) = (data)

#define Z8x36_READ( z8536, reg, data ) \
   *(VOL8(z8536+0x7)) = (reg); \
   (data) = *(VOL8(z8536+0x7))

rtems_isr Clock_isr(
  rtems_vector_number vector
)
{
  /* enable_tracing(); */
  Clock_driver_ticks += 1;
  Z8x36_WRITE( TIMER, CT1_CMD_STATUS, 0x26 );

  if ( Clock_isrs == 1 ) {
    clock_tick();
    Clock_isrs = Configuration.milliseconds_per_tick;
  }
  else 
    Clock_isrs -= 1;
}
