/*  bsp_start()
 *
 *  This routine starts the application.  It includes application,
 *  board, and monitor specific initialization and configuration.
 *  The generic CPU dependent initialization has been performed
 *  before this routine is invoked.
 * 
 *  INPUT:  NONE
 * 
 *  OUTPUT: NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "rtems.h"
#include "dmv152.h"
#include "cpu.h"
#include "clockdrv.h"

extern rtems_configuration_table  Configuration;

rtems_cpu_table Cpu_table;

void bsp_start()
{
  mc68020_isr *monitors_vector_table;
  int          index;

  monitors_vector_table = (mc68020_isr *)0;   /* 135Bug Vectors are at 0 */
  set_vbr( monitors_vector_table );

  for ( index=2 ; index<=255 ; index++ )
    M68Kvec[ index ] = monitors_vector_table[ 32 ];

  M68Kvec[  2 ] = monitors_vector_table[  2 ];   /* bus error vector */
  M68Kvec[  4 ] = monitors_vector_table[  4 ];   /* breakpoints vector */
  M68Kvec[  9 ] = monitors_vector_table[  9 ];   /* trace vector */

/*
  set_vbr( &M68Kvec );
*/

  /* ??? Is it already in this mode? */ /* make VME access round-robin */

  enable_caching();

  Cpu_table.interrupt_vector_table = (mc68020_isr *) &M68Kvec;

  Configuration.work_space_start = (void *)
     (0x100000 - Configuration.work_space_size);

  initialize_executive( &Configuration, &Cpu_table );/* does not return */

  /* Clock_exit is done as an atexit() function */
}
