/*  set_vector
 *
 *  This routine installs an interrupt vector on the DMV152.
 *
 *  INPUT:
 *    handler - interrupt handler entry point
 *    vector  - vector number
 *    type    - 0 indicates raw hardware connect
 *              1 indicates RTEMS interrupt connect
 *
 *  RETURNS:
 *    address of previous interrupt handler
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "rtems.h"
#include "cpu.h"

mc68020_isr set_vector(                         /* returns old vector */
  rtems_isr_entry     handler,                  /* isr routine        */
  rtems_vector_number vector,                   /* vector number      */
  int                 type                      /* RTEMS or RAW intr  */
)
{
  mc68020_isr *vbr = 0;
  mc68020_isr  previous_isr;

  get_vbr( vbr );
  if ( type ) 
    interrupt_catch( handler, vector, (rtems_isr_entry *) &previous_isr );
  else {
    previous_isr  = vbr[ vector ];            /* return old ISR  */
    vbr[ vector ] = (mc68020_isr) handler;    /* install new ISR */
  }
  return previous_isr;
}

