/*  Shm_init
 *
 *  This routine is the shared memory communications initerface
 *  driver initialization routine.
 *
 *  Input parameters:
 *    cfg - address of configuration table
 *
 *  Output parameters: NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#define _SHM_INIT

#include "rtems.h"
#include "shm.h"

extern int Shm_statistics;
int SHM_ADA_INITTED_VAR = 1;

rtems_mpci_entry Shm_init(
  rtems_configuration_table *cfg )
{
  rtems_unsigned32 i, *ptr, *endshm, all_initialized, time, len, val;
  rtems_unsigned32 *addr;
  struct node_status_cb *nscb, *localnscb;
  struct env_cb *ecb;
  rtems_id id;

  Shm_config_tbl    = cfg; 
  Shm_mp_config_tbl = Shm_config_tbl->User_multiprocessing_table;
  Shm_local_node    = Local_node();
  Shm_get_config( Shm_local_node ,&Shm_config );
  time = 300*(1000/cfg->milliseconds_per_tick);

  Shm_recvmsgcnt = 0;
  Shm_nullmsgcnt = 0;
  Shm_isr_count  = 0;
  Ns_cbs         = (struct node_status_cb *) START_NS_CBS;
  Lq_cbs         = (struct lockedq_cb *) START_LQ_CBS;
  Envelopes      = (struct env_cb *) START_ENVELOPES;

  Local_lqcb     = &Lq_cbs[ Shm_local_node ];
  localnscb      = &Ns_cbs[ Shm_local_node ];
 
  addr = (rtems_unsigned32 *)Shm_convert( 
                 (rtems_unsigned32)Int_address(Shm_local_node) );
  val  = Shm_convert( Int_value(Shm_local_node) );
  len  = Shm_convert( Int_length(Shm_local_node) );

  if ( Shm_config->poll_intr == POLLED_MODE ) Shm_setclockvec();
  else                                        Shm_setvec();

  if ( Is_master() ) {
    for ( ptr = (rtems_unsigned32 *)START_SHARED_MEM, 
          endshm = (rtems_unsigned32 *)END_SHARED_MEM ; 
          ptr < endshm ; ) 
      *ptr++ = 0;

    Init_lockedq( FREE_ENV_CB, FREE_ENV_POOL );

    for ( ecb=Envelopes, i=0 ; i<MAX_ENVELOPES ; i++, ecb++ ) {
      ecb->index = Shm_convert(i);
      Free_envelope( ecb );
    }

    for ( i=FIRST_NODE ; i<=MAX_NODES ; i++ ) 
      Init_recvq( i );

    for ( nscb = &Ns_cbs[FIRST_NODE], i = FIRST_NODE ;
          i <= MAX_NODES ;
          i++, nscb++ ) {
      nscb->status = PENDING_INIT;
      nscb->error  = 0;
    }

    Set_local_intr_info( localnscb, addr, val, len );
    localnscb->status      = INIT_COMPLETE;
    
    for ( all_initialized=0 ; !all_initialized ; ) 
    for ( nscb = &Ns_cbs[FIRST_NODE], i = FIRST_NODE, all_initialized = 1 ;
          i <= MAX_NODES ;
          i++, nscb++ )
        if ( nscb->status != INIT_COMPLETE )
           all_initialized = 0;

    for ( nscb = &Ns_cbs[FIRST_NODE], i = FIRST_NODE ;
          i <= MAX_NODES ;
          i++, nscb++ )
      nscb->status = ACTIVE_NODE;

  } 
  else {
    localnscb->status = PENDING_INIT;
    do {
      if ( localnscb->status == PENDING_INIT ) {
        Set_local_intr_info( localnscb, addr, val, len );
        localnscb->status      = INIT_COMPLETE;
      }
    } while ( localnscb->status != ACTIVE_NODE ) ;
  }


  for ( nscb = &Ns_cbs[FIRST_NODE], i = FIRST_NODE ;
        i <= MAX_NODES ;
        i++, nscb++ ) {
    Intr_info[i].address = (rtems_unsigned32 *)
       Shm_addrconv((rtems_unsigned32 *)Shm_convert((rtems_unsigned32)nscb->int_address));
    Intr_info[i].value   = Shm_convert( nscb->int_value );
    Intr_info[i].length  = Shm_convert( nscb->int_length );
  }

  if ( Shm_statistics ) {
    Lo_idle = Hi_idle = 0;
    task_create( 1, 1, 1024, NO_PREEMPT, DEFAULT_ATTRIBUTES, &id );
    task_start( id, Shm_dump, time );
    task_create( 1, 254, 1024, DEFAULT_MODES, DEFAULT_ATTRIBUTES, &id );
    task_start( id, Shm_idle, 0 );
  }

}
