/*  Shm_send
 *
 *  This routine is the shared memory driver locked queue write 
 *  MPCI driver routine.  This routine sends the specified packet
 *  to the destination specified by "node".  A "node" value of
 *  zero designates that this packet is to be broadcasted. 
 *
 *  If this driver is used in interrupt mode, it will interrupt the 
 *  the receiving node using the MPCSR available on the Motorola
 *  MVME136 board.
 *
 *  Input parameters:
 *    node          - destination of this packet (0 = broadcast)
 *    packet        - address of packet 
 *    packet_length - length of packet 
 * 
 *  Output parameters: NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "rtems.h"
#include "shm.h"

struct pkt_cpy {
  rtems_unsigned32 packet[MAX_PACKET_SIZE/4];
};

rtems_mpci_entry Shm_send(
  rtems_unsigned32 node,
  rtems_unsigned8 *packet,
  rtems_unsigned32 packet_length )
{
  struct env_cb *ecb, *tmp_ecb;
  rtems_unsigned32 nnum;

  ecb = Pkt_to_ecb( packet );
  if ( node ) {
    Build_preamble( ecb, node );
    Build_postamble( ecb );
    Add_recvq( node, ecb );
    Shm_interrupt( node );
  }
  else {
    for( nnum = FIRST_NODE ; nnum <= MAX_NODES ; nnum++ )
      if ( Shm_local_node != nnum ) {
        Get_envelope( tmp_ecb );
        if ( !tmp_ecb ) 
          fatal_error_occurred( SHM_NO_FREE_PKTS );
        Build_preamble( tmp_ecb, nnum );
        *((struct pkt_cpy *)tmp_ecb->packet) = *((struct pkt_cpy *)packet);
        Build_postamble( tmp_ecb );
        Add_recvq( nnum, tmp_ecb );
        Shm_interrupt( nnum );
      }
    Free_envelope( ecb );
  }
}
