/*  rtems
 *
 *  This routine is the single entry point used by assembly language 
 *  programs. It jumps to the requested directive.
 *
 *  INPUT PARAMETERS: 
 *     CPU DEPENDENT - directive function code
 *
 *  OUTPUT PARAMETERS: DOES NOT RETURN
 *
 *  NOTE: The table _Entry_points is included here to prevent the
 *        entire executive from being pulled from the library
 *        unless this entry point is utilized.
 * 
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "rtems.h"
#include "asmiface.h"

const void * _Entry_points[] = {
  (void *) _Initialize_Executive,                      /*  0 */
  (void *) _Shutdown_Executive,                        /*  1 */
  (void *) _RTEMS_tasks_Create,                        /*  2 */
  (void *) _RTEMS_tasks_Name_to_id,                    /*  3 */
  (void *) _RTEMS_tasks_Start,                         /*  4 */
  (void *) _RTEMS_tasks_Restart,                       /*  5 */
  (void *) _RTEMS_tasks_Delete,                        /*  6 */
  (void *) _RTEMS_tasks_Suspend,                       /*  7 */
  (void *) _RTEMS_tasks_Resume,                        /*  8 */
  (void *) _RTEMS_tasks_Set_priority,                  /*  9 */
  (void *) _RTEMS_tasks_Mode,                          /* 10 */
  (void *) _RTEMS_tasks_Get_note,                      /* 11 */
  (void *) _RTEMS_tasks_Set_note,                      /* 12 */
  (void *) _RTEMS_tasks_Wake_after,                    /* 13 */
  (void *) _RTEMS_tasks_Wake_when,                     /* 14 */
  (void *) _Interrupt_Catch,                           /* 15 */
  (void *) _Clock_Set,                                 /* 16 */
  (void *) _Clock_Get,                                 /* 17 */
  (void *) _Clock_Tick,                                /* 18 */
  (void *) _Timer_Create,                              /* 19 */
  (void *) _Timer_Name_to_id,                          /* 20 */
  (void *) _Timer_Cancel,                              /* 21 */
  (void *) _Timer_Delete,                              /* 22 */
  (void *) _Timer_Fire_after,                          /* 23 */
  (void *) _Timer_Fire_when,                           /* 24 */
  (void *) _Timer_Reset,                               /* 25 */
  (void *) _Semaphore_Create,                          /* 26 */
  (void *) _Semaphore_Name_to_id,                      /* 27 */
  (void *) _Semaphore_Delete,                          /* 28 */
  (void *) _Semaphore_Obtain,                          /* 29 */
  (void *) _Semaphore_Release,                         /* 30 */
  (void *) _Message_queue_Create,                      /* 31 */
  (void *) _Message_queue_Name_to_id,                  /* 32 */
  (void *) _Message_queue_Delete,                      /* 33 */
  (void *) _Message_queue_Send,                        /* 34 */
  (void *) _Message_queue_Urgent,                      /* 35 */
  (void *) _Message_queue_Broadcast,                   /* 36 */
  (void *) _Message_queue_Receive,                     /* 37 */
  (void *) _Message_queue_Flush,                       /* 38 */
  (void *) _Event_Send,                                /* 39 */
  (void *) _Event_Receive,                             /* 40 */
  (void *) _Signal_Catch,                              /* 41 */ 
  (void *) _Signal_Send,                               /* 42 */
  (void *) _Partition_Create,                          /* 43 */
  (void *) _Partition_Name_to_id,                      /* 44 */
  (void *) _Partition_Delete,                          /* 45 */
  (void *) _Partition_Get_buffer,                      /* 46 */
  (void *) _Partition_Return_buffer,                   /* 47 */
  (void *) _Region_Create,                             /* 48 */
  (void *) _Region_Name_to_id,                         /* 49 */
  (void *) _Region_Delete,                             /* 50 */
  (void *) _Region_Get_segment,                        /* 51 */
  (void *) _Region_Return_segment,                     /* 52 */
  (void *) _Dual_ported_memory_Create,                 /* 53 */
  (void *) _Dual_ported_memory_Name_to_id,             /* 54 */
  (void *) _Dual_ported_memory_Delete,                 /* 55 */
  (void *) _Dual_ported_memory_External_to_internal,   /* 56 */
  (void *) _Dual_ported_memory_Internal_to_external,   /* 57 */
  (void *) _IO_Initialize,                             /* 58 */
  (void *) _IO_Open,                                   /* 59 */
  (void *) _IO_Close,                                  /* 60 */
  (void *) _IO_Read,                                   /* 61 */
  (void *) _IO_Write,                                  /* 62 */
  (void *) _IO_Control,                                /* 63 */
  (void *) _Fatal_Error_occurred,                      /* 64 */
  (void *) _Rate_monotonic_Create,                     /* 65 */
  (void *) _Rate_monotonic_Name_to_id,                 /* 66 */
  (void *) _Rate_monotonic_Delete,                     /* 67 */
  (void *) _Rate_monotonic_Cancel,                     /* 68 */
  (void *) _Rate_monotonic_Period,                     /* 69 */
  (void *) _Multiprocessing_Announce                   /* 70 */
};

enum status_codes rtems()
{
  _Jump_to_directive();

  /* Should never really get here, but this turns off compiler warning
   * and will cause us to do something intelligible when the impossible
   * happens
   */
  return UNSATISFIED;
}
