/*				f r e a d
 *
 * Read a series of records from an input stream. The function
 * returns the number of items (not bytes) read. The function
 * stops reading when the number of items is satisfied or when
 * EOF is encountered.
 */

#include "stdiolib.h"
#include "block_io.h"


/*LINTLIBRARY*/

size_t
fread(ptr, size, nitems, fp)
    void *ptr;		/* buffer */
    size_t size;	/* size of record */
    size_t nitems;	/* number of items */
    FILE *fp;		/* stream */

{
	unsigned int nbytes;	/* Total number of bytes to read	*/
	unsigned int avail;	/* Number of bytes available in input buffer */
	unsigned char *fbuf;	/* Pointer into file's input buffer	*/
	int r;
	int c;



	/* Flush stdout */
	if (fp == stdin && TESTFLAG(stdout, _IOLBF)){
		(void) fflush(stdout);
	}

	if ( (nbytes = nitems*size) == 0 ){
		return 0;
	}

	/* Unallocated buffer */
	if (! HASBUFFER(fp)) {
		if (_allocbuf(fp) < 0){
			return 0;
		}
		SETIOFLUSH();
	}

	/* First copy bytes already available from buffer
	 */
	if ( (avail = BYTESINREADBUFFER(fp)) > 0 ){
		fbuf = GETREADPTR(fp);
		while ( avail-- && nbytes ){
			*((unsigned char*)ptr)++ = *fbuf++ ;
			nbytes--;
		}
		SETREADPTR(fp, fbuf);
	}

	/* If we still need more than a buffer's worth, read it all at once.
	 */
	if ( nbytes >= BUFFERSIZE(fp) ){
		r = read( fileno(fp), (char*)ptr, (int)nbytes );
		if ( r != nbytes ){
			return ( nitems - ((nbytes-r)/size) );
		}
		nbytes = 0;
	}

	/* If we still need more, we must have needed less than a buffer's
	 * worth, but more than was in the buffer.  Re-fill the buffer and
	 * get the rest.
	 */
	if ( nbytes > 0 ){
		c = _filbuf(fp);
		*((unsigned char*)ptr)++ = c;
		nbytes--;

		avail = BYTESINREADBUFFER(fp);
		fbuf = GETREADPTR(fp);
		while ( avail-- && nbytes ){
			*((unsigned char*)ptr)++ = *fbuf++;
			nbytes--;
		}
		SETREADPTR(fp, fbuf);
	}

	/* If we *still* need input, calculate the number of items we
	 * failed to get.
	 */
	if  ( nbytes > 0 ){
		nitems -= nbytes / size;
		if ( nbytes % size ){
			nitems--;
		}
	}

	return nitems;
}
