/*			f w r i t e
 *
 * This function writes a series of records to the output stream.
 * The function returns the number of items written to the stream.
 * Writing stops when the correct number of items have been written
 * or when the function encounters an error.
 */

#include "stdiolib.h"

/*LINTLIBRARY*/

size_t
fwrite(ptr, size, nitems, fp)
    CONST void *ptr;	/* buffer */
    size_t size;	/* size of item */
    size_t nitems;	/* number of items */
    FILE *fp;		/* stream */
{
	extern int write();

	unsigned int nbytes;	/* Total number of bytes to write	*/
	unsigned int f;		/* Number of bytes free in output buffer*/
	unsigned char *fbuf;	/* Pointer into file's output  buffer	*/
	int w;			/* Return value from write		*/


	if ( (nbytes = nitems*size) == 0 ){
		return 0;
	}

	/* Unallocated buffer */
	if (! HASBUFFER(fp)) {
		if (_allocbuf(fp) < 0){
			return 0;
		}
		SETIOFLUSH();
	}

	f = UNUSEDINWRITEBUFFER(fp);
	if ( nbytes >= f ){

		/* Copy f bytes to buffer */
		fbuf = GETWRITEPTR(fp);
		while ( f-- ){
			*fbuf++ = *((unsigned char*)ptr)++;
			nbytes--;
		}
		SETWRITEPTR(fp, fbuf);

		/* Flush buffer */
		if ( fflush(fp) == EOF ){
			return 0;
		}

		/* If we still have more than a buffer's worth, write it all */
		if ( nbytes >= BUFFERSIZE(fp) ){
			w = write( fileno(fp), (char*)ptr, (int)nbytes );
			if ( w != nbytes ){
				return ( nitems - ((nbytes-w)/size) );
			}
			nbytes = 0;
		}
	}

	/* copy remaining bytes to buffer */
	fbuf = GETWRITEPTR(fp);
	while ( nbytes-- ){
		*fbuf++ = *((unsigned char*)ptr)++;
	}
	SETWRITEPTR(fp, fbuf);

	return nitems;
}
