/*
 * Copyright (c) 1988 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */


#ifdef LIBC_SCCS
static char sccsid[] = "@(#)doprnt.c	5.35 (Berkeley) 6/27/88";
#endif /* LIBC_SCCS */

#define __SRC__
#include <sys/types.h>
#include <varargs.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <ansidecl.h>
#include <printf.h>
typedef unsigned long u_long;

int __printf_fp(
  FILE *fp, 
  const struct printf_info *info,
  LONG_DOUBLE fp_num
/*
  va_list *args
*/
);


/* 11-bit exponent (VAX G floating point) is 308 decimal digits */
#define	MAXEXP		308
/* 128 bit fraction takes up 39 decimal digits; max reasonable precision */
#define	MAXFRACT	39

#define	DEFPREC		6

#define	BUF		(MAXEXP+MAXFRACT+1)	/* + decimal point */

#define	PUTC(ch)	(cnt++, (void) putc(ch, fp))


/*
 * on the i960, all integer types
 * occupy 4 bytes.
 */
#define	ARG() \
	_ulong = va_arg(argp, long);

#define TEN_MUL(x)	((x) * 10)

#define	todigit(c)	((c) - '0')
#define	tochar(n)	((n) + '0')

#define	LONGINT		0x01		/* long integer */
#define	LONGDBL		0x02		/* long double; unimplemented */
#define	SHORTINT	0x04		/* short integer */
#define	ALT		0x08		/* alternate form */
#define	LADJUST		0x10		/* left adjustment */
#define	ZEROPAD		0x20		/* zero (as opposed to blank) pad */
#define	HEXPREFIX	0x40		/* add 0x or 0X prefix */

int vfprintf(
	register FILE *fp,
	const char *fmt0,
	va_list argp
)
{
	register const char *fmt;	/* format string */
	register int ch;	/* character from fmt */
	register int cnt;	/* return value accumulator */
	register int n;		/* random handy integer */
	register char *t;	/* buffer pointer */
	u_long _ulong;		/* integer arguments %[diouxX] */
	int base;		/* base for [diouxX] conversion */
	int dprec;		/* decimal precision in [diouxX] */
	int fieldsz;		/* field size expanded by sign, etc */
	int flags;		/* flags as above */
	int fpprec;		/* `extra' floating precision in [eEfgG] */
	int prec;		/* precision from format (%.3d), or -1 */
	int realsz;		/* field size expanded by decimal precision */
	int size;		/* size of converted field or string */
	int width;		/* width from format (%8d), or 0 */
	char sign;		/* sign prefix (' ', '+', '-', or \0) */
	char *digs;		/* digits for [diouxX] conversion */
	char buf[BUF];		/* space for %c, %[diouxX], %[eEfgG] */
        char fc;                /* format letter */
	void *vp;		/* generic pointer var */

	fmt = fmt0;
	digs = "0123456789abcdef";
	for (cnt = 0;; ++fmt) {
		if (!(ch = *fmt))
			return (cnt);
		if (ch != '%') {
			PUTC(ch);
			continue;
		}
		flags = 0; dprec = 0; fpprec = 0; width = 0;
		prec = -1;
		sign = '\0';

rflag:		fc = *++fmt;
 		switch (fc) {
		case ' ':
			/*
			 * ``If the space and + flags both appear, the space
			 * flag will be ignored.''
			 *	-- ANSI X3J11
			 */
			if (!sign)
				sign = ' ';
			goto rflag;
		case '#':
			flags |= ALT;
			goto rflag;
		case '*':
			/*
			 * ``A negative field width argument is taken as a
			 * - flag followed by a  positive field width.''
			 *	-- ANSI X3J11
			 * They don't exclude field widths read from args.
			 */
			width = va_arg(argp, int);
			if (width >= 0)
				goto rflag;
			width = -width;
			/* FALLTHROUGH */
		case '-':
			flags |= LADJUST;
			goto rflag;
		case '+':
			sign = '+';
			goto rflag;
		case '.':
			if (*++fmt == '*')
				n = va_arg(argp, int);
			else {
				n = 0;
				while (isascii(*fmt) && isdigit(*fmt))
					n = TEN_MUL(n) + todigit(*fmt++);
				--fmt;
			}
			prec = n < 0 ? -1 : n;
			goto rflag;
		case '0':
			/*
			 * ``Note that 0 is taken as a flag, not as the
			 * beginning of a field width.''
			 *	-- ANSI X3J11
			 */
			flags |= ZEROPAD;
			goto rflag;
		case '1': case '2': case '3': case '4':
		case '5': case '6': case '7': case '8': case '9':
			n = 0;
			do {
				n = TEN_MUL(n) + todigit(*fmt);
			} while (isascii(*++fmt) && isdigit(*fmt));
			width = n;
			--fmt;
			goto rflag;
		case 'L':
			flags |= LONGDBL;
			goto rflag;
		case 'h':
			flags |= SHORTINT;
			goto rflag;
		case 'l':
			flags |= LONGINT;
			goto rflag;
		case 'c':
			*(t = buf) = va_arg(argp, int);
			size = 1;
			sign = '\0';
			goto pforw;
		case '%':
			/*
			 * "Note that padding is performed for this conversion
			 * operation just as for any other conversion operation."
			 * -- Harbison & Steele, 2nd. edition
			 */
			t = "%";
			size = 1;
			sign = '\0';
			goto pforw;
		case 'D':
			flags |= LONGINT;
			/*FALLTHROUGH*/
		case 'd':
		case 'i':
			ARG();
			if ((long)_ulong < 0) {
				_ulong = -_ulong;
				sign = '-';
			}
			base = 10;
			goto number;
		case 'e':
		case 'E':
		case 'f':
		case 'g':
		case 'G':
                  {
                    int function_done;
                    struct printf_info info;
                    LONG_DOUBLE fpnum;

                    info.prec           = prec;
                    info.width          = width;
                    info.spec           = fc;
                    info.is_long_double = (flags & LONGDBL)  ? 1 : 0;
                    info.is_short       = (flags & SHORTINT) ? 1 : 0;
                    info.is_long        = (flags & LONGINT)  ? 1 : 0;
                    info.alt            = (flags & ALT)      ? 1 : 0;
                    info.space          = (sign == ' ')      ? 1 : 0;
                    info.left           = (flags & LADJUST)  ? 1 : 0;
                    info.showsign       = (sign == '+')      ? 1 : 0;
                    info.pad            = (flags & ZEROPAD)  ? '0' : ' ';

                    if (info.is_long_double)
                      fpnum = va_arg (argp, LONG_DOUBLE);
                    else
                      fpnum = (LONG_DOUBLE) va_arg (argp, double);

                    function_done = __printf_fp( fp, &info, fpnum );
/*
                    function_done = __printf_fp( fp, &info, (va_list *) &argp );
*/
                    if (function_done < 0)
                      return(-1);

                    cnt += function_done;
                  }
                  continue;
		case 'n':
			vp = va_arg(argp, void *);
			if (flags & LONGINT)
				*((long *)vp) = cnt;
			else if (flags & SHORTINT)
				*((short *)vp) = cnt;
			else
				*((int *)vp) = cnt;
			break;
		case 'O':
			flags |= LONGINT;
			/*FALLTHROUGH*/
		case 'o':
			ARG();
			base = 8;
			goto nosign;
		case 'p':
			/*
			 * ``The argument shall be a pointer to void.  The
			 * value of the pointer is converted to a sequence
			 * of printable characters, in an implementation-
			 * defined manner.''
			 *	-- ANSI X3J11
			 */
			/* NOSTRICT */
			_ulong = (u_long)va_arg(argp, void *);
			base = 16;
			goto nosign;
		case 's':
			if (!(t = va_arg(argp, char *)))
				t = "(null)";
			if (prec >= 0) {
				/*
				 * can't use strlen; can only look for the
				 * NUL in the first `prec' characters, and
				 * strlen() will go further.
				 */
				char *p;
				void *memchr();

				if ( (p = (char *)memchr(t, 0, prec)) ) {
					size = p - t;
					if (size > prec)
						size = prec;
				} else
					size = prec;
			} else
				size = strlen(t);
			sign = '\0';
			goto pforw;
		case 'U':
			flags |= LONGINT;
			/*FALLTHROUGH*/
		case 'u':
			ARG();
			base = 10;
			goto nosign;
		case 'X':
			digs = "0123456789ABCDEF";
			/* FALLTHROUGH */
		case 'x':
			ARG();
			base = 16;
			/* leading 0x/X only if non-zero */
			if (flags & ALT && _ulong != 0)
				flags |= HEXPREFIX;

			/* unsigned conversions */
nosign:			sign = '\0';
			/*
			 * ``... diouXx conversions ... if a precision is
			 * specified, the 0 flag will be ignored.''
			 *	-- ANSI X3J11
			 */
number:			if ((dprec = prec) >= 0)
				flags &= ~ZEROPAD;

			/*
			 * ``The result of converting a zero value with an
			 * explicit precision of zero is no characters.''
			 *	-- ANSI X3J11
			 */
			t = buf + BUF;
			if (_ulong != 0 || prec != 0) {
				do {
					*--t = digs[_ulong % base];
					_ulong /= base;
				} while (_ulong);
				digs = "0123456789abcdef";
				if (flags & ALT && base == 8 && *t != '0')
					*--t = '0'; /* octal leading 0 */
			}
			size = buf + BUF - t;

pforw:
			/*
			 * All reasonable formats wind up here.  At this point,
			 * `t' points to a string which (if not flags&LADJUST)
			 * should be padded out to `width' places.  If
			 * flags&ZEROPAD, it should first be prefixed by any
			 * sign or other prefix; otherwise, it should be blank
			 * padded before the prefix is emitted.  After any
			 * left-hand padding and prefixing, emit zeroes
			 * required by a decimal [diouxX] precision, then print
			 * the string proper, then emit zeroes required by any
			 * leftover floating precision; finally, if LADJUST,
			 * pad with blanks.
			 */

			/*
			 * compute actual size, so we know how much to pad
			 * fieldsz excludes decimal prec; realsz includes it
			 */
			fieldsz = size + fpprec;
			if (sign)
				fieldsz++;
			if (flags & HEXPREFIX)
				fieldsz += 2;
			realsz = dprec > fieldsz ? dprec : fieldsz;

			/* right-adjusting blank padding */
			if ((flags & (LADJUST|ZEROPAD)) == 0 && width)
				for (n = realsz; n < width; n++)
					PUTC(' ');
			/* prefix */
			if (sign)
				PUTC(sign);
			if (flags & HEXPREFIX) {
				PUTC('0');
				PUTC((char)*fmt);
			}
			/* right-adjusting zero padding */
			if ((flags & (LADJUST|ZEROPAD)) == ZEROPAD)
				for (n = realsz; n < width; n++)
					PUTC('0');
			/* leading zeroes from decimal precision */
			for (n = fieldsz; n < dprec; n++)
				PUTC('0');

			/* the string or number proper */
			for (n = size; --n >= 0; )
				PUTC(*t++);
			/* trailing f.p. zeroes */
			while (--fpprec >= 0)
				PUTC('0');
			/* left-adjusting padding (always blank) */
			if (flags & LADJUST)
				for (n = realsz; n < width; n++)
					PUTC(' ');
			break;
		case '\0':	/* "%?" prints ?, unless ? is NULL */
			return (cnt);
		default:
			PUTC((char)*fmt);
		}
	}
	/* NOTREACHED */
}
/**************************************************************/
/**************************************************************/
/**************************************************************/
/**************************************************************/

/* Copyright (C) 1991 Free Software Foundation, Inc.
This file is part of the GNU C Library.

The GNU C Library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

The GNU C Library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with the GNU C Library; see the file COPYING.LIB.  If
not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA.  */

#define FLT_RADIX	2 /* JRS -- added */
#define DBL_MANT_DIG	24 /* JRS -- added */
#define DBL_DIG		6 /* JRS -- added */
#define DBL_MAX_10_EXP	+38 /* JRS -- added */

#include <ansidecl.h>

#ifdef	__GNUC__

static __inline int
DEFUN(is_neg_68881, (num), register LONG_DOUBLE num)
{
  if (num == 0.0)
    {
      char negative_p;
      asm("ftst%.x %1\n"
	  "fslt %0" : "=g" (negative_p) : "f" (num));
      return negative_p;
    }
  return num < 0.0;
}

#define	IS_NEGATIVE(num)	is_neg_68881(num)

#endif

/* -- JRS included here
  #include <../sysdeps/ieee754/printf_fp.c>
*/

/* Copyright (C) 1991, 1992 Free Software Foundation, Inc.
This file is part of the GNU C Library.

The GNU C Library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

The GNU C Library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with the GNU C Library; see the file COPYING.LIB.  If
not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA.  */

#include <ansidecl.h>
#include <stdio.h>
#include <math.h>
#include "ieee754.h"

#ifndef	HANDLE_SPECIAL

#ifdef	__GNUC__
 __inline
#endif
static int
DEFUN(special_ieee754, (fp, info, fpnum),
      FILE *fp AND CONST struct printf_info *info AND LONG_DOUBLE fpnum)
{
  int is_neg;
  CONST char *string;

  if (__isnan ((double) fpnum))
    {
      string = "NaN";
      is_neg = 0;
    }
  else if (__isinf ((double) fpnum))
    {
      string = "Inf";
      is_neg = fpnum < 0;
    }
  else
    return 0;

  {
    size_t cnt = 0;
    int width = info->prec > info->width ? info->prec : info->width;

    if (is_neg || info->showsign || info->space)
      --width;
    width -= 3;

    if (!info->left)
      while (width-- > 0)
	PUTC (' ');

    if (is_neg)
      PUTC ('-');
    else if (info->showsign)
      PUTC ('+');
    else if (info->space)
      PUTC (' ');

    {
      register size_t len = 3;
      while (len-- > 0)
	PUTC (*string++);
    }

    if (info->left)
      while (width-- > 0)
	PUTC (' ');

    return cnt;
  }
}

#define	HANDLE_SPECIAL(cnt, s, info, fpnum)				      \
  {									      \
    int more_cnt = special_ieee754 (s, info, fpnum);			      \
    if (more_cnt == -1)						      \
      return -1;							      \
    else if (more_cnt != 0)						      \
      return cnt + more_cnt;						      \
  }

#endif

/* Floating-point printing for `printf'.
   This is an implementation of a restricted form of the `Dragon4'
   algorithm described in "How to Print Floating-Point Numbers Accurately",
   by Guy L. Steele, Jr. and Jon L. White, presented at the ACM SIGPLAN '90
   Conference on Programming Language Design and Implementation.

Copyright (C) 1992, 1993 Free Software Foundation, Inc.
This file is part of the GNU C Library.

The GNU C Library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

The GNU C Library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with the GNU C Library; see the file COPYING.LIB.  If
not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA.  */

#include <ctype.h>
#include <stdio.h>
/*
#include <float.h>
*/
#include <math.h>
/* -- JRS not needed 
#include <stdarg.h>
*/
#include <stdlib.h>
#include <localeinfo.h>

#include <printf.h>

#if FLT_RADIX != 2

double
frexp (double f, int *e)
{
  #error "Don't know how to extract fraction and exponent from `double'."
}

#undef	ldexp
#ifdef	__GNUC__
inline
#endif
static double
ldexp (double f, int e)
{
  while (e > 0)
    {
      f *= FLT_RADIX;
      --e;
    }
  while (e < 0)
    {
      f /= FLT_RADIX;
      ++e;
    }
}

#endif


/*
int
DEFUN(__printf_fp, (fp, info, args),
      FILE *fp AND CONST struct printf_info *info AND va_list *args)
*/
int
DEFUN(__printf_fp, (fp, info, fpnum),
      FILE *fp AND CONST struct printf_info *info AND LONG_DOUBLE fpnum)
{
  int cnt = 0;

  /* Decimal point character.  */
/* JRS
  CONST char *CONST decimal = _numeric_info->decimal_point;
*/

  CONST char *CONST decimal = ".";

  /*LONG_DOUBLE fpnum;*/		/* Input.  */
  int is_neg;

  LONG_DOUBLE f;		/* Fraction.  */

  int e;			/* Base-2 exponent of the input.  */
  CONST int p = DBL_MANT_DIG;	/* Internal precision.  */
  LONG_DOUBLE scale, scale10;	/* Scale factor.  */
  LONG_DOUBLE loerr, hierr;	/* Potential error in the fraction.  */
  int k;			/* Digits to the left of the decimal point.  */
  int cutoff;			/* Where to stop generating digits.  */
  LONG_DOUBLE r, r2, r10;	/* Remainder.  */
  int roundup;
  int low, high;
  char digit;

  int j;

  char type = tolower (info->spec);
  int prec = info->prec;
  int width = info->width;

  /* This algorithm has the nice property of not needing a buffer.
     However, to get the padding right for %g format, we need to know
     the length of the number before printing it.  */

#ifndef	LDBL_DIG
#define	LDBL_DIG	DBL_DIG
#endif
#ifndef	LDBL_MAX_10_EXP
#define	LDBL_MAX_10_EXP	DBL_MAX_10_EXP
#endif

  char buf[64];

#if JRS
  char *buf = __alloca ((prec > LDBL_DIG ? prec : LDBL_DIG) +
			LDBL_MAX_10_EXP + 3); /* Dot, e, exp. sign.  */
#endif

  register char *bp = buf;
#define	put(c)	*bp++ = (c)

  /* Fetch the argument value.  */
/*
  if (info->is_long_double)
    fpnum = va_arg (*args, LONG_DOUBLE);
  else
    fpnum = (LONG_DOUBLE) va_arg (*args, double);
*/

#ifdef	HANDLE_SPECIAL
  /* Allow for machine-dependent (or floating point format-dependent) code.  */
  HANDLE_SPECIAL (cnt, fp, info, fpnum);
#endif

  is_neg = IS_NEGATIVE (fpnum);
  if (is_neg)
    fpnum = - fpnum;

  if (prec == -1)
    prec = 6;
  
  if (type == 'g')
    {
      if (prec == 0)
	prec = 1;

      if (fpnum != 0)
	{
	  if (fpnum < 1e-4)
	    type = 'e';
	  else
	    {
	      /* Is (int) floor (log10 (FPNUM)) >= PREC?  */
	      j = prec;
	      if (j > p)
		j = p;
	      f = 10;
	      while (--j > 0)
		{
		  f *= 10;
		  if (fpnum < f)
		    /* log10 (F) == floor (log10 (FPNUM)) + 1
		       log10 (FPNUM) == Number of iterations minus one.  */
		    break;
		}
	      if (j <= 0)
		/* We got all the way through the loop and F (i.e., 10**J)
		   never reached FPNUM, so we want to use %e format.  */
		type = 'e';
	    }
	}

      /* For 'g'/'G' format, the precision specifies "significant digits",
	 not digits to come after the decimal point.  */
      --prec;
    }

  if (fpnum == 0)
    /* Special case for zero.
       The general algorithm does not work for zero.  */
    {
      put ('0');
      if (tolower (info->spec) != 'g' || info->alt)
	{
	  if (prec > 0 || info->alt)
	    put (*decimal);
	  while (prec-- > 0)
	    put ('0');
	}
      if (type == 'e')
	{
	  put (info->spec);
	  put ('+');
	  put ('0');
	  put ('0');
	}
    }
  else
    {
      /* Split the number into a fraction and base-2 exponent.  */
      f = frexp (fpnum, &e);

      /* Scale the fractional part by the highest possible number of
	 significant bits of fraction.  We want to represent the
	 fractional part as a (very) large integer.  */
      f = ldexp (f, p);

      cutoff = -prec;

      roundup = 0;

      if (e > p)
	{
	  /* The exponent is bigger than the number of fractional digits.  */
	  r = ldexp (f, e - p);
	  scale = 1;
	  /* The number is (E - P) factors of two larger than
	     the fraction can represent; this is the potential error.  */
	  loerr = ldexp (1.0, e - p);
	}
      else
	{
	  /* The number of fractional digits is greater than the exponent.
	     Scale by the difference factors of two.  */
	  r = f;
	  scale = ldexp (1.0, p - e);
	  loerr = 1.0;
	}
      hierr = loerr;

      /* Fixup.  */

      if (f == ldexp (1.0, p - 1))
	{
	  /* Account for unequal gaps.  */
	  hierr = ldexp (hierr, 1);
	  r = ldexp (r, 1);
	  scale = ldexp (scale, 1);
	}

      scale10 = ceil (scale / 10.0);
      k = 0;
      while (r < scale10)
	{
	  --k;
	  r *= 10;
	  loerr *= 10;
	  hierr *= 10;
	}
      do
	{
	  r2 = 2 * r;

	  while (r2 + hierr >= 2 * scale)
	    {
	      scale *= 10;
	      ++k;
	    }

	  /* Perform any necessary adjustment of loerr and hierr to
	     take into account the formatting requirements.  */

	  if (type == 'e')
	    cutoff += k - 1;	/* CutOffMode == "relative".  */
	  /* Otherwise CutOffMode == "absolute".  */

	  {			/* CutOffAdjust.  */
	    int a = cutoff - k;
	    double y = scale;
	    while (a > 0)
	      {
		y *= 10;
		--a;
	      }
	    while (a < 0)
	      {
		y = ceil (y / 10);
		++a;
	      }
	    /* y == ceil (scale * pow (10.0, (double) (cutoff - k))) */
	    if (y > loerr)
	      loerr = y;
	    if (y > hierr)
	      {
		hierr = y;
		roundup = 1;
	      }
	  }			/* End CutOffAdjust.  */
	} while (r2 + hierr >= 2 * scale);

      /* End Fixup.  */

      /* First digit.  */
      --k;
      r10 = r * 10;
      digit = '0' + (unsigned int) floor (r10 / scale);
      r = fmod (r10, scale);
      loerr *= 10;
      hierr *= 10;
      
      low = 2 * r < loerr;
      if (roundup)
	high = 2 * r >= (2 * scale) - hierr;
      else
	high = 2 * r > (2 * scale) - hierr;
      
      if (low || high || k == cutoff)
	{
	  if ((high && !low) || (2 * r > scale))
	    ++digit;
	}

      if (type == 'e')
	{
	  /* Exponential notation.  */

	  int expt = k;		/* Base-10 exponent.  */
	  int expt_neg;

	  expt_neg = k < 0;
	  if (expt_neg)
	    expt = - expt;

	  /* Find the magnitude of the exponent.  */
	  j = 10;
	  while (j <= expt)
	    j *= 10;

	  /* Write the first digit.  */
	  put (digit);

	  if (low || high || k == cutoff)
	    {
	      if ((tolower (info->spec) != 'g' && prec > 0) || info->alt)
		put (*decimal);
	    }
	  else
	    {
	      put (*decimal);

	      /* Remaining digits.  */
	      while (1)
		{
		  --k;
		  r10 = r * 10;
		  digit = '0' + (unsigned int) floor (r10 / scale);
		  r = fmod (r10, scale);
		  loerr *= 10;
		  hierr *= 10;
		      
		  low = 2 * r < loerr;
		  if (roundup)
		    high = 2 * r >= (2 * scale) - hierr;
		  else
		    high = 2 * r > (2 * scale) - hierr;
		      
		  if (low || high || k == cutoff)
		    {
		      if ((high && !low) || (2 * r > scale))
			++digit;
		      put (digit);
		      break;
		    }
		      
		  put (digit);
		}
	    }

	  if (tolower (info->spec) != 'g' || info->alt)
	    /* Pad with zeros.  */
	    while (--k >= cutoff)
	      put ('0');

	  /* Write the exponent.  */
	  put (isupper (info->spec) ? 'E' : 'e');
	  put (expt_neg ? '-' : '+');
	  if (expt < 10)
	    /* Exponent always has at least two digits.  */
	    put ('0');
	  do
	    {
	      j /= 10;
	      put ('0' + (expt / j));
	      expt %= j;
	    }
	  while (j > 1);
	}
      else
	{
	  /* Decimal fraction notation.  */

	  if (k < 0)
	    {
	      put ('0');
	      if (prec > 0 || info->alt)
		put (*decimal);

	      /* Write leading fractional zeros.  */
	      j = 0;
	      while (--j > k)
		put ('0');
	    }

	  if (low || high || k == cutoff)
	    put (digit);
	  else
	    while (1)
	      {
		put (digit);
		
		--k;
		r10 = r * 10;
		digit = '0' + (unsigned int) floor (r10 / scale);
		r = fmod (r10, scale);
		loerr *= 10;
		hierr *= 10;
		
		low = 2 * r < loerr;
		if (roundup)
		  high = 2 * r >= (2 * scale) - hierr;
		else
		  high = 2 * r > (2 * scale) - hierr;
		
		if (k == -1)
		  put (*decimal);

		if (low || high || k == cutoff)
		  {
		    if ((high && !low) || (2 * r > scale))
		      ++digit;
		    put (digit);
		    break;
		  }
	      }
	  while (k > 0)
	    {
	      put ('0');
	      --k;
	    }
	  if ((type != 'g' && prec > 0) || info->alt)
	    {
	      if (k == 0)
		put (*decimal);
	      while (k-- > -prec)
		put ('0');
	    }
	}
    }

#undef	put

  /* The number is all converted in BUF.
     Now write it with sign and appropriate padding.  */

  if (is_neg || info->showsign || info->space)
    --width;

  width -= bp - buf;

  if (!info->left && info->pad == ' ')
    /* Pad with spaces on the left.  */
    while (width-- > 0)
      PUTC (' ');

  /* Write the sign.  */
  if (is_neg)
    PUTC ('-');
  else if (info->showsign)
    PUTC ('+');
  else if (info->space)
    PUTC (' ');

  if (!info->left && info->pad == '0')
    /* Pad with zeros on the left.  */
    while (width-- > 0)
      PUTC ('0');

/* JRS */
{ int i; 

  for ( i = 0 ; i < (bp - buf) ; i++ )
    PUTC( buf[ i ] );

}
/* JRS */
#if 0
/* fwrite() does not work with this printf!!! -JRS */
  if (fwrite (buf, bp - buf, 1, fp) != 1)
    return -1;
#endif
  cnt += bp - buf;

  if (info->left)
    /* Pad with spaces on the right.  */
    while (width-- > 0)
      PUTC (' ');

  return cnt;
}
