#  entry.s
#
#  This file contains the entry point for the application.
#  The name of this entry point is compiler dependent.
#  It jumps to the BSP which is responsible for performing
#  all initialization.
#
#
#  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
#  On-Line Applications Research Corporation (OAR).
#  All rights assigned to U.S. Government, 1994.
#
#  This material may be reproduced by or for the U.S. Government pursuant
#  to the copyright license under the clause at DFARS 252.227-7013.  This
#  notice must appear in all copies of this file and its derivatives.
#
#  $Id$
#

include(system.eq)

#
#  A Descriptor table register has the following format
#

.set DTR_LIMIT,   0    # offset of two byte limit
.set DTR_BASE,    2    # offset of four byte base address
.set DTR_SIZE,    6    # size of DTR register

BEGIN_DATA
        EXTERN(Do_Load_IDT)
        EXTERN(Do_Load_GDT)
END_DATA 

        BEGIN_CODE

        .global start                   # GNU default entry point

        EXTERN(bsp_start)
        EXTERN(_load_segments)
        EXTERN(_exit)

start:
        nop
        cli                             # DISABLE INTERRUPTS!!!
#
#  Load the segment registers
#
#  NOTE: Upon return, gs will contain the segment descriptor for
#        a segment which maps directly to all of physical memory.
#
        jmp     _load_segments          # load board dependent segments

#
#  Set up the stack
#

PUBLIC_LABEL(_establish_stack)
        movl    $_end,%eax              # eax = end of bss/start of heap
        addl    $heap_size,%eax         # eax = end of heap
        movl    %eax,_stack_start       # Save for brk() routine
        addl    $stack_size,%eax        # make room for stack
        andl    $0xffffffc0,%eax        # align it on 16 byte boundary
        movl    %eax,%esp               # set stack pointer
        movl    %eax,%ebp               # set base pointer
#
#  Zero out the BSS segment
#
zero_bss:
        cld                             # make direction flag count up
        movl    $_end,%ecx              # find end of .bss
        movl    $_bss_start,%edi        # edi = beginning of .bss
        subl    %edi,%ecx               # ecx = size of .bss in bytes
        shll    %ecx                    # size of .bss in longs
        xorl    %eax,%eax               # value to clear out memory
        repne                           # while ecx != 0
        stosl                           #   clear a long in the bss

#
#  Copy the Global Descriptor Table to our space
#

        sgdt    _Original_GDTR          # save original GDT
        movzwl  _Original_GDTR+DTR_LIMIT,%ecx # size of GDT in bytes; limit is
                                        #   8192 entries * 8 bytes per

        # make ds:esi point to the original GDT

        movl    _Original_GDTR+DTR_BASE,%esi
        push    %ds                     # save ds
        movw    %gs,%ax
        movw    %ax,%ds 

        # make es:edi point to the new (our copy) GDT
        movl    $_Global_descriptor_table,%edi

        rep
        movsb                            # copy the GDT (ds:esi -> es:edi)

        pop     %ds                      # restore ds
      
        # Build and load new contents of GDTR
        movw    _Original_GDTR+DTR_LIMIT,%ecx # set new limit
        movw    %cx,_New_GDTR+DTR_LIMIT

        push    $_Global_descriptor_table
        push    %es
        call    _Logical_to_physical
        addl    $6,%esp
        movl    %eax,_New_GDTR+DTR_BASE      # set new base

        cmpb    $0,_Do_Load_GDT          # Should the new GDT be loaded?
        je      no_gdt_load              # NO, then branch
        lgdt    _New_GDTR                # load the new GDT
no_gdt_load:

#
#  Copy the Interrupt Descriptor Table to our space
#

        sidt    _Original_IDTR          # save original IDT
        movzwl  _Original_IDTR+DTR_LIMIT,%ecx # size of IDT in bytes; limit is
                                        #   256 entries * 8 bytes per
 

        # make ds:esi point to the original IDT
        movl    _Original_IDTR+DTR_BASE,%esi

        push    %ds                     # save ds
        movw    %gs,%ax
        movw    %ax,%ds 

        # make es:edi point to the new (our copy) IDT
        movl    $_Interrupt_descriptor_table,%edi

        rep
        movsb                            # copy the IDT (ds:esi -> es:edi)
        pop     %ds                      # restore ds

        # Build and load new contents of IDTR
        movw    _Original_IDTR+DTR_LIMIT,%ecx # set new limit
        movw    %cx,_New_IDTR+DTR_LIMIT

        push    $_Interrupt_descriptor_table
        push    %es
        call    _Logical_to_physical
        addl    $6,%esp
        movl    %eax,_New_IDTR+DTR_BASE      # set new base

        cmpb    $0,_Do_Load_IDT          # Should the new IDT be loaded?
        je      no_idt_load              # NO, then branch
        lidt    _New_IDTR                # load the new IDT
no_idt_load:

#
#  Initialize the i387.
#
#  Using the NO WAIT form of the instruction insures that 
#  if it is not present the board will not lock up or get an
#  exception.
#

        fninit                           # MUST USE NO-WAIT FORM

#
#  Initialize the Standard Libary
#
        call    __HL_init               # initialize STDIO

#
#  Transfer control to User's Board Support Package
#
        call    _bsp_start

        jmp     __exit
#
#  Clean up
#

        EXTERN(return_to_monitor)

PUBLIC_LABEL(_Bsp_cleanup)
        cmpb    $0,_Do_Load_IDT          # Was the new IDT loaded?
        je      no_idt_restore           # NO, then branch
        lidt    _Original_IDTR           # restore the new IDT
no_idt_restore:

        cmpb    $0,_Do_Load_GDT          # Was the new GDT loaded?
        je      no_gdt_restore           # NO, then branch
        lgdt    _Original_GDTR           # restore the new GDT
no_gdt_restore:
        jmp     _return_to_monitor

#
#  void *Logical_to_physical( 
#     rtems_unsigned16  segment,
#     void             *address
#  );
#
#  Returns thirty-two bit physical address for segment:address.
#

.set SEGMENT_ARG, 4
.set ADDRESS_ARG, 8

PUBLIC_LABEL(_Logical_to_physical)

        xorl    %eax,%eax                # clear eax
        movzwl  SEGMENT_ARG(%esp),%ecx   # ecx = segment value
        movl    $_Global_descriptor_table,%edx # edx = address of our GDT
        addl    %ecx,%edx                # edx = address of desired entry
        movb    7(%edx),%ah              # ah = base 31:24
        movb    4(%edx),%al              # al = base 23:16
        shll    $16,%eax                 # move ax into correct bits
        movw    2(%edx),%ax              # ax = base 0:15
        movl    ADDRESS_ARG(%esp),%ecx   # ecx = address to convert
        addl    %eax,%ecx                # ecx = physical address equivalent
        movl    %ecx,%eax                # eax = ecx
        ret
       
#
#  void *Physical_to_logical( 
#     rtems_unsigned16  segment,
#     void             *address
#  );
#
#  Returns thirty-two bit physical address for segment:address.
#

#.set SEGMENT_ARG, 4
#.set ADDRESS_ARG, 8   -- use sets from above

PUBLIC_LABEL(_Physical_to_logical)

        xorl    %eax,%eax                # clear eax
        movzwl  SEGMENT_ARG(%esp),%ecx   # ecx = segment value
        movl    $_Global_descriptor_table,%edx # edx = address of our GDT
        addl    %ecx,%edx                # edx = address of desired entry
        movb    7(%edx),%ah              # ah = base 31:24
        movb    4(%edx),%al              # al = base 23:16
        shll    $16,%eax                 # move ax into correct bits
        movw    2(%edx),%ax              # ax = base 0:15
        movl    ADDRESS_ARG(%esp),%ecx   # ecx = address to convert
        subl    %eax,%ecx                # ecx = logical address equivalent
        movl    %ecx,%eax                # eax = ecx
        ret
       
END_CODE

BEGIN_DATA

        .align  2
PUBLIC_LABEL(_start_frame)
        .long  0
PUBLIC_LABEL(_stack_start)
        .long  0

END_DATA

BEGIN_BSS

        PUBLIC(heap_size)
.set    heap_size,0x2000

        PUBLIC(stack_size)
.set    stack_size,0x1000

PUBLIC_SPACE(_Interrupt_descriptor_table,256*8)
PUBLIC_SPACE(_Original_IDTR,DTR_SIZE)
PUBLIC_SPACE(_New_IDTR,DTR_SIZE)

PUBLIC_SPACE(_Global_descriptor_table,8192*8)
PUBLIC_SPACE(_Original_GDTR,DTR_SIZE)
PUBLIC_SPACE(_New_GDTR,DTR_SIZE)

PUBLIC_SPACE(_Physical_base_of_ds,4)
PUBLIC_SPACE(_Physical_base_of_cs,4)

END_BSS

END
