/*  Test_task
 *
 *  This task is used for three test tasks.  It obtains its task id and
 *  based upon that id, performs certain actions.  
 *
 *       Task_1 delays 5 seconds and deletes itself.
 *       Task_2 delays 10 seconds and then loops until 
 *             deleted by the third task.
 *       Task 3 delays 15 seconds, then deletes task 2 and itself.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

rtems_task Test_task(
  rtems_task_argument argument
)
{
  rtems_status_code status;
  rtems_id          tid;
  rtems_time_of_day time;

  status = task_ident( SELF, SEARCH_ALL_NODES, &tid );
  directive_failed( status, "task_ident" );

  status = clock_get( &time );
  directive_failed( status, "clock_get" );

  put_name( Task_name[ task_number( tid ) ], FALSE );
  print_time( " - clock_get - ", &time, "\n" );

  status = task_wake_after( ( task_number( tid ) ) * 5 * TICKS_PER_SECOND );
  directive_failed( status, "task_wake_after" );

  status = clock_get( &time );
  directive_failed( status, "clock_get" );
  put_name( Task_name[ task_number( tid ) ], FALSE );
  print_time( " - clock_get - ", &time, "\n" );

  if ( task_number(tid) == 1 ) {          /* TASK 1 */
    put_name( Task_name[ 1 ], FALSE );
    printf( " - deleting self\n" );
    status = task_delete( SELF );
    directive_failed( status, "task_delete of SELF" );
  } 
  else if ( task_number(tid) == 2 ) {     /* TASK 2 */
    put_name( Task_name[ 2 ], FALSE );
    printf( " - waiting to be deleted by " );
    put_name( Task_name[ 3 ], TRUE );
    while ( FOREVER );
  } 
  else {                                  /* TASK 3 */
    put_name( Task_name[ 3 ], FALSE );
    printf( " - getting TID of " );
    put_name( Task_name[ 2 ], TRUE );
    status = task_ident( Task_name[ 2 ], SEARCH_ALL_NODES, &tid );
    directive_failed( status, "task_ident" );

    put_name( Task_name[ 3 ], FALSE );
    printf( " - deleting " );
    put_name( Task_name[ 2 ], TRUE );
    status = task_delete( tid );
    directive_failed( status, "task_delete of Task 2" );

    puts( "*** END OF TEST 1 ***" );
    exit( 0 );
  }
}
