/*  Init
 *
 *  This routine is the initialization routine for this test program.
 *  Other than creating all objects needed by this test, if this routine
 *  is running on node one, it acquires a global semaphore to 
 *  force all other tasks to pend.  If running on node two, this task
 *  sleeps for a while, and then deletes two local tasks which are 
 *  waiting on a remote message queue or a semaphore.
 *  This routine is the initialization task for this test program.
 *  It is a user initialization task and has the responsibility for creating
 *  and starting the tasks that make up the test.  If the time of day
 *  clock is required for the test, it should also be set to a known
 *  value by this function.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#undef EXTERN
#define EXTERN
#include "shm.h"
#include "conftbl.h"
#include "gvar.h"

int Shm_statistics = 0;

rtems_task Init(
  rtems_task_argument argument
)
{
  rtems_status_code status;

  printf( 
   "\n\n*** TEST 10 -- NODE %d ***\n", 
   Multiprocessing_configuration.node
  );

  Task_name[ 1 ] =  build_name( 'T', 'A', '1', ' ' );
  Task_name[ 2 ] =  build_name( 'T', 'A', '2', ' ' );
  Task_name[ 3 ] =  build_name( 'S', 'A', '3', ' ' );

  Queue_name[ 1 ] = build_name( 'M', 'S', 'G', ' ' );

  Semaphore_name[ 1 ] = build_name( 'S', 'E', 'M', ' ' );

  if ( Multiprocessing_configuration.node == 1 ) {
    puts( "Creating Message Queue (Global)" );
    status = message_queue_create( 
      Queue_name[ 1 ], 
      3, 
      GLOBAL|LIMIT, 
      &Queue_id[ 1 ] 
    );
    directive_failed( status, "message_queue_create" );

    puts( "Creating Semaphore (Global)" );
    status = semaphore_create( 
      Semaphore_name[ 1 ], 
      0, 
      GLOBAL | PRIORITY, 
      &Semaphore_id[ 1 ] 
    );
    directive_failed( status, "semaphore_create" );

    status = task_wake_after( 10 * TICKS_PER_SECOND );
    directive_failed( status, "task_wake_after" );

  } else {

    puts( "Creating Test_task 1 (local)" );
    status = task_create( 
      Task_name[ 1 ], 
      1, 
      1024, 
      TIMESLICE, 
      DEFAULT_ATTRIBUTES, 
      &Task_id[ 1 ] 
    );
    directive_failed( status, "task_create" );

    puts( "Starting Test_task 1 (local)" );
    status = task_start( Task_id[ 1 ], Test_task1, 0 );
    directive_failed( status, "task_start" );

    puts( "Creating Test_task 2 (local)" );
    status = task_create( 
      Task_name[ 2 ], 
      1, 
      1024, 
      TIMESLICE, 
      DEFAULT_ATTRIBUTES, 
      &Task_id[ 2 ] 
    );
    directive_failed( status, "task_create" );

    puts( "Starting Test_task 2 (local)" );
    status = task_start( Task_id[ 2 ], Test_task2, 0 );
    directive_failed( status, "task_start" );

    puts( "Creating Test_task 3 (local)" );
    status = task_create( 
      Task_name[ 3 ], 
      1, 
      1024, 
      TIMESLICE, 
      DEFAULT_ATTRIBUTES, 
      &Task_id[ 3 ] 
    );
    directive_failed( status, "task_create" );

    puts( "Starting Test_task 3 (local)" );
    status = task_start( Task_id[ 3 ], Test_task2, 0 );
    directive_failed( status, "task_start" );

    puts( "Sleeping for 1 seconds ..." );
    status = task_wake_after( TICKS_PER_SECOND );
    directive_failed( status, "task_wake_after" );

    puts( "Deleting Test_task2" );
    status = task_delete( Task_id[ 2 ] );
    directive_failed( status, "task_delete of Task 2" );

    puts( "Deleting Test_task1" );
    status = task_delete( Task_id[ 1 ] );
    directive_failed( status, "task_delete of Task 1" );

    puts( "Restarting Test_task3" );
    status = task_restart( Task_id[ 3 ], 1 );
    directive_failed( status, "task_restart of Task 3" );

  } 
  puts( "*** END OF TEST 10 ***" );
  exit( 0 );
}
