/*  Init
 *
 *  This routine is the initialization task for this test program.
 *  It is a user initialization task and has the responsibility for creating
 *  and starting the tasks that make up the test.  If the time of day
 *  clock is required for the test, it should also be set to a known
 *  value by this function.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#undef EXTERN
#define EXTERN
#include "shm.h"
#include "conftbl.h"
#include "gvar.h"

int Shm_statistics = 0;

rtems_task Init(
  rtems_task_argument argument
)
{
  rtems_status_code status;

  printf( 
    "\n\n*** TEST 13 -- NODE %d ***\n", 
    Multiprocessing_configuration.node
  );

  Task_name[ 1 ] = build_name( '1', '1', '1', ' ' );
  Task_name[ 2 ] = build_name( '2', '2', '2', ' ' );

  Queue_name[ 1 ] = build_name( 'M', 'S', 'G', ' ' );

  Semaphore_name[ 1 ] = build_name( 'S', 'E', 'M', ' ' );

  if ( Multiprocessing_configuration.node == 1 ) {
    puts( "Creating Message Queue (Global)" );
    status = message_queue_create( 
      Queue_name[ 1 ], 
      3, 
      GLOBAL|LIMIT, 
      &Queue_id[ 1 ] 
    );
    directive_failed( status, "message_queue_create" );

    puts( "Creating Semaphore (Global)" );
    status = semaphore_create( 
      Semaphore_name[ 1 ], 
      1, 
      GLOBAL | PRIORITY, 
      &Semaphore_id[ 1 ] 
    );
    directive_failed( status, "semaphore_create" );

    status = semaphore_obtain( 
      Semaphore_id[ 1 ], 
      DEFAULT_OPTIONS, 
      NO_TIMEOUT 
    );
    directive_failed( status, "semaphore_obtain" );
  }

  puts( "Creating Test_task 1 (local)" );
  status = task_create( 
    Task_name[ 1 ], 
    1, 
    1024, 
    TIMESLICE, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 1 ] 
  );
  directive_failed( status, "task_create" );

  puts( "Starting Test_task 1 (local)" );
  status = task_start( Task_id[ 1 ], Test_task1, 0 );
  directive_failed( status, "task_start" );

  puts( "Creating Test_task 2 (local)" );
  status = task_create( 
    Task_name[ 2 ], 
    1, 
    1024, 
    TIMESLICE, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 2 ] 
  );
  directive_failed( status, "task_create" );

  puts( "Starting Test_task 2 (local)" );
  status = task_start( Task_id[ 2 ], Test_task2, 0 );
  directive_failed( status, "task_start" );

  if ( Multiprocessing_configuration.node == 1 ) {
    status = task_wake_after( 5 * TICKS_PER_SECOND );
    directive_failed( status, "task_wake_after" );

    puts( "*** END OF TEST 13 ***" );
    exit( 0 );
  } 
  puts( "Deleting initialization task" );
  status = task_delete( SELF );
  directive_failed( status, "task_delete of SELF" );
}
