/*  Delayed_events_task
 *
 *  This task continuously sends itself events at one tick
 *  intervals.
 * 
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

#define E_COUNT 100

rtems_task Delayed_events_task(
  rtems_task_argument argument
) 
{
  rtems_unsigned32  count;
  rtems_unsigned32  previous_mode;
  rtems_id          timer_id;
  rtems_status_code status;
  rtems_event_set   events;

  status = task_mode( 
    PREEMPT | TIMESLICE, 
    PREEMPT_MASK | TIMESLICE_MASK, 
    &previous_mode 
  );
  directive_failed( status, "task_mode" );

  status = timer_create(
    build_name( 'T', 'M', '1', ' ' ),
    &timer_id
  );
  directive_failed( status, "timer_create" );

  while ( FOREVER ) {
    for ( count=E_COUNT ; count ; count-- ) {
      status = timer_fire_after( timer_id, 1, Delayed_send_event, NULL );
      directive_failed( status, "timer_reset" );

      status = event_receive( 
        EVENT_16, 
        DEFAULT_OPTIONS, 
        NO_TIMEOUT, 
        &events 
      );
      directive_failed( status, "event_receive" );

    }
    put_dot('.');
  }
}
