/*  Init
 *
 *  This routine is the initialization task for this test program.
 *  It is a user initialization task and has the responsibility for creating
 *  and starting the tasks that make up the test.  If the time of day
 *  clock is required for the test, it should also be set to a known
 *  value by this function.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#undef EXTERN
#define EXTERN
#include "shm.h"
#include "conftbl.h"
#include "gvar.h"

int  Shm_statistics = 1;

rtems_task Init(
  rtems_task_argument argument
)
{
  rtems_status_code   status;
  rtems_task_priority previous_priority;

  printf(
    "\n\n*** TEST 14 -- NODE %d ***\n",
    Multiprocessing_configuration.node
  );

  Task_name[ 1 ] = build_name( '1', '1', '1', ' ' );
  Task_name[ 2 ] = build_name( '2', '2', '2', ' ' );

  Queue_task_name[ 1 ] = build_name( 'M', 'T', '1', ' ' );
  Queue_task_name[ 2 ] = build_name( 'M', 'T', '2', ' ' );

  Partition_task_name[ 1 ] = build_name( 'P', 'T', '1', ' ' );
  Partition_task_name[ 2 ] = build_name( 'P', 'T', '2', ' ' );

  Semaphore_task_name[ 1 ] = build_name( 'S', 'M', '1', ' ' );
  Semaphore_task_name[ 2 ] = build_name( 'S', 'M', '2', ' ' );

  Semaphore_name[ 1 ] =  build_name( 'S', 'E', 'M', ' ' );

  Queue_name[ 1 ] = build_name( 'M', 'S', 'G', ' ' );

  Partition_name[ 1 ] = build_name( 'P', 'A', 'R', ' ' );

  if ( Multiprocessing_configuration.node == 1 ) {
    puts( "Creating Semaphore (Global)" );
    status = semaphore_create( 
      Semaphore_name[ 1 ], 
      1, 
      GLOBAL, 
      &Semaphore_id[ 1 ] 
    );
    directive_failed( status, "semaphore_create" );

    puts( "Creating Message Queue (Global)" );
    status = message_queue_create( 
      Queue_name[ 1 ], 
      1, 
      GLOBAL, 
      &Queue_id[ 1 ]
    );
    directive_failed( status, "message_queue_create" );

    puts( "Creating Partition (Global)" );
    status = partition_create( 
      Partition_name[ 1 ], 
      (void *)0x30000, 
      0x8000, 
      0x3800, 
      GLOBAL, 
      &Partition_id[ 1 ] 
    );
    directive_failed( status, "partition_create" );
  } 

  puts( "Creating Event task (Global)" );
  status = task_create( 
    Task_name[ Multiprocessing_configuration.node ], 
    2, 
    2048, 
    TIMESLICE, 
    GLOBAL, 
    &Event_task_id[ 1 ] 
  );
  directive_failed( status, "task_create" );

  puts( "Starting Event task (Global)" );
  status = task_start( Event_task_id[ 1 ], Test_task, 0 );
  directive_failed( status, "task_start" );

  puts( "Creating Semaphore task (Global)" );
  status = task_create( 
    Semaphore_task_name[ Multiprocessing_configuration.node ], 
    2, 
    2048, 
    TIMESLICE, 
    GLOBAL, 
    &Semaphore_task_id[ 1 ] 
  );
  directive_failed( status, "task_create" );

  puts( "Starting Semaphore task (Global)" );
  status = task_start( Semaphore_task_id[ 1 ], Semaphore_task, 0 );
  directive_failed( status, "task_start" );

  puts( "Creating Message Queue task (Global)" );
  status = task_create( 
    Queue_task_name[ Multiprocessing_configuration.node ], 
    2, 
    2048, 
    TIMESLICE, 
    GLOBAL, 
    &Queue_task_id[ 1 ] 
  );
  directive_failed( status, "task_create" );

  puts( "Starting Message Queue task (Global)" );
  status = task_start( Queue_task_id[ 1 ], Message_queue_task, 0 );
  directive_failed( status, "task_start" );

  puts( "Creating Partition task (Global)" );
  status = task_create( 
    Partition_task_name[ Multiprocessing_configuration.node ], 
    2, 
    2048, 
    TIMESLICE, 
    GLOBAL, 
    &Partition_task_id[ 1 ] 
  );
  directive_failed( status, "task_create" );

  puts( "Starting Partition task (Global)" );
  status = task_start( Partition_task_id[ 1 ], Partition_task, 0 );
  directive_failed( status, "task_start" );

  status = task_set_priority( SELF, 2, &previous_priority );
  directive_failed( status, "task_set_priority" );

  Delayed_events_task( 0 );
}
