/*  Message_queue_task
 *
 *  This task continuously sends messages to and receives messages from 
 *  a global message queue.    The message buffer is viewed as an array
 *  of two sixty-four bit counts which are incremented when a message is
 *  received.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

#define MSG_COUNT 10000

rtems_task Message_queue_task(
  rtems_task_argument argument
)
{
  rtems_status_code  status;
  rtems_unsigned32   count;
  rtems_unsigned32   yield_count;
  rtems_unsigned32  *buf_count;
  rtems_unsigned32  *ov_flow;

  Msg_buffer1[0] = 0;
  Msg_buffer1[1] = 0;
  Msg_buffer1[2] = 0;
  Msg_buffer1[3] = 0;

  puts( "Getting ID of msg queue" );
  while ( FOREVER ) {
    status = message_queue_ident( 
      Queue_name[ 1 ], 
      SEARCH_ALL_NODES, 
      &Queue_id[ 1 ] 
    );
    if ( status == SUCCESSFUL )
      break;
    puts( "message_queue_ident FAILED!!" );
  }
   
  if ( Multiprocessing_configuration.node == 1 ) {
      status = message_queue_send( 
        Queue_id[ 1 ], 
        (long (*)[4])Msg_buffer1 
      );
      directive_failed( status, "message_queue_send" );
      ov_flow   = &Msg_buffer1[0];
      buf_count = &Msg_buffer1[1];
  } else {
      ov_flow   = &Msg_buffer1[2];
      buf_count = &Msg_buffer1[3];
  }

  while ( FOREVER ) {
    yield_count = 100;

    for ( count=MSG_COUNT ; count ; count-- ) {
      status = message_queue_receive( 
        Queue_id[ 1 ], 
        (long (*)[4])Msg_buffer1, 
        DEFAULT_OPTIONS, 
        NO_TIMEOUT 
      );
      directive_failed( status, "message_queue_receive" );

      if ( *buf_count == (rtems_unsigned32)0xffffffff ) 
         (*ov_flow)++;
      (*buf_count)++;
      status = message_queue_send( 
        Queue_id[ 1 ], 
        (long (*)[4])Msg_buffer1 
      );
      directive_failed( status, "message_queue_send" );

      if ( Multiprocessing_configuration.node == 1 && --yield_count == 0 ) {
        status = task_wake_after( YIELD_PROCESSOR );
        directive_failed( status, "task_wake_after" );

        yield_count = 100;
      }
    }
    put_dot( 'm' );
  }
}
