/*  Partition_task
 *
 *  This task continuously gets a buffer from and returns that buffer
 *  to a global partition.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

#define PT_COUNT 1000

extern rtems_multiprocessing_table Multiprocessing_configuration;

rtems_task Partition_task(
  rtems_task_argument argument
)
{
  rtems_unsigned32   count;
  rtems_status_code  status;
  rtems_unsigned8   *bufaddr;

  puts( "Getting ID of partition" );
  while ( FOREVER ) {
    status = partition_ident( 
      Partition_name[ 1 ], 
      SEARCH_ALL_NODES, 
      &Partition_id[ 1 ] 
    );

    if ( status == SUCCESSFUL )
      break;

    puts( "partition_ident FAILED!!" );
  }
   
  while ( FOREVER ) {
    for ( count=PT_COUNT ; count ; count-- ) {
      status = partition_get_buffer( Partition_id[ 1 ], &bufaddr );
      directive_failed( status, "partition_get_buffer" );
   
      status = partition_return_buffer( Partition_id[ 1 ], bufaddr );
      directive_failed( status, "partition_return_buffer" );

      if ( Multiprocessing_configuration.node == 1 ) {
        status = task_wake_after( YIELD_PROCESSOR ); 
        directive_failed( status, "task_wake_after" );
      }
    }
    put_dot( 'p' );
  }
}
