/*  Sm_test_task
 *
 *  This task continuously obtains and releases a global semaphore.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

#define PV_COUNT 10000

extern rtems_multiprocessing_table Multiprocessing_configuration;

rtems_task Semaphore_task(
  rtems_task_argument argument
)
{
  rtems_unsigned32  count;
  rtems_status_code status;
  rtems_unsigned32  yield_count;

  puts( "Getting SMID of semaphore" );
  while ( FOREVER ) {
    status = semaphore_ident( 
      Semaphore_name[ 1 ], 
      SEARCH_ALL_NODES, 
      &Semaphore_id[ 1 ] 
    );
    if ( status == SUCCESSFUL )
      break;
    puts( "semaphore_ident FAILED!!" );
  }

  while ( FOREVER ) {
    yield_count = 100;

    for ( count=PV_COUNT ; count ; count-- ) {
      status = semaphore_obtain( 
        Semaphore_id[ 1 ], 
        DEFAULT_OPTIONS, 
        NO_TIMEOUT 
      );
      directive_failed( status, "semaphore_obtain" );

      status = semaphore_release( Semaphore_id[ 1 ] );
      directive_failed( status, "semaphore_release" );

      if ( Multiprocessing_configuration.node == 1 && --yield_count == 0 ) {
        status = task_wake_after( YIELD_PROCESSOR );
        directive_failed( status, "task_wake_after" );

        yield_count = 100;
      }
    }
    put_dot( 's' );
  }
}
