/*  Test_task
 *
 *  This task tests the task_set_note directive on a remote task and that 
 *  errors are returned when attempting to delete, start, or restart 
 *  a remote task.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

extern rtems_multiprocessing_table Multiprocessing_configuration;

rtems_task Test_task(
  rtems_task_argument argument
)
{
  rtems_id          tid;
  rtems_status_code status;
  rtems_unsigned32  remote_node;
  rtems_id          remote_tid;
  rtems_id          test_tid;
  rtems_unsigned32  np_val;

  status = task_ident( SELF, SEARCH_ALL_NODES, &tid );

  remote_node = (Multiprocessing_configuration.node == 1) ? 2 : 1;
  printf( "Remote task's name is : " );
  put_name( Task_name[ remote_node ], TRUE );

  puts( "Getting TID of remote task (all nodes)" );
  status = task_ident( 
    Task_name[ remote_node ], 
    SEARCH_ALL_NODES, 
    &remote_tid 
  );
  directive_failed( status, "task_ident" );

  puts( "Getting TID of remote task (1 node)" );
  status = task_ident( Task_name[ remote_node ], remote_node, &test_tid );
  directive_failed( status, "task_ident" );

  if ( test_tid != remote_tid ) {
    puts( "task_ident tid's do not match!!" );
    fatal_error_occurred( status );
  }

  status = task_delete( remote_tid );
  fatal_directive_status( 
    status, 
    ILLEGAL_ON_REMOTE_OBJECT, 
    "task_delete of remote task" 
  );
  puts( "task_delete of remote task returned the correct error" );

  status = task_start( remote_tid, Test_task, 0 ); 
  fatal_directive_status( 
    status, 
    ILLEGAL_ON_REMOTE_OBJECT, 
    "task_start of remote task" 
  );
  puts( "task_start of remote task returned the correct error" );

  status = task_restart( remote_tid, 0 );
  fatal_directive_status( 
    status, 
    ILLEGAL_ON_REMOTE_OBJECT, 
    "task_restart of remote task" 
  );
  puts( "task_restart of remote task returned the correct error" );

  printf( 
   "Setting notepad %d of the remote task to %d\n", 
   get_node(tid), 
   get_node(tid) 
  );
  status = task_set_note( remote_tid, get_node(tid), get_node(tid) );
  directive_failed( status, "task_set_note" );

  puts( "Getting a notepad of the remote task" );
  status = task_get_note( remote_tid, get_node(tid), &np_val );
  directive_failed( status, "task_get_note" );

  if ( np_val == get_node(tid) )
    puts( "Remote notepad set and read correctly" );
  else
    printf( 
      "FAILURE!! Remote notepad was not set and read correctly (%d, %d)\n",
      np_val,
      get_node( tid )
    );

  status = task_wake_after( TICKS_PER_SECOND );
  directive_failed( status, "task_wake_after" );

  puts( "*** END OF TEST 2 ***" );
  exit( 0 );
}
