/*  Test_task
 *
 *  This task suspends and resumes a remote task.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

rtems_task Test_task(
  rtems_task_argument argument
)
{
  rtems_id          tid;
  rtems_status_code status;
  rtems_unsigned32  remote_node;
  rtems_id          remote_tid;
  rtems_event_set   events;

  status = task_ident( SELF, SEARCH_ALL_NODES, &tid );
  directive_failed( status, "task_ident" );

  puts( "Getting TID of remote task" );
  remote_node = (Multiprocessing_configuration.node == 1) ? 2 : 1;
  printf( "Remote task's name is : " );
  put_name( Task_name[ remote_node ], TRUE );

  status = task_ident( 
    Task_name[ remote_node ], 
    SEARCH_ALL_NODES, 
    &remote_tid 
  );
  directive_failed( status, "task_ident" );

  status = timer_fire_after( 
    Timer_id[ 1 ], 
    30 * TICKS_PER_SECOND,
    Delayed_send_event,
    NULL
  );
  directive_failed( status, "timer_fire_after" );
 
  if ( Multiprocessing_configuration.node == 1 )
    while ( event_receive( EVENT_16, NO_WAIT, NO_TIMEOUT, &events ) 
               != SUCCESSFUL) {
      status = task_wake_after( 5 * TICKS_PER_SECOND );
      directive_failed( status, "task_wake_after" );

      puts( "111 - Suspending remote task" );
      status = task_suspend( remote_tid );
      directive_failed( status, "task_suspend" );

      status = task_wake_after( 5 * TICKS_PER_SECOND );
      directive_failed( status, "task_wake_after" );

      puts( "111 - Resuming remote task" );
      status = task_resume( remote_tid ) ;
      directive_failed( status, "task_resume" );
    }
  else 
    while ( event_receive( EVENT_16, NO_WAIT, NO_TIMEOUT, &events ) 
               != SUCCESSFUL) {
      puts( "222 - Have I been suspended?????" );
      status = task_wake_after( TICKS_PER_SECOND/2 );
      directive_failed( status, "task_wake_after" );
    } 
  
  status = timer_fire_after( 
    Timer_id[ 1 ], 
    30 * TICKS_PER_SECOND,
    Delayed_send_event,
    NULL
  );
  directive_failed( status, "timer_fire_after" );
 
  if ( Multiprocessing_configuration.node == 2 ) {
    status = task_wake_after( 3 * TICKS_PER_SECOND );
    directive_failed( status, "task_wake_after" );

    while ( event_receive( EVENT_16, NO_WAIT, NO_TIMEOUT, &events ) 
               != SUCCESSFUL){
      status = task_wake_after( 5 * TICKS_PER_SECOND );
      directive_failed( status, "task_wake_after" );

      puts( "222 - Suspending remote task" );
      status = task_suspend( remote_tid );
      directive_failed( status, "task_suspend" );

      status = task_wake_after( 5 * TICKS_PER_SECOND );
      directive_failed( status, "task_wake_after" );

      puts( "222 - Resuming remote task" );
      status = task_resume( remote_tid ) ;
      directive_failed( status, "task_resume" );
    }
  }
  else 
    while ( event_receive( EVENT_16, NO_WAIT, NO_TIMEOUT, &events )
             != SUCCESSFUL){
      puts( "111 - Have I been suspended?????" );
      status = task_wake_after( TICKS_PER_SECOND/2 );
      directive_failed( status, "task_wake_after" );
    } 
  
  puts( "*** END OF TEST 3 ***" );
  exit( 0 );
}
