/*  Test_task
 *
 *  This task initializes the signal catcher, sends the first signal
 *  if running on the first node, and loops while waiting for signals.
 *
 *  NOTE: The signal catcher is not reentrant and hence NO_ASR must 
 *        be a part of its execution mode.
 * 
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

#define SIGNALS_PER_DOT 15

int dots;

rtems_task Test_task(
  rtems_task_argument argument
)
{
  rtems_status_code status;

  signal_caught = 0;
  signal_count  = 0;

  puts( "signal_catch: initializing signal catcher" );
  status = signal_catch( Process_asr, NO_ASR|NO_PREEMPT );
  directive_failed( status, "signal_catch" );

  if (Multiprocessing_configuration.node == 1) {
     remote_node = 2;
     remote_signal  = SIGNAL_18;
     expected_signal = SIGNAL_17;
  }
  else {
     remote_node = 1;
     remote_signal  = SIGNAL_17;
     expected_signal = SIGNAL_18;
  }
  puts_nocr( "Remote task's name is : " );
  put_name( Task_name[ remote_node ], TRUE );

  puts( "Getting TID of remote task" );
  status = task_ident( 
    Task_name[ remote_node ], 
    SEARCH_ALL_NODES, 
    &remote_tid 
  );
  directive_failed( status, "task_ident" );

  if ( Multiprocessing_configuration.node == 1 ) {
    puts( "Sending signal to remote task" );
    status = signal_send( remote_tid, remote_signal );
    directive_failed( status, "signal_send" );
  }

  for ( dots=0 ; dots != 160 ; ) {
    if ( signal_caught ) {
      signal_caught = 0;
      if ( ++signal_count >= SIGNALS_PER_DOT ) {
        signal_count = 0;
        put_dot( '.' ); 
        dots++;
      }
      status = signal_send( remote_tid, remote_signal );
      directive_failed( status, "signal_send" );
    }
  }
  puts( "*** END OF TEST 5 ***" );
  exit( 0 );
}
