/*  Test_task
 *
 *  This task tests global event operations.  If running on node one, it
 *  continuously sends events.   If running on node two, it continuously
 *  receives events.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

#define DOT_COUNT 25

rtems_task Test_task(
  rtems_task_argument argument
)
{
  rtems_status_code status;
  rtems_unsigned32  count;
  rtems_unsigned32  remote_node;
  rtems_id          remote_tid;
  rtems_event_set   event_out;
  rtems_event_set   event_for_this_iteration;

  remote_node = (Multiprocessing_configuration.node == 1) ? 2 : 1;
  puts_nocr( "Remote task's name is : " );
  put_name( Task_name[ remote_node ], TRUE );

  puts( "Getting TID of remote task" );
  status = task_ident( 
    Task_name[ remote_node ], 
    SEARCH_ALL_NODES, 
    &remote_tid 
  );
  directive_failed( status, "task_ident FAILED!!" );

  if ( Multiprocessing_configuration.node == 1 ) 
    puts( "Sending events to remote task" );
  else
    puts( "Receiving events from remote task" );

  for ( count = DOT_COUNT * 160 ; count ; count-- ) {
    event_for_this_iteration = EVENT_1 << (count % 32);

    if ( Multiprocessing_configuration.node == 1 ) {
      status = event_send( remote_tid, event_for_this_iteration );
      directive_failed( status, "event_send" );

      status = task_wake_after( 1 );
      directive_failed( status, "task_wake_after" );
    } else {
      status = event_receive( 
        event_for_this_iteration, 
        DEFAULT_OPTIONS, 
        NO_TIMEOUT,
        &event_out 
      );
      directive_failed( status, "event_receive" );
    }

    if ( (count % DOT_COUNT) == 0 )
      put_dot('.');
  }
  putchar( '\n' );

  if ( Multiprocessing_configuration.node == 2 ) {
    status = event_receive( 
      EVENT_16, 
      DEFAULT_OPTIONS, 
      1 * TICKS_PER_SECOND, 
      &event_out 
    );
    fatal_directive_status( status, TIMEOUT, "event_receive" );
    puts( "\nevent_receive - correctly returned TIMEOUT" );
  }
  puts( "*** END OF TEST 6 ***" );
  exit( 0 );
}
