/*  Test_task
 *
 *  This task tests global semaphore operations.  It verifies that 
 *  global semaphore errors are correctly returned.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

rtems_task Test_task(
  rtems_task_argument argument
)
{
  rtems_unsigned32  count;
  rtems_status_code status;

  puts( "Getting SMID of semaphore" );
  status = semaphore_ident( 
    Semaphore_name[ 1 ], 
    SEARCH_ALL_NODES, 
    &Semaphore_id[ 1 ] 
  );
  directive_failed( status, "semaphore_ident FAILED!!" );

  if ( Multiprocessing_configuration.node == 2 ) {
    status = semaphore_delete( Semaphore_id[ 1 ] );
    fatal_directive_status( 
      status, 
      ILLEGAL_ON_REMOTE_OBJECT,
      "semaphore_delete did not return ILLEGAL_ON_REMOTE_OBJECT" 
    );
    puts( 
      "semaphore_delete correctly returned ILLEGAL_ON_REMOTE_OBJECT"
    );
  }
  
  count = 0;            /* number of times node 1 releases semaphore */
  while ( FOREVER ) {
    put_dot( 'p' );
    status = semaphore_obtain( 
      Semaphore_id[ 1 ], 
      DEFAULT_OPTIONS, 
      NO_TIMEOUT 
    );
    if ( status != SUCCESSFUL ) {
      fatal_directive_status(
        status, 
        OBJECT_WAS_DELETED, 
        "semaphore_obtain"
      );
      puts( "\nGlobal semaphore deleted" );
      puts( "*** END OF TEST 8 ***" );
      exit( 0 );
    }

    if ( Multiprocessing_configuration.node == 1 && ++count == 1000 ) {
      status = task_wake_after( TICKS_PER_SECOND );
      directive_failed( status, "task_wake_after" );

      puts( "\nDeleting global semaphore" );
      status = semaphore_delete( Semaphore_id[ 1 ] );
      directive_failed( status, "semaphore_delete" );

      puts( "*** END OF TEST 8 ***" );
      exit( 0 );
    }
    else {
      put_dot( 'v' );
      status = semaphore_release( Semaphore_id[ 1 ] );
      directive_failed( status, "semaphore_release FAILED!!" );
    }
  }
}
