/*  Test_task
 *
 *  This task tests global message queue operations.  It also generates
 *  an error condition.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

char buffer1[16] = "123456789012345";
char buffer2[16] = "abcdefghijklmno";
char buffer3[16] = "ABCDEFGHIJKLMNO"; 
char buffer4[16] = "PQRSTUVWXYZ(){}";

rtems_task Test_task(
  rtems_task_argument argument
)
{
  rtems_status_code status;
  rtems_unsigned32  count;
  char              receive_buffer[16];

  status = task_wake_after( TICKS_PER_SECOND );
  directive_failed( status, "task_wake_after" );

  puts( "Getting QID of message queue" );
  status = message_queue_ident( 
    Queue_name[ 1 ], 
    SEARCH_ALL_NODES, 
    &Queue_id[ 1 ]
  );
  directive_failed( status, "message_queue_ident" );

  if ( Multiprocessing_configuration.node == 2 ) {
    status = message_queue_delete( Queue_id[ 1 ] );
    fatal_directive_status( 
      status, 
      ILLEGAL_ON_REMOTE_OBJECT, 
      "message_queue_delete" 
    );
    puts( 
   "message_queue_delete correctly returned ILLEGAL_ON_REMOTE_OBJECT"
    );
    
    Send_messages();
    Receive_messages();

    puts( "Flushing remote empty queue" );
    status = message_queue_flush( Queue_id[ 1 ], &count );
    directive_failed( status, "message_queue_flush" );
    printf( "%02d messages were flushed on the remote queue\n", count );

    puts( "Send messages to be flushed from remote queue" );
    status = message_queue_send( Queue_id[ 1 ], (long (*)[4])buffer1 );
    directive_failed( status, "message_queue_send" );

    puts( "Flushing remote queue" );
    status = message_queue_flush( Queue_id[ 1 ], &count );
    directive_failed( status, "message_queue_flush" );
    printf( "%02d messages were flushed on the remote queue\n", count );

    puts( "Waiting for message queue to be deleted" );
    status = message_queue_receive(
      Queue_id[ 1 ], 
      (long (*)[4])receive_buffer, 
      DEFAULT_OPTIONS, 
      NO_TIMEOUT
    );
    fatal_directive_status(
      status,
      OBJECT_WAS_DELETED,
      "message_queue_receive" 
    );
    puts( "\nGlobal message queue deleted" );
  } 
  else {                   /* node == 1 */
    Receive_messages();
    Send_messages();

    puts( "Delaying for 5 seconds" );
    status = task_wake_after( 5*TICKS_PER_SECOND );
    directive_failed( status, "task_wake_after" );

    puts( "Deleting Message queue" );
    status = message_queue_delete( Queue_id[ 1 ] );
    directive_failed( status, "message_queue_delete" );
  }

  puts( "*** END OF TEST 9 ***" );
  exit( 0 );
}
