/*  Init
 *
 *  This routine is the initialization task for this test program.
 *  It is a user initialization task and has the responsibility for creating
 *  and starting the tasks that make up the test.  If the time of day
 *  clock is required for the test, it should also be set to a known
 *  value by this function.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#undef EXTERN
#define EXTERN
#include "conftbl.h"
#include "gvar.h"

rtems_task Init(
  rtems_task_argument argument
)
{
  rtems_status_code status;

  puts( "\n\n*** TEST 11 ***" );

  Task_name[ 1 ] = build_name( 'T', 'A', '1', ' ' );
  Task_name[ 2 ] = build_name( 'T', 'A', '2', ' ' );

  status = task_create( 
    Task_name[ 1 ], 
    4, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 1 ] 
  );
  directive_failed( status, "task_create of TA1" );

  status = task_create( 
    Task_name[ 2 ], 
    4, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 2 ] 
  );
  directive_failed( status, "task_create of TA2" );

  status = task_start( Task_id[ 1 ], Task_1, 0 );
  directive_failed( status, "task_start of TA1" );

  status = task_start( Task_id[ 2 ], Task_2, 0 );
  directive_failed( status, "task_start of TA2" );

  Timer_name[ 1 ] = build_name( 'T', 'M', '1', ' ' );
  Timer_name[ 2 ] = build_name( 'T', 'M', '2', ' ' );
  Timer_name[ 3 ] = build_name( 'T', 'M', '3', ' ' );
  Timer_name[ 4 ] = build_name( 'T', 'M', '4', ' ' );
  Timer_name[ 5 ] = build_name( 'T', 'M', '5', ' ' );
  Timer_name[ 6 ] = build_name( 'T', 'M', '6', ' ' );

  status = timer_create( Timer_name[ 1 ], &Timer_id[ 1 ] );
  directive_failed( status, "timer_create of TM1" );

  status = timer_create( Timer_name[ 2 ], &Timer_id[ 2 ] );
  directive_failed( status, "timer_create of TM2" );

  status = timer_create( Timer_name[ 3 ], &Timer_id[ 3 ] );
  directive_failed( status, "timer_create of TM3" );

  status = timer_create( Timer_name[ 4 ], &Timer_id[ 4 ] );
  directive_failed( status, "timer_create of TM4" );

  status = timer_create( Timer_name[ 5 ], &Timer_id[ 5 ] );
  directive_failed( status, "timer_create of TM5" );

  status = timer_create( Timer_name[ 6 ], &Timer_id[ 6 ] );
  directive_failed( status, "timer_create of TM6" );

  status = task_delete( SELF );
  directive_failed( status, "task_delete of SELF" );
}
