/*  Task_1
 *
 *  This routine serves as a test task.  It verifies the event maager.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

rtems_task Task_1(
  rtems_task_argument argument
)
{
  rtems_event_set   eventout;
  rtems_time_of_day time;
  rtems_status_code status;
  rtems_unsigned32  index;

  puts( "TA1 - event_send - send EVENT_16 to TA2" );
  status = event_send( Task_id[ 2 ], EVENT_16 );
  directive_failed( status, "event_send" );

  puts( 
    "TA1 - event_receive - waiting forever on EVENT_14 and EVENT_15"
  );
  status = event_receive( 
    EVENT_14 | EVENT_15, 
    DEFAULT_OPTIONS, 
    NO_TIMEOUT, 
    &eventout 
  );
  directive_failed( status, "event_receive" );
  printf( "TA1 - EVENT_14 and EVENT_15 received - eventout => %08x\n",
           eventout
        );

  puts( "TA1 - event_send - send EVENT_18 to TA2" );
  status = event_send( Task_id[ 2 ], EVENT_18 );
  directive_failed( status, "event_send" );

  puts( 
    "TA1 - event_receive - waiting with 10 second timeout on EVENT_14"
  );
  status = event_receive( 
    EVENT_14, 
    DEFAULT_OPTIONS, 
    10 * TICKS_PER_SECOND, 
    &eventout 
  );
  directive_failed( status, "event_receive" );
  printf( "TA1 - EVENT_14 received - eventout => %08x\n", eventout );

  puts( "TA1 - event_send - send EVENT_19 to TA2" );
  status = event_send( Task_id[ 2 ], EVENT_19 );
  directive_failed( status, "event_send" );

  status = clock_get( &time );
  directive_failed( status, "clock_get" );
  print_time( "TA1 - clock_get - ", &time, "\n" );

pause();

  puts( "TA1 - event_send - send EVENT_18 to self after 5 seconds" );
  status = timer_fire_after( 
    Timer_id[ 1 ], 
    5 * TICKS_PER_SECOND,
    TA1_send_18_to_self_5_seconds,
    NULL
  );
  directive_failed( status, "timer_fire_after 5 seconds" );

  puts( "TA1 - event_receive - waiting forever on EVENT_18"  );
  status = event_receive( 
    EVENT_18, 
    DEFAULT_OPTIONS, 
    NO_TIMEOUT, 
    &eventout 
  );
  directive_failed( status, "event_receive of 18" );
  printf( "TA1 - EVENT_18 received - eventout => %08x\n", eventout );

  status = clock_get( &time );
  directive_failed( status, "TA1 clock_get" );
  print_time( "TA1 - clock_get - ", &time, "\n" );

  puts( "TA1 - event_send - send EVENT_3 to self" );
  status = event_send( SELF, EVENT_3 );
  directive_failed( status, "event_send" );

  puts( 
    "TA1 - event_receive - EVENT_3 or EVENT_22 - NO_WAIT and EVENT_ANY"
  );
  status = event_receive( 
    EVENT_3 | EVENT_22, 
    NO_WAIT | EVENT_ANY, 
    NO_TIMEOUT, 
    &eventout 
  );
  directive_failed( status, "event_receive of 3 and 22" );
  printf( "TA1 - EVENT_3 received - eventout => %08x\n", eventout );

  puts( "TA1 - event_send - send EVENT_4 to self" );
  status = event_send( SELF, EVENT_4 );
  directive_failed( status, "event_send" );

  puts (
  "TA1 - event_receive - waiting forever on EVENT_4 or EVENT_5 - EVENT_ANY"
  );
  status = event_receive( 
    EVENT_4 | EVENT_5, 
    EVENT_ANY, 
    NO_TIMEOUT, 
    &eventout 
  );
  directive_failed( status, "event_receive" );
  printf( "TA1 - EVENT_4 received - eventout => %08x\n", eventout );

pause();

  puts( "TA1 - event_send - send EVENT_18 to self after 5 seconds" );
  status = timer_fire_after( 
    Timer_id[ 1 ],
    5 * TICKS_PER_SECOND, 
    TA1_send_18_to_self_5_seconds,
    NULL
  );
  directive_failed( status, "timer_fire_after 5 seconds" );

  puts( "TA1 - timer_cancel - cancelling timer for event EVENT_18" );
  status = timer_cancel( Timer_id[ 1 ] );
  directive_failed( status, "timer_cancel" );

  puts( "TA1 - event_send - send EVENT_8 to self after 60 seconds" );
  status = timer_fire_after( 
    Timer_id[ 1 ],
    60 * TICKS_PER_SECOND, 
    TA1_send_8_to_self_60_seconds,
    NULL
  );
  directive_failed( status, "timer_fire_after 60 seconds" );

  puts( "TA1 - event_send - send EVENT_9 to self after 60 seconds" );
  status = timer_fire_after( 
    Timer_id[ 2 ],
    60 * TICKS_PER_SECOND, 
    TA1_send_9_to_self_60_seconds,
    NULL
  );
  directive_failed( status, "timer_fire_after 60 seconds" );

  puts( "TA1 - event_send - send EVENT_10 to self after 60 seconds" );
  status = timer_fire_after( 
    Timer_id[ 3 ],
    60 * TICKS_PER_SECOND, 
    TA1_send_10_to_self,
    NULL
  );
  directive_failed( status, "timer_fire_after 60 seconds" );

  puts( "TA1 - timer_cancel - cancelling timer for event EVENT_8" );
  status = timer_cancel( Timer_id[ 1 ] );
  directive_failed( status, "timer_cancel" );

  build_time( &time, 2, 12, 1988, 8, 15, 0, 0 );

  print_time( "TA1 - clock_set - ", &time, "\n" );
  status = clock_set( &time );
  directive_failed( status, "clock_set" );

  puts( "TA1 - event_send - send EVENT_1 every second" );
  status = timer_fire_after( 
    Timer_id[ 1 ],
    TICKS_PER_SECOND, 
    TA1_send_1_to_self_every_second,
    NULL
  );
  directive_failed( status, "timer_fire_after 1 second" );

  for ( index = 0; index < 3; index++ ) {

    status = event_receive( EVENT_1, EVENT_ANY, NO_TIMEOUT, &eventout );
    directive_failed( status, "event_receive" );

    status = clock_get( &time );
    directive_failed( status, "clock_get" );

    printf( "TA1 - EVENT_1 received - eventout => %08x - ", eventout );
    print_time( "at ", &time, "\n" );

    if ( index < 2 ) {
      status = timer_reset( Timer_id[ 1 ] );
      directive_failed( status, "timer_reset" );
    };
      
  }

  puts( "TA1 - timer_cancel - cancelling timer for event EVENT_1" );
  status = timer_cancel( Timer_id[ 1 ] );
  directive_failed( status, "timer_cancel" );

pause();

  time.day = 13;
  puts( "TA1 - event_send - sending EVENT_11 to self in 1 day" );
  status = timer_fire_when( 
    Timer_id[ 1 ],
    &time,
    TA1_send_11_to_self,
    NULL
  );
  directive_failed( status, "timer_fire_when 1 day" );

  time.hour = 7;
  puts( "TA1 - event_send - sending EVENT_11 to self in 1 day" );
  status = timer_fire_when( 
    Timer_id[ 2 ],
    &time,
    TA1_send_11_to_self,
    NULL
  );
  directive_failed( status, "timer_fire_when 1 day" );

  time.hour = 8;   /* so code below has correct time/date */
  time.day = 14;
  puts( "TA1 - event_send - sending EVENT_11 to self in 2 days" );
  status = timer_fire_when( 
    Timer_id[ 3 ],
    &time,
    TA1_send_11_to_self,
    NULL
  );
  directive_failed( status, "timer_fire_when 2 days" );

  puts( "TA1 - timer_cancel - cancelling EVENT_11 to self in 1 day" );
  status = timer_cancel( Timer_id[ 1 ] );
  directive_failed( status, "timer_cancel" );

  puts( "TA1 - timer_cancel - cancelling EVENT_11 to self in 2 days" );
  status = timer_cancel( Timer_id[ 3 ] );
  directive_failed( status, "timer_cancel" );

  puts( "TA1 - event_send - resending EVENT_11 to self in 2 days" );
  status = timer_fire_when( 
    Timer_id[ 3 ],
    &time,
    TA1_send_11_to_self,
    NULL
  );
  directive_failed( status, "timer_fire_when 2 days" );

  time.day = 15;
  print_time( "TA1 - clock_set - ", &time, "\n" );
  status = clock_set( &time );
  directive_failed( status, "TA1 clock_set" );
  
  puts( "TA1 - event_receive - waiting forever on EVENT_11" );
  status = event_receive( 
    EVENT_11, 
    DEFAULT_OPTIONS, 
    NO_TIMEOUT, 
    &eventout 
  );
  directive_failed( status, "event_receive" );
  printf( "TA1 - EVENT_11 received - eventout => %08x\n", eventout );

pause();

  puts( "TA1 - event_send/event_receive combination" );
  status = timer_fire_after( 
    Timer_id[ 1 ],
    10,
    TA1_send_11_to_self,
    NULL
  );
  directive_failed( status, "timer_fire_after 10 ticks" );

  status = event_receive( 
    EVENT_11, 
    DEFAULT_OPTIONS, 
    NO_TIMEOUT, 
    &eventout 
  );
  directive_failed( status, "event_receive" );

  build_time( &time, 2, 12, 1988, 8, 15, 0, 0 );

  print_time( "TA1 - clock_set - ", &time, "\n" );
  status = clock_set( &time );
  directive_failed( status, "clock_set" );

  time.day = 13;
  puts( "TA1 - event_receive all outstanding events" );
  status  = event_receive( 
    ALL_EVENTS, 
    NO_WAIT | EVENT_ANY, 
    0, 
    &eventout 
  );
  fatal_directive_status( 
    status, 
    UNSATISFIED, 
    "event_receive all events" 
  );

  puts( "TA1 - event_send - sending EVENT_10 to self in 1 day" );
  status = timer_fire_when( 
    Timer_id[ 1 ],
    &time,
    TA1_send_10_to_self,
    NULL
  );
  directive_failed( status, "timer_fire_when 1 day" );

  time.day = 14;
  puts( "TA1 - event_sent - sending EVENT_11 to self in 2 days" );
  status = timer_fire_when( 
    Timer_id[ 2 ],
    &time,
    TA1_send_11_to_self,
    NULL
  );
  directive_failed( status, "timer_fire_when 2 days" );

  build_time( &time, 2, 12, 1988, 7, 15, 0, 0 );

  print_time( "TA1 - clock_set - ", &time, "\n" );
  puts( "TA1 - set time backwards" );
  status = clock_set( &time );
  directive_failed( status, "clock_set" );

  status  = event_receive( 
    ALL_EVENTS, 
    NO_WAIT | EVENT_ANY, 
    NO_TIMEOUT, 
    &eventout 
  );
  if ( eventout ) printf( "ERROR -0x%08x events received\n", eventout );
  else            puts( "TA1 - no events received" );
  fatal_directive_status( 
    status, 
    UNSATISFIED, 
    "event_receive all events" 
  );

  build_time( &time, 2, 14, 1988, 7, 15, 0, 0 );

  print_time( "TA1 - clock_set - ", &time, "\n" );
  puts( "TA1 - set time forwards (leave a timer)" );
  status = clock_set( &time );
  directive_failed( status, "clock_set" );

  status  = event_receive( 
    ALL_EVENTS, 
    NO_WAIT | EVENT_ANY, 
    NO_TIMEOUT, 
    &eventout 
  );
  if ( eventout == EVENT_10 ) puts( "TA1 - EVENT_10 received" );
  else      printf( "ERROR -0x%08x events received\n", eventout );
  directive_failed( status, "event_receive all events" );

  puts( "TA1 - event_send - sending EVENT_11 to self in 100 ticks" );
  status = timer_fire_after( 
    Timer_id[ 1 ],
    100,
    TA1_send_11_to_self,
    NULL
  );
  directive_failed( status, "timer_fire_after 100 ticks" );

  puts( "TA1 - event_send - sending EVENT_11 to self in 200 ticks" );
  status = timer_fire_after( 
    Timer_id[ 1 ],
    200,
    TA1_send_11_to_self,
    NULL
  );
  directive_failed( status, "timer_fire_after 200 ticks" );

  puts( "*** END OF TEST 11 ***" );
  exit( 0 );
}
