/*  Task_2
 *
 *  This routine serves as a test task.  Multiple tasks are required to
 *  verify all capabilities of the event manager.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

rtems_task Task_2(
  rtems_task_argument argument
)
{
  rtems_event_set   eventout;
  rtems_time_of_day time;
  rtems_status_code status;

  status = task_wake_after( 1*TICKS_PER_SECOND );
  directive_failed( status, "task_wake_after" );

  puts( "TA2 - event_receive - waiting forever on EVENT_16" );
  status = event_receive( 
    EVENT_16, 
    DEFAULT_OPTIONS, 
    NO_TIMEOUT, 
    &eventout 
  );
  directive_failed( status, "event_receive" );
  printf( "TA2 - EVENT_16 received - eventout => %08x\n", eventout );

  puts( "TA2 - event_send - send EVENT_14 and EVENT_15 to TA1" );
  status = event_send( Task_id[ 1 ], EVENT_14 | EVENT_15 );
  directive_failed( status, "event_send" );

  puts(
 "TA2 - event_receive - waiting forever on EVENT_17 or EVENT_18 - EVENT_ANY"
  );
  status = event_receive( 
    EVENT_17 | EVENT_18, 
    EVENT_ANY, 
    NO_TIMEOUT, 
    &eventout 
  );
  directive_failed( status, "event_receive" );
  printf( "TA2 - EVENT_17 or EVENT_18 received - eventout => %08x\n", 
           eventout 
        );

  puts( "TA2 - event_send - send EVENT_14 to TA1" );
  status = event_send( Task_id[ 1 ], EVENT_14 );
  directive_failed( status, "event_send" );

  build_time( &time, 2, 12, 1988, 8, 15, 0, 0 );
  print_time( "TA2 - clock_set - ", &time, "\n" );
  status = clock_set( &time );
  directive_failed( status, "TA2 clock_set" );

  time.second += 5;
  puts( "TA2 - event_send - sending EVENT_10 to self after 5 seconds" );
  status = timer_fire_when( 
    Timer_id[ 5 ],
    &time,
    TA2_send_10_to_self,
    NULL
  );
  directive_failed( status, "timer_fire_when after 5 seconds" );

  puts( "TA2 - event_receive - waiting forever on EVENT_10" );
  status = event_receive( 
    EVENT_10, 
    DEFAULT_OPTIONS, 
    NO_TIMEOUT, 
    &eventout 
  );
  directive_failed( status, "event_receive" );

  status = clock_get( &time );
  directive_failed( status, "clock_get" );

  printf( "TA2 - EVENT_10 received - eventout => %08x\n", eventout );
  print_time( "TA2 - clock_get - ", &time, "\n" );

  puts( "TA2 - event_receive - PENDING_EVENTS" );
  status = event_receive( 
    PENDING_EVENTS, 
    DEFAULT_OPTIONS, 
    NO_TIMEOUT, 
    &eventout 
  );
  directive_failed( status, "event_receive" );
  printf( "TA2 - eventout => %08x\n", eventout );

  puts( "TA2 - event_receive - EVENT_19 - NO_WAIT" );
  status = event_receive( EVENT_19, NO_WAIT, NO_TIMEOUT, &eventout );
  directive_failed( status, "event_receive" );
  printf( "TA2 - EVENT_19 received - eventout => %08x\n", eventout );

  puts( "TA2 - task_delete - deletes self" );
  status = task_delete( Task_id[ 2 ] );
  directive_failed( status, "task_delete of TA2" );
}
