/*  Task_1
 *
 *  This routine serves as a test task.  It verifies the semaphore manager.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

rtems_task Task_1(
  rtems_task_argument argument
)
{
  rtems_id          smid;
  rtems_status_code status;

  status = semaphore_ident( Semaphore_name[ 1 ], SEARCH_ALL_NODES, &smid );
  printf( "TA1 - semaphore_ident - smid => %08x\n", smid );
  directive_failed( status, "semaphore_ident of SM1" );

  puts( "TA1 - semaphore_obtain - wait forever on SM2" );
  status = semaphore_obtain( 
    Semaphore_id[ 2 ], 
    DEFAULT_OPTIONS, 
    NO_TIMEOUT 
  );
  directive_failed( status, "semaphore_obtain of SM2" );
  puts( "TA1 - got SM2" );

  puts( "TA1 - semaphore_obtain - wait forever on SM3" );
  status = semaphore_obtain( 
    Semaphore_id[ 3 ], 
    DEFAULT_OPTIONS, 
    NO_TIMEOUT 
  );
  directive_failed( status, "semaphore_obtain of SM3" );
  puts( "TA1 - got SM3" );

  puts( "TA1 - semaphore_obtain - get SM1 - NO_WAIT" );
  status = semaphore_obtain( Semaphore_id[ 1 ], NO_WAIT, NO_TIMEOUT );
  directive_failed( status, "semaphore_obtain of SM1" );
  puts( "TA1 - got SM1" );

  puts( "TA1 - task_wake_after - sleep 5 seconds" );
  status = task_wake_after( 5*TICKS_PER_SECOND );
  directive_failed( status, "task_wake_after" );

pause();

  puts( "TA1 - semaphore_release - release SM1" );
  status = semaphore_release( Semaphore_id[ 1 ] );
  directive_failed( status, "semaphore_release of SM1" );

  puts(
    "TA1 - semaphore_obtain - waiting for SM1 with 10 second timeout"
  );
  status = semaphore_obtain( 
    Semaphore_id[ 1 ], 
    DEFAULT_OPTIONS, 
    10 * TICKS_PER_SECOND 
  );
  directive_failed( status, "semaphore_obtain of SM1" );
  puts( "TA1 - got SM1" );

  puts( "TA1 - semaphore_release - release SM2" );
  status = semaphore_release( Semaphore_id[ 2 ] );
  directive_failed( status, "semaphore_release of SM2" );

  puts( "TA1 - task_wake_after - sleep 5 seconds" );
  status = task_wake_after( 5*TICKS_PER_SECOND );
  directive_failed( status, "task_wake_after" );

pause();

  puts( "TA1 - task_delete - delete TA3" );
  status = task_delete( Task_id[ 3 ] );
  directive_failed( status, "task_delete of TA3" );

  status = task_create( 
    Task_name[ 4 ], 
    4, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 4 ] 
  );
  directive_failed( status, "task_create of TA4" );

  status = task_create( 
    Task_name[ 5 ], 
    4, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 5 ]
   );
  directive_failed( status, "task_create of TA5" );

  status = task_start( Task_id[ 4 ], Task_4, 0 );
  directive_failed( status, "task_start of TA4" );

  status = task_start( Task_id[ 5 ], Task5, 0 );
  directive_failed( status, "task_start of TA5" );

  puts( "TA1 - task_wake_after - sleep 5 seconds" );
  status = task_wake_after( 5*TICKS_PER_SECOND );
  directive_failed( status, "task_wake_after" );

  puts( "TA1 - task_delete - delete TA4" );
  status = task_delete( Task_id[ 4 ] );  
  directive_failed( status, "task_delete of TA4" );

  puts( "TA1 - semaphore_release - release SM1" );
  status = semaphore_release( Semaphore_id[ 1 ] );
  directive_failed( status, "semaphore_release on SM1" );

  puts( "TA1 - task_wake_after - sleep 5 seconds" );
  status = task_wake_after( 5*TICKS_PER_SECOND );
  directive_failed( status, "task_wake_after" );

  puts( "TA1 - semaphore_delete - delete SM1" );
  status = semaphore_delete( Semaphore_id[ 1 ] );
  directive_failed( status, "semaphore_delete of SM1" );

  puts( "TA1 - semaphore_delete - delete SM3" );
  status = semaphore_delete( Semaphore_id[ 3 ] );
  directive_failed( status, "semaphore_delete of SM3" );

  puts( "TA1 - task_delete - delete self" );
  status = task_delete( SELF );
  directive_failed( status, "task_delete of TA1" );
}
