/*  Task_1
 *
 *  This routine serves as a test task.  It establishes an ASR and
 *  sends signal to itself to determine if the ASR gets to execute.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

rtems_timer_service_routine Signal_3_to_task_1(
  rtems_id  id,
  void     *ptr
)
{
  rtems_status_code status;

  status = signal_send( Task_id[ 1 ], SIGNAL_3 );
  directive_failed( status, "signal_send of 3" );

  Timer_got_this_id  = id;
  Timer_got_this_ptr = ptr;

  Signals_sent = TRUE;
}

rtems_task Task_1(
  rtems_task_argument argument
)
{
  rtems_mode        previous_mode;
  rtems_status_code status;

  puts( "TA1 - signal_catch - INTERRUPT_LEVEL( 3 )" );
  status = signal_catch( Process_asr, INTERRUPT_LEVEL(3) );
  directive_failed( status, "signal_catch" );

  puts( "TA1 - signal_send - SIGNAL_16 to self" );
  status = signal_send( SELF, SIGNAL_16 );
  directive_failed( status, "signal_send" );

  puts( "TA1 - signal_send - SIGNAL_0 to self" );
  status = signal_send( SELF, SIGNAL_0 );
  directive_failed( status, "signal_send" );

  puts( "TA1 - signal_catch - NO_ASR" );
  status = signal_catch( Process_asr, NO_ASR );
  directive_failed( status, "signal_catch" );

pause();

  puts( "TA1 - signal_send - SIGNAL_1 to self" );
  status = signal_send( SELF, SIGNAL_1 );
  directive_failed( status, "signal_send" );

  puts( "TA1 - task_mode - disable ASRs" );
  status = task_mode( NO_ASR, ASR_MASK, &previous_mode );
  directive_failed( status, "task_mode" );

  Timer_got_this_id = 0;
  Timer_got_this_ptr = NULL;

  puts( "TA1 - sending signal to SELF from timer" );
  status = timer_fire_after(
    Timer_id[ 1 ],
    TICKS_PER_SECOND / 2,
    Signal_3_to_task_1,
    Task_1
  );
  directive_failed( status, "timer_fire_after" );

  puts( "TA1 - waiting for signal to arrive" );

  Signals_sent = FALSE;
  Asr_fired   = FALSE;

  while ( Signals_sent == FALSE )
    ;

  if ( Timer_got_this_id == Timer_id[ 1 ] && 
       Timer_got_this_ptr == Task_1 )
    puts( "TA1 - timer routine got the correct arguments" );
  else 
    printf( 
      "TA1 - timer got (0x%x, %p) instead of (0x%x, %p)!!!!\n",
      Timer_got_this_id, 
      Timer_got_this_ptr,
      Timer_id[ 1 ],
      Task_1 
    );

  puts( "TA1 - task_mode - enable ASRs" );
  status = task_mode( ASR, ASR_MASK, &previous_mode );
  directive_failed( status, "task_mode" );

  puts( "TA1 - signal_catch - asraddr of NULL" );
  status = signal_catch( NULL, DEFAULT_MODES );
  directive_failed( status, "signal_catch" );

  puts( "TA1 - task_delete - delete self" );
  status = task_delete( SELF );
  directive_failed( status, "task_delete of SELF" );
}
