/*  Init
 *
 *  This routine is the initialization task for this test program.
 *  It is a user initialization task and has the responsibility for creating
 *  and starting the tasks that make up the test.  If the time of day
 *  clock is required for the test, it should also be set to a known
 *  value by this function.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#undef EXTERN
#define EXTERN
#include "conftbl.h"
#include "gvar.h"

rtems_task Init(
  rtems_task_argument argument
)
{
  rtems_status_code status;

  puts( "\n\n*** TEST 15 ***" );

  Task_name[ 1 ] =  build_name( 'T', 'A', '1', ' ' );

  Partition_name[ 1 ] =  build_name( 'P', 'T', '1', ' ' );
  Partition_name[ 2 ] =  build_name( 'P', 'T', '2', ' ' );

  status = task_create( 
    Task_name[ 1 ], 
    4, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 1 ] 
  );
  directive_failed( status, "task_create of TA1" );

  status = task_start( Task_id[ 1 ], Task_1, 0 );
  directive_failed( status, "task_start of TA1" );

  puts("INIT - partition_create - partition 1");
  status = partition_create( 
    Partition_name[ 1 ], 
    Area_1,
    4096, 
    512, 
    DEFAULT_ATTRIBUTES, 
    &Partition_id[ 1 ]
  );
  directive_failed( status, "partition_create of PT1" );

  puts("INIT - partition_create - partition 2");
  status = partition_create( 
    Partition_name[ 2 ], 
    Area_2,
    274, 
    128, 
    DEFAULT_ATTRIBUTES, 
    &Partition_id[ 2 ]
  );
  directive_failed( status, "partition_create of PT2" );

  status = task_delete( SELF );
  directive_failed( status, "task_delete of SELF" );
}
