/*  FP_task
 *
 *  This routine serves as a floating point test task.  It verifies the 
 *  basic task switching capabilities of the executive when floating 
 *  point is configured.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

rtems_task FP_task(
  rtems_task_argument argument
)
{
  rtems_status_code status;
  rtems_id          tid;
  rtems_time_of_day time;

  status = task_ident( SELF, SEARCH_ALL_NODES, &tid );
  directive_failed( status, "task_ident of self" );

  FP_load( FP_context[ task_number( tid ) - 3 ] );

  while( FOREVER ) {

    status = clock_get( &time );
    directive_failed( status, "clock_get" );

    if ( time.second == 16 ) {

      if ( task_number( tid ) == 4 ) {
        puts( "TA4 - task_delete - self" );
        status = task_delete( SELF );
        directive_failed( status, "task_delete of TA4" );
      }
      puts( "TA5 - task_delete - TA3" );
      status = task_delete( Task_id[ 3 ] );
      directive_failed( status, "task_delete of TA3" );

      puts( "*** END OF TEST 19 *** " );
      exit( 0 );
    }

    put_name( Task_name[ task_number( tid ) ], FALSE );
    print_time( " - clock_get - ", &time, "\n" );

    if ( FP_check( FP_context[ task_number( tid ) - 3 ] ) )
      puts( "FP Context was incorrect" );

    status = task_wake_after( TICKS_PER_SECOND );
    directive_failed( status, "task_wake_after" );
  }
}
