/*  Init
 *
 *  This routine is the initialization task for this test program.
 *  It is a user initialization task and has the responsibility for creating
 *  and starting the tasks that make up the test.  If the time of day
 *  clock is required for the test, it should also be set to a known
 *  value by this function.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#undef EXTERN
#define EXTERN 
#include "conftbl.h"
#include "gvar.h"

rtems_task Init(
  rtems_task_argument argument
)
{
  rtems_status_code status;

  puts( "\n\n*** TEST 19 ***" );

  Task_name[ 1 ] =  build_name( 'T', 'A', '1', ' ' );
  Task_name[ 2 ] =  build_name( 'T', 'A', '2', ' ' );
  Task_name[ 3 ] =  build_name( 'T', 'A', '3', ' ' );
  Task_name[ 4 ] =  build_name( 'T', 'A', '4', ' ' );
  Task_name[ 5 ] =  build_name( 'T', 'A', '5', ' ' );
  Task_name[ 6 ] =  build_name( 'F', 'P', '1', ' ' );

  status = task_create( 
    Task_name[ 1 ], 
    2, 
    2048, 
    DEFAULT_MODES, 
    FLOATING_POINT, 
    &Task_id[ 1 ] 
  );
  directive_failed( status, "task_create of TA1" );

  status = task_create( 
    Task_name[ 2 ], 
    2, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 2 ] 
  );
  directive_failed( status, "task_create of TA2" );

  status = task_create( 
    Task_name[ 3 ], 
    2, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 3 ] 
  );
  directive_failed( status, "task_create of TA3" );

  status = task_create( 
    Task_name[ 4 ], 
    2, 
    2048, 
    DEFAULT_MODES, 
    FLOATING_POINT, 
    &Task_id[ 4 ] 
  );
  directive_failed( status, "task_create of TA4" );

  status = task_create( 
    Task_name[ 5 ], 
    2, 
    2048, 
    DEFAULT_MODES, 
    FLOATING_POINT, 
    &Task_id[ 5 ] 
  );
  directive_failed( status, "task_create of TA5" );

  status = task_create( 
    Task_name[ 6 ], 
    1, 
    2048, 
    DEFAULT_MODES, 
    FLOATING_POINT, 
    &Task_id[ 6 ] 
  );
  directive_failed( status, "task_create of FP1" );

  status = task_start( Task_id[ 6 ], First_FP_task, 0 );
  directive_failed( status, "task_start of FP1" );

  status = task_start( Task_id[ 1 ], Task_1, 0 );
  directive_failed( status, "task_start of TA1" );

  status = task_start( Task_id[ 2 ], Task_1, 0 );
  directive_failed( status, "task_start of TA2" );

  status = task_start( Task_id[ 3 ], Task_1, 0 );
  directive_failed( status, "task_start of TA3" );

  status = task_start( Task_id[ 4 ], FP_task, 0 );
  directive_failed( status, "task_start of TA4" );

  status = task_start( Task_id[ 5 ], FP_task, 0 );
  directive_failed( status, "task_create of TA5" );

  FP_context[1][0] = 0.0;
  FP_context[1][1] = 1.0;
  FP_context[1][2] = 2.0;
  FP_context[1][3] = 3.0;
  FP_context[1][4] = 4.0;
  FP_context[1][5] = 5.0;
  FP_context[1][6] = 6.0;
  FP_context[1][7] = 7.0;
  
  FP_context[2][0] = 7.0;
  FP_context[2][1] = 6.0;
  FP_context[2][2] = 5.0;
  FP_context[2][3] = 4.0;
  FP_context[2][4] = 3.0;
  FP_context[2][5] = 2.0;
  FP_context[2][6] = 1.0;
  FP_context[2][7] = 0.0;

  status = task_delete( SELF );
  directive_failed( status, "task_delete of TA1" );
}
