/*  Task_1
 *
 *  This routine serves as a test task.  It tests the I/O manager.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

#define STUB_DRIVER_MAJOR     0x2 
#define NO_DRIVER_MAJOR       0x1
#define INVALID_DRIVER_MAJOR  0x5

rtems_task Task_1(
  rtems_task_argument argument
)
{
  rtems_status_code status;
  rtems_status_code return_value;

  puts( "-----  TESTING THE NULL DRIVER CHECKS  -----" );

  status = io_initialize( NO_DRIVER_MAJOR, 0, NULL, &return_value );
  directive_failed( status, "io_initialize" );
  puts( "TA1 - io_initialize  - NULL DRIVER SUCCESSFUL" );

  status = io_open( NO_DRIVER_MAJOR, 0, NULL, &return_value );
  directive_failed( status, "io_open" );
  puts( "TA1 - io_open        - NULL DRIVER SUCCESSFUL" );

  status = io_close( NO_DRIVER_MAJOR, 0, NULL, &return_value );
  directive_failed( status, "io_close" );
  puts( "TA1 - io_close       - NULL DRIVER SUCCESSFUL" );

  status = io_read( NO_DRIVER_MAJOR, 0, NULL, &return_value );
  directive_failed( status, "io_read" );
  puts( "TA1 - io_read        - NULL DRIVER SUCCESSFUL" );

  status = io_write( NO_DRIVER_MAJOR, 0, NULL, &return_value );
  directive_failed( status, "io_write" );
  puts( "TA1 - io_write       - NULL DRIVER SUCCESSFUL" );

  status = io_control( NO_DRIVER_MAJOR, 0, NULL, &return_value );
  directive_failed( status, "io_control" );
  puts( "TA1 - io_control     - NULL DRIVER SUCCESSFUL" );

  puts( "-----  TESTING THE I/O MANAGER DIRECTIVES  -----" );

  status = io_initialize( STUB_DRIVER_MAJOR, 0, NULL, &return_value );
  directive_failed( status, "io_initialize" );
  puts( "TA1 - io_initialize  - STUB DRIVER SUCCESSFUL" );

  status = io_open( STUB_DRIVER_MAJOR, 0, NULL, &return_value );
  directive_failed( status, "io_open" );
  puts( "TA1 - io_open        - STUB DRIVER SUCCESSFUL" );

  status = io_close( STUB_DRIVER_MAJOR, 0, NULL, &return_value );
  directive_failed( status, "io_close" );
  puts( "TA1 - io_close       - STUB DRIVER SUCCESSFUL" );

  status = io_read( STUB_DRIVER_MAJOR, 0, NULL, &return_value );
  directive_failed( status, "io_read" );
  puts( "TA1 - io_read        - STUB DRIVER SUCCESSFUL" );

  status = io_write( STUB_DRIVER_MAJOR, 0, NULL, &return_value );
  directive_failed( status, "io_write" );
  puts( "TA1 - io_write       - STUB DRIVER SUCCESSFUL" );

  status = io_control( STUB_DRIVER_MAJOR, 0, NULL, &return_value );
  directive_failed( status, "io_control" );
  puts( "TA1 - io_control     - STUB DRIVER SUCCESSFUL" );

  puts( "-----  RETURNING INVALID MAJOR NUMBER -----" );

  status = io_initialize( INVALID_DRIVER_MAJOR, 0, NULL, &return_value );
  fatal_directive_status(
    status,
    INVALID_NUMBER,
    "io_initialize"
  );
  puts( "TA1 - io_initialize  - INVALID_NUMBER" );

  status = io_open( INVALID_DRIVER_MAJOR, 0, NULL, &return_value );
  fatal_directive_status(
    status,
    INVALID_NUMBER,
    "io_open"
  );
  puts( "TA1 - io_open        - INVALID_NUMBER" );

  puts( "*** END OF TEST 21 ***" );
  exit( 0 );
}
