/*  Init
 *
 *  This routine is the initialization task for this test program.
 *  It is a user initialization task and has the responsibility for creating
 *  and starting the tasks that make up the test.  If the time of day
 *  clock is required for the test, it should also be set to a known
 *  value by this function.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#undef EXTERN
#define EXTERN 
#include "conftbl.h"
#include "gvar.h"

rtems_task Init(
  rtems_task_argument argument
)
{
  rtems_time_of_day  time;
  rtems_status_code  status;

  puts( "\n\n*** TEST 22 ***" );

  build_time( &time, 12, 31, 1988, 9, 0, 0, 0 );

  status = clock_set( &time );
  directive_failed( status, "clock_set" );

  Task_name[ 1 ]  = build_name( 'T', 'A', '1', ' ' );
  Timer_name[ 1 ] = build_name( 'T', 'M', '1', ' ' );

  status = task_create( 
    Task_name[ 1 ], 
    1, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 1 ] 
  );
  directive_failed( status, "task_create of TA1" );

  status = task_start( Task_id[ 1 ], Task_1, 0 );
  directive_failed( status, "task_start of TA1" );

  puts( "INIT - timer_create - creating timer 1" );
  status = timer_create( Timer_name[ 1 ], &Timer_id[ 1 ] );
  directive_failed( status, "timer_create" );

  printf( "INIT - timer 1 has id (0x%x)\n", Timer_id[ 1 ] );

  status = task_delete( SELF );
  directive_failed( status, "task_delete of SELF" );
}
