/*  Fatal
 *
 *  This routine is the fatal error handler.
 *
 *  Input parameters:
 *    error - error code
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#include "iodrv.h"

char Workspace[ 64 * 1024 ];

rtems_extension Workspace_of_zero_fatal_extension( 
  rtems_unsigned32 error
)
{

  Io_init();   /* must initialize console IO in C version */

  puts( "\n\n*** TEST 25 ***" );

  put_error( error, UNSATISFIED );

  Extensions.fatal = Null_workspace_fatal_extension;

  Configuration.work_space_start = NULL;

  initialize_executive( &Configuration, &Cpu_table );
}

rtems_extension Null_workspace_fatal_extension( 
  rtems_unsigned32 error
)
{
  put_error( error, INVALID_ADDRESS );

  Extensions.fatal = Workspace_too_small_fatal_extension;

  Configuration.work_space_start = Workspace;
  Configuration.work_space_size  = 256;

  initialize_executive( &Configuration, &Cpu_table );
}

rtems_extension Workspace_too_small_fatal_extension( 
  rtems_unsigned32 error
)
{
  put_error( error, UNSATISFIED );

  Extensions.fatal = Init_task_create_fatal_extension;

  Initialization_tasks[ 0 ].initial_priority = CURRENT_PRIORITY;

  Configuration.work_space_size = 64 * 1024;

  /* Task Create Error */
  initialize_executive( &Configuration, &Cpu_table );

  /* Task Start Error */
  initialize_executive( &Configuration, &Cpu_table );

  /* Task Exit Error */
  initialize_executive( &Configuration, &Cpu_table );
}

rtems_extension Init_task_create_fatal_extension( 
  rtems_unsigned32 error
)
{
  put_error( error, INVALID_PRIORITY );

  Extensions.fatal = Init_task_start_fatal_extension;

  Initialization_tasks[ 0 ].initial_priority = 1;
  Initialization_tasks[ 0 ].entry_point      = NULL;

  shutdown_executive( 0 );   /* to go back to main stack */
}

rtems_extension Init_task_start_fatal_extension( 
  rtems_unsigned32 error
)
{
  put_error( error, INVALID_ADDRESS );

  Extensions.fatal = Final_fatal_extension;

  Initialization_tasks[ 0 ].entry_point = Init;

  shutdown_executive( 0 );   /* to go back to main stack */
}

rtems_extension Final_fatal_extension( 
  rtems_unsigned32 error
)
{
  put_error( error, TASK_EXITTED );
  puts( "*** END OF TEST 25 ***" );

  /* returns to the default fatal error handler */
}
