/*  put_error
 *
 *  This routine verifies that the given error is the expected error.
 *
 *  Input parameters:
 *    error    - actual error code
 *    expected - expected error code
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

char *Errors[] = {
  "SUCCESSFUL",               /* successful completion */
  "TASK_EXITTED",             /* returned from a task */
  "MP_NOT_CONFIGURED",        /* multiprocessing not configured */
  "INVALID_NAME",             /* invalid object name */
  "INVALID_ID",               /* invalid object id */
  "TOO_MANY",                 /* too many */
  "TIMEOUT",                  /* timed out waiting */
  "OBJECT_WAS_DELETED",       /* object was deleted while waiting */
  "INVALID_SIZE",             /* specified size was invalid */
  "INVALID_ADDRESS",          /* address specified is invalid */
  "INVALID_NUMBER",           /* number was invalid */
  "NOT_DEFINED",              /* item has not been initialized */
  "RESOURCE_IN_USE",          /* resources still outstanding */
  "UNSATISFIED",              /* request not satisfied */
  "INCORRECT_STATE",          /* task is in wrong state */
  "ALREADY_SUSPENDED",        /* task already in state */
  "ILLEGAL_ON_SELF",          /* illegal operation on calling task */
  "ILLEGAL_ON_REMOTE_OBJECT", /* illegal operation for remote object */
  "CALLED_FROM_ISR",          /* called from ISR */
  "INVALID_PRIORITY",         /* invalid task priority */
  "INVALID_TIME_OF_DAY",      /* invalid date/time */
  "INVALID_NODE",             /* invalid node id */
  "NOT_OWNER_OF_RESOURCE",    /* not owner of resource */
  "NOT_CONFIGURED",           /* directive not configured */
  "NOT_IMPLEMENTED"           /* directive not implemented */
};

/* Task states */

void put_error( 
  rtems_unsigned32  error,
  rtems_status_code expected
)
{

  if ( error <= NOT_IMPLEMENTED )
    printf( "FATAL - error code is %s\n", Errors[ error ] );
  else
    printf( "ERROR - out of range error code is %d\n", error );

  if ( error != expected ) {
    printf( "ERROR - did not get expected code of %d", expected );
    exit( 0 );
  } 
}
