/*  Init
 *
 *  This routine is the initialization task for this test program.
 *  It is a user initialization task and has the responsibility for creating
 *  and starting the tasks that make up the test.  If the time of day
 *  clock is required for the test, it should also be set to a known
 *  value by this function.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#undef EXTERN
#define EXTERN
#include "conftbl.h"
#include "gvar.h"

rtems_task Init(
  rtems_task_argument argument
)
{
  rtems_status_code status;
  rtems_time_of_day time;

  puts( "\n\n*** TEST 4 ***" );
  build_time( &time, 12, 31, 1988, 9, 15, 0, 0 );

  status = clock_set( &time );
  directive_failed( status, "clock_set" );

  Task_name[ 1 ] =  build_name( 'T', 'A', '1', ' ' );
  Task_name[ 2 ] =  build_name( 'T', 'A', '2', ' ' );
  Task_name[ 3 ] =  build_name( 'T', 'A', '3', ' ' );

  Run_count[ 1 ] = 0;
  Run_count[ 2 ] = 0;
  Run_count[ 3 ] = 0;

  status = task_create(
     Task_name[ 1 ],
     1,
     2048,
     TIMESLICE,
     DEFAULT_ATTRIBUTES,
     &Task_id[ 1 ]
  );
  directive_failed( status, "task_create of TA1" );

  status = task_create(
     Task_name[ 2 ],
     1,
     2048,
     TIMESLICE,
     DEFAULT_ATTRIBUTES,
     &Task_id[ 2 ]
  );
  directive_failed( status, "task_create of TA2" );

  status = task_create(
     Task_name[ 3 ],
     1,
     2048,
     TIMESLICE,
     DEFAULT_ATTRIBUTES,
     &Task_id[ 3 ]
  );
  directive_failed( status, "task_create of TA3" );

  status = task_start( Task_id[ 1 ], Task_1, 0 );
  directive_failed( status, "task_start of TA1" );

  status = task_start( Task_id[ 2 ], Task_2, 0 );
  directive_failed( status, "task_start of TA2" );

  status = task_start( Task_id[ 3 ], Task_3, 0 );
  directive_failed( status, "task_start of TA3" );

  status = task_delete( SELF );
  directive_failed( status, "task_delete of SELF" );
}
