/*  Task_1
 *
 *  This test serves as a test task.  It verifies timeslicing activities
 *  and tswitch extension processing.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#include "clockdrv.h"

rtems_task Task_1(
  rtems_task_argument argument
)
{
  rtems_unsigned32  seconds;
  rtems_unsigned32  old_seconds;
  rtems_unsigned32  started_delay;
  rtems_mode        previous_mode;
  rtems_time_of_day time;
  rtems_status_code status;

  puts( "TA1 - task_suspend - on Task 2" );
  status = task_suspend( Task_id[ 2 ] ); 
  directive_failed( status, "task_suspend of TA2" );

  puts( "TA1 - task_suspend - on Task 3" );
  status = task_suspend( Task_id[ 3 ] ); 
  directive_failed( status, "task_suspend of TA3" );

  puts( "TA1 - killing time" );

  for ( started_delay = Clock_driver_ticks ;
        Clock_driver_ticks < (started_delay + 1000) ;
      )
    ;
  
  puts( "TA1 - task_resume - on Task 2" );
  status = task_resume( Task_id[ 2 ] ); 
  directive_failed( status, "task_resume of TA2" );

  puts( "TA1 - task_resume - on Task 3" );
  status = task_resume( Task_id[ 3 ] ); 
  directive_failed( status, "task_resume of TA3" );

  while ( FOREVER ) {
    if ( Run_count[ 1 ] == 3 ) {
      puts( "TA1 - task_mode - change mode to NO PREEMPT" );

      status = task_mode( NO_PREEMPT, PREEMPT_MASK, &previous_mode );    
      directive_failed( status, "task_mode" );

      status = clock_get( &time );
      directive_failed( status, "clock_get" );

      old_seconds = time.second;

      for ( seconds = 0 ; seconds < 6 ; ) {
        status = clock_get( &time );
        directive_failed( status, "clock_get" );

        if ( time.second != old_seconds ) {
          old_seconds = time.second;
          seconds++;
          print_time( "TA1 - ", &time, "\n" );
        }
      }

      puts( "TA1 - task_mode - change mode to PREEMPT" );
      status = task_mode( PREEMPT, PREEMPT_MASK, &previous_mode );
      directive_failed( status, "task_mode" );

      while ( FOREVER );
    }
  }
}
