/*  conftbl.h
 *
 *  This include file contains the configuration table of user defined
 *  configuration parameters.
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "clockdrv.h"
#include "iodrv.h"

rtems_driver_address_table Device_drivers[] = {
  {Clock_initialize,  NULL, NULL, NULL, NULL, NULL },
  {Io_init,     NULL, NULL, NULL, NULL, NULL }
};

rtems_initialization_tasks_table Initialization_tasks[] = {
  { build_name( 'U', 'I', '1', ' ' ), /* init task name */
    2048,                             /* init task stack size */
    1,                                /* init task priority */
    DEFAULT_ATTRIBUTES,               /* init task attributes */
    Init,                             /* init task entry point */
    NO_PREEMPT,                       /* init task initial mode */
    0                                 /* init task argument list */ 
  }
};

/* user extension table */

rtems_extensions_table Extensions = {
  Task_create_extension,     /* tcreate user extension */
  Task_start_extension,      /* tstart user extension */
  Task_restart_extension,    /* trestart user extension */
  Task_delete_extension,     /* tdelete user extension */
  NULL,                      /* tswitch user extension */
  Task_exit_extension,       /* task exitted user extension */
  NULL                       /* fatal error user extension */
};

rtems_configuration_table Configuration = {
  NULL,                      /* executive RAM work area */
  64 * 1024,                 /* executive RAM size */
  10,                        /* maximum # tasks */
  0,                         /* maximum # timers */
  0,                         /* maximum # semaphores */
  0,                         /* maximum # message queues */
  0,                         /* maximum # messages */
  0,                         /* maximum # partitions */
  0,                         /* maximum # regions */
  0,                         /* maximum # dp memory areas */
  0,                         /* maximum # periods */
  10,                        /* # ms in a tick */
  100,                       /* # ticks in a timeslice */
  sizeof (Initialization_tasks) / sizeof(rtems_initialization_tasks_table),
                             /* number of init tasks */
  Initialization_tasks,      /* init task(s) table */
  sizeof (Device_drivers) / sizeof(rtems_driver_address_table),
                             /* number of device drivers */
  Device_drivers,            /* pointer to driver address table */
  &Extensions,               /* user extension address */
  NULL                       /* ptr to MP config table */
};

/* end of include file */
