/*  Init
 *
 *  This routine is the initialization task for this test program.
 *  It is a user initialization task and has the responsibility for creating
 *  and starting the tasks that make up the test.  If the time of day
 *  clock is required for the test, it should also be set to a known
 *  value by this function.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#undef EXTERN
#define EXTERN
#include "conftbl.h"
#include "gvar.h"

rtems_task Init(
  rtems_task_argument argument
)
{
  rtems_status_code status;

  puts( "\n\n*** TEST 7 ***" );

  Task_name[ 1 ] = build_name( 'T', 'A', '1', ' ' );
  Task_name[ 2 ] = build_name( 'T', 'A', '2', ' ' );
  Task_name[ 3 ] = build_name( 'T', 'A', '3', ' ' );
  Task_name[ 4 ] = build_name( 'T', 'A', '4', ' ' );

  status = task_create( 
    Task_name[ 1 ], 
    4, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 1 ] 
  );
  directive_failed( status, "task_create of TA1" );

  status = task_create( 
    Task_name[ 2 ], 
    4, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 2 ] 
  );
  directive_failed( status, "task_create of TA2" );

  status = task_create( 
    Task_name[ 3 ], 
    250, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 3 ] 
  );
  directive_failed( status, "task_create of TA3" );

  status = task_create( 
    Task_name[ 4 ], 
    254, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 4 ] 
  );
  directive_failed( status, "task_create of TA4" );

  status = task_start( Task_id[ 1 ], Task_1, 0 );
  directive_failed( status, "task_start of TA1" );

  status = task_start( Task_id[ 2 ], Task_2, 0 );
  directive_failed( status, "task_start of TA2" );

  status = task_start( Task_id[ 3 ], Task_3, 0 );
  directive_failed( status, "task_start of TA3" );

  status = task_start( Task_id[ 4 ], Task_4, 0 );
  directive_failed( status, "task_start of TA4" );

  status = task_restart( Task_id[ 3 ], 0 );
  directive_failed( status, "task_restart of TA3" );

  status = task_set_note( Task_id[ 1 ], NOTEPAD_8, 4 );
  directive_failed( status, "task_set_node of TA1" );
  printf( "INIT - task_set_note - set TA1's NOTEPAD_8 " );
  puts  ( "to TA1's initial priority: 04" );

  status = task_set_note( Task_id[ 2 ], NOTEPAD_8, 4 );
  directive_failed( status, "task_set_node of TA2" );
  printf( "INIT - task_set_note - set TA2's NOTEPAD_8 " );
  puts  ( "to TA2's initial priority: 04");

  status = task_delete( SELF );
  directive_failed( status, "task_delete of SELF" );
}
