/*  Task_1
 *
 *  This routine serves as a test task.  It verifies intertask communication
 *  using task notepads and verifies the the user extensions (tcreate, etc).
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

rtems_task Task_1(
  rtems_task_argument argument
)
{
  rtems_status_code   status;
  rtems_task_priority the_priority;
  rtems_task_priority previous_priority;

  pause();

  status = task_set_priority( SELF, CURRENT_PRIORITY, &the_priority );
  directive_failed( status, "task_set_priority" );
  printf( "TA1 - task_set_priority - get initial priority of self: %02d\n",
          the_priority
  );

  while( FOREVER ) {
    status = task_get_note( SELF, NOTEPAD_8, &the_priority );
    directive_failed( status, "task_get_note" );
    printf( 
       "TA1 - task_get_note - get NOTEPAD_8 - current priority: %02d\n",
       the_priority
    );

    if ( --the_priority == 0 ) {
      puts( "TA1 - task_suspend - suspend TA2" );
      status = task_suspend( Task_id[ 2 ] );
      directive_failed( status, "task_suspend" );

      puts( "TA1 - task_set_priority - set priority of TA2 ( blocked )" );
      status = task_set_priority( Task_id[ 2 ], 5, &previous_priority );
      directive_failed( status, "task_set_priority" );

      status = task_delete( Task_id[ 2 ] );
      directive_failed( status, "task_delete of TA2" );

      status = task_delete( SELF );
      directive_failed( status, "task_delete of SELF" );
    }

    printf( "TA1 - task_set_note - set TA2's NOTEPAD_8: %02d\n", 
            the_priority 
    );
    status = task_set_note( Task_id[ 2 ], NOTEPAD_8, the_priority );
    directive_failed( status, "task_set_note" );

    printf( "TA1 - task_set_priority - set TA2's priority: %02d\n", 
            the_priority 
    );
    status = task_set_priority( 
               Task_id[ 2 ], 
               the_priority, 
               &previous_priority 
    );
    directive_failed( status, "task_set_priority" );
  }
}
