/*  Task_1
 *
 *  This routine serves as a test task.  It verifies all combinations of
 *  task mode changes.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

rtems_task Task_1(
  rtems_task_argument argument
)
{
  rtems_mode        previous_mode;
  rtems_status_code status;

/* BEGINNING OF ASR */

  status = task_mode( ASR, ASR_MASK, &previous_mode );
  directive_failed( status, "task_mode" );
  put_mode( "TA1 - task_mode - ASR                  - previous mode: ",
            previous_mode
          );

  status = task_mode( NO_ASR, ASR_MASK, &previous_mode );
  directive_failed( status, "task_mode" );
  put_mode( "TA1 - task_mode - NO_ASR               - previous mode: ",
            previous_mode
          );

  status = task_mode( NO_ASR, ASR_MASK, &previous_mode );
  directive_failed( status, "task_mode" );
  put_mode( "TA1 - task_mode - NO_ASR               - previous mode: ",
            previous_mode
          );

  status = task_mode( ASR, ASR_MASK, &previous_mode );
  directive_failed( status, "task_mode" );
  put_mode( "TA1 - task_mode - ASR                  - previous mode: ",
            previous_mode
          );

/* END OF ASR */

/* BEGINNING OF TIMESLICE */

  status = task_mode( NO_TIMESLICE, TIMESLICE_MASK, &previous_mode );
  directive_failed( status, "task_mode" );
  put_mode( "TA1 - task_mode - NO_TIMESLICE         - previous mode: ",
            previous_mode
          );

  status = task_mode( TIMESLICE, TIMESLICE_MASK, &previous_mode );
  directive_failed( status, "task_mode" );
  put_mode( "TA1 - task_mode - TIMESLICE            - previous mode: ",
            previous_mode
          );

  status = task_mode( TIMESLICE, TIMESLICE_MASK, &previous_mode );
  directive_failed( status, "task_mode" );
  put_mode( "TA1 - task_mode - TIMESLICE            - previous mode: ",
            previous_mode
          );

  status = task_mode( NO_TIMESLICE, TIMESLICE_MASK, &previous_mode );
  directive_failed( status, "task_mode" );
  put_mode( "TA1 - task_mode - NO_TIMESLICE         - previous mode: ",
            previous_mode
          );

/* END OF TIMESLICE */

/* BEGINNING OF PREEMPT */

  status = task_mode( PREEMPT, PREEMPT_MASK, &previous_mode );
  directive_failed( status, "task_mode" );
  put_mode( "TA1 - task_mode - PREEMPT              - previous mode: ",
            previous_mode
          );

  status = task_mode( NO_PREEMPT, PREEMPT_MASK, &previous_mode );
  directive_failed( status, "task_mode" );
  put_mode( "TA1 - task_mode - NO_PREEMPT           - previous mode: ",
            previous_mode
          );

  status = task_mode( NO_PREEMPT, PREEMPT_MASK, &previous_mode );
  directive_failed( status, "task_mode" );
  put_mode( "TA1 - task_mode - NO_PREEMPT           - previous mode: ",
            previous_mode
          );

  status = task_mode( PREEMPT, PREEMPT_MASK, &previous_mode );
  directive_failed( status, "task_mode" );
  put_mode( "TA1 - task_mode - PREEMPT              - previous mode: ",
            previous_mode
          );

/* END OF PREEMPT */

/* BEGINNING OF INTERRUPT_LEVEL */

  status = task_mode( INTERRUPT_LEVEL(3), INTERRUPT_MASK, &previous_mode );
  directive_failed( status, "task_mode" );
  put_mode( "TA1 - task_mode - INTERRUPT_LEVEL( 3 ) - previous mode: ",
            previous_mode
          );

  status = task_mode( INTERRUPT_LEVEL(5), INTERRUPT_MASK, &previous_mode );
  directive_failed( status, "task_mode" );
  put_mode( "TA1 - task_mode - INTERRUPT_LEVEL( 5 ) - previous mode: ",
            previous_mode
          );

/* END OF INTERRUPT_LEVEL */

/* BEGINNING OF COMBINATIONS */

  status = task_mode( 
             INTERRUPT_LEVEL(3) | NO_ASR | TIMESLICE | NO_PREEMPT,
             INTERRUPT_MASK | ASR_MASK |
                TIMESLICE_MASK | PREEMPT_MASK, 
             &previous_mode 
           );
  directive_failed( status, "task_mode" );
  put_mode( "TA1 - task_mode - set all modes        - previous mode: ",
            previous_mode
          );

  status = task_mode( 
             INTERRUPT_LEVEL(3) | NO_ASR | TIMESLICE | NO_PREEMPT,
             INTERRUPT_MASK | ASR_MASK | 
               TIMESLICE_MASK | PREEMPT_MASK, 
             &previous_mode 
           );
  directive_failed( status, "task_mode" );
  put_mode( "TA1 - task_mode - set all modes        - previous mode: ",
            previous_mode
          );

  status = task_mode( 
             INTERRUPT_LEVEL(0) | ASR | NO_TIMESLICE | PREEMPT,
             INTERRUPT_MASK | ASR_MASK | 
               TIMESLICE_MASK | PREEMPT_MASK, 
             &previous_mode 
           );
  directive_failed( status, "task_mode" );
  put_mode( "TA1 - task_mode - clear all modes      - previous mode: ",
            previous_mode
          );

/* END OF COMBINATIONS */

/* BEGINNING OF CURRENT MODE */

  status = task_mode( CURRENT_MODE, CURRENT_MODE, &previous_mode );
  directive_failed( status, "task_mode" );
  put_mode( "TA1 - task_mode - get current mode     - previous mode: ",
            previous_mode
          );

/* END OF CURRENT MODE */

  puts( "*** END OF TEST 8 ***" );
  exit( 0 );
}
