/*  Init
 *
 *  This routine is the initialization task for this test program.
 *  It is a user initialization task and has the responsibility for creating
 *  and starting the tasks that make up the test.  If the time of day
 *  clock is required for the test, it should also be set to a known
 *  value by this function.
 *
 *  Input parameters:  
 *    argument - task argument
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#undef EXTERN
#define EXTERN
#include "conftbl.h"
#include "gvar.h"

rtems_task Init(
  rtems_task_argument argument
)
{
  rtems_status_code status;

  puts( "\n\n*** TEST 9 ***" );

  Task_name[ 1 ]       =  build_name( 'T', 'A', '1', ' ' );
  Task_name[ 2 ]       =  build_name( 'T', 'A', '2', ' ' );
  Task_name[ 3 ]       =  build_name( 'T', 'A', '3', ' ' );
  Task_name[ 4 ]       =  build_name( 'T', 'A', '4', ' ' );
  Task_name[ 5 ]       =  build_name( 'T', 'A', '5', ' ' );
  Task_name[ 6 ]       =  build_name( 'T', 'A', '6', ' ' );
  Task_name[ 7 ]       =  build_name( 'T', 'A', '7', ' ' );
  Task_name[ 8 ]       =  build_name( 'T', 'A', '8', ' ' );
  Task_name[ 9 ]       =  build_name( 'T', 'A', '9', ' ' );
  Task_name[ 10 ]      =  build_name( 'T', 'A', 'A', ' ' );

  Timer_name[ 1 ]      =  build_name( 'T', 'M', '1', ' ' );

  Semaphore_name[ 1 ]  =  build_name( 'S', 'M', '1', ' ' );
  Semaphore_name[ 2 ]  =  build_name( 'S', 'M', '2', ' ' );
  Semaphore_name[ 3 ]  =  build_name( 'S', 'M', '3', ' ' );

  Queue_name[ 1 ]      =  build_name( 'M', 'Q', '1', ' ' );
  Queue_name[ 2 ]      =  build_name( 'M', 'Q', '2', ' ' );

  Partition_name[ 1 ]  =  build_name( 'P', 'T', '1', ' ' );

  Region_name[ 1 ]     =  build_name( 'R', 'N', '1', ' ' );

  Port_name[ 1 ]       =  build_name( 'D', 'P', '1', ' ' );

  Period_name[ 1 ]     =  build_name( 'T', 'M', '1', ' ' );

  status = task_create( 
    Task_name[1],
    4,
    10,
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES,
    &Task_id[ 1 ] 
  );
  fatal_directive_status( 
    status, 
    INVALID_SIZE, 
    "task_create with illegal stack size"
  );
  puts( "INIT - task_create - INVALID_SIZE" );

  status = task_create( 
     Task_name[1],
     0,
     2048,
     DEFAULT_MODES, 
     DEFAULT_ATTRIBUTES,
     &Task_id[ 1 ]
  );
  fatal_directive_status( 
    status, 
    INVALID_PRIORITY,
    "task_create with illegal priority"
  );
  puts( "INIT - task_create - INVALID_PRIORITY" );

  status = task_create( 
    Task_name[ 1 ], 
    4, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 1 ] 
  );
  directive_failed( status, "task_create of TA1" );

  status = task_restart( Task_id[ 1 ], 0 );
  fatal_directive_status( 
    status, 
    INCORRECT_STATE,
    "task_restart of DORMANT task"
  );
  puts( "INIT - task_restart - INCORRECT_STATE" );

  status = task_start( Task_id[ 1 ], Task_1, 0 );
  directive_failed( status, "task_start of TA1" );

  status = task_delete( SELF );
  directive_failed( status, "task_delete of SELF" );
}
