/*  Screen1
 *
 *  This routine generates error screen 1 for test 9.
 *
 *  Input parameters:  NONE
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

void Screen1()
{
  rtems_unsigned32    notepad_value;
  rtems_id            self_id;
  rtems_task_priority previous_priority;
  rtems_status_code   status;

  status = task_delete( 100 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "task_delete with illegal id"
  );
  puts( "TA1 - task_delete - INVALID_ID" );

  status = task_get_note( SELF, 100, &notepad_value );
  fatal_directive_status( 
    status,
    INVALID_NUMBER,
    "task_get_note with illegal notepad"
  );
  puts( "TA1 - task_get_note - INVALID_NUMBER" );

  status = task_get_note( 100, NOTEPAD_15, &notepad_value );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "task_get_note with illegal id"
  );
  puts( "TA1 - task_get_note - INVALID_ID" );

  status = task_ident( SELF, SEARCH_ALL_NODES, &self_id );
  directive_failed( status, "task_ident of self" );

  if ( self_id != Task_id[ 1 ] ) {
    puts( "ERROR - task_ident - incorrect ID returned!" );
  }
  puts( "TA1 - task_ident - current task SUCCESSFUL" );

  status = task_ident( 100, SEARCH_ALL_NODES, &Junk_id );
  fatal_directive_status( 
    status,
    INVALID_NAME,
    "task_ident with illegal name (local)"
  );
  puts( "TA1 - task_ident - global INVALID_NAME" );

  status = task_ident( 100, 1, &Junk_id );
  fatal_directive_status( 
    status,
    INVALID_NAME,
    "task_ident with illegal name (global)"
  );
  puts( "TA1 - task_ident - local INVALID_NAME" );

  status = task_ident( 100, 2, &Junk_id );
  fatal_directive_status( 
    status,
    INVALID_NODE,
    "task_ident with illegal node"
  );
  puts( "TA1 - task_ident - INVALID_NODE" );

  status = task_restart( 100, 0 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "task_restart with illegal id"
  );
  puts( "TA1 - task_restart - INVALID_ID" );

  status = task_resume( 100 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "task_resume with illegal id"
  );
  puts( "TA1 - task_resume - INVALID_ID" );

  status = task_resume( SELF );
  fatal_directive_status( 
    status,
    INCORRECT_STATE,
    "task_resume of ready task"
  );
  puts( "TA1 - task_resume - INCORRECT_STATE" );

  status = task_set_priority( SELF, 512, &previous_priority );
  fatal_directive_status( 
    status,
    INVALID_PRIORITY,
    "task_set_priority with illegal priority"
  );
  puts( "TA1 - task_set_priority - INVALID_PRIORITY" );

  status = task_set_priority( 100, 8, &previous_priority );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "task_set_priority with illegal id"
  );
  puts( "TA1 - task_set_priority - INVALID_ID" );

  status = task_set_note( SELF, NOTEPAD_15+10, notepad_value );
  fatal_directive_status( 
    status,
    INVALID_NUMBER,
    "task_set_note with illegal notepad"
  );
  puts( "TA1 - task_set_note - INVALID_NUMBER" );
  
  status = task_set_note( 100, NOTEPAD_15, notepad_value );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "task_set_note with illegal id"
  );
  puts( "TA1 - task_set_note - INVALID_ID" );

  status = task_start( 100, Task_1, 0 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "task_start with illegal id"
  );
  puts( "TA1 - task_start - INVALID_ID" );

  status = task_start( 0, Task_1, 0 );
  fatal_directive_status( 
    status,
    INCORRECT_STATE,
    "task_start of ready task"
  );
  puts( "TA1 - task_start - INCORRECT_STATE" );

  status = task_suspend( 100 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "task_suspend with illegal id"
  );
  puts( "TA1 - task_suspend - INVALID_ID" );
}
