/*  Screen10
 *
 *  This routine generates error screen 10 for test 9.
 *
 *  Input parameters:  NONE
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

void Screen10()
{
  rtems_status_code status;

  status = rate_monotonic_create( 0, &Junk_id );
  fatal_directive_status( 
    status,
    INVALID_NAME,
    "rate_monotonic_create with illegal name"
  );
  puts( "TA1 - rate_monotonic_create - INVALID_NAME" );

  status = rate_monotonic_create( Period_name[ 1 ], &Period_id[ 1 ] );
  directive_failed( status, "rate_monotonic_create successful" );
  puts( "TA1 - rate_monotonic_create - SUCCESSFUL" );

  status = rate_monotonic_create( Period_name[ 1 ], &Junk_id );
  fatal_directive_status( 
    status,
    TOO_MANY,
    "rate_monotonic_create of too many"
  );
  puts( "TA1 - rate_monotonic_create - TOO_MANY" );

  status = rate_monotonic_ident( 0, &Junk_id );
  fatal_directive_status( 
    status,
    INVALID_NAME,
    "rate_monotonic_ident with illegal name"
  );
  puts( "TA1 - rate_monotonic_ident - INVALID_NAME" );

  status = rate_monotonic_period( 100, 5 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "rate_monotonic_period with illegal id"
  );
  puts( "TA1 - rate_monotonic_period - unknown INVALID_ID" );

  status = rate_monotonic_period( 0x10100, 5 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "rate_monotonic_period with illegal id"
  );
  puts( "TA1 - rate_monotonic_period - local INVALID_ID" );

  status = rate_monotonic_period( Period_id[ 1 ], PERIOD_STATUS );
  fatal_directive_status( 
    status,
    NOT_DEFINED,
    "rate_monotonic_period status not defined"
  );
  puts( "TA1 - rate_monotonic_period(PERIOD_STATUS) - NOT_DEFINED" );

  status = rate_monotonic_period( Period_id[ 1 ], 100 );
  directive_failed( status, "rate_monotonic_period successful" );
  puts( "TA1 - rate_monotonic_period - 100 ticks - SUCCESSFUL" );

  status = rate_monotonic_period( Period_id[ 1 ], PERIOD_STATUS );
  directive_failed( status, "rate_monotonic_period status" );
  puts( "TA1 - rate_monotonic_period(PERIOD_STATUS) - SUCCESSFUL" );

  while ( FOREVER ) {

     status = rate_monotonic_period( Period_id[ 1 ], PERIOD_STATUS );

     if ( status == TIMEOUT ) break;
   
     directive_failed( 
       status, 
       "rate_monotonic_period waiting for timeout" 
     );
  }
  puts( "TA1 - rate_monotonic_period(PERIOD_STATUS) - TIMEOUT" );
  
  status = rate_monotonic_cancel( 100 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "rate_monotonic_cancel with illegal id"
  );
  puts( "TA1 - rate_monotonic_cancel - unknown INVALID_ID" );

  status = rate_monotonic_cancel( 0x10100 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "rate_monotonic_cancel will illegal id"
  );
  puts( "TA1 - rate_monotonic_cancel - local INVALID_ID" );

  status = rate_monotonic_cancel( Period_id[ 1 ] );
  directive_failed( status, "rate_monotonic_cancel" );
  puts( "TA1 - rate_monotonic_cancel - SUCCESSFUL" );

  status = rate_monotonic_period( Period_id[ 1 ], 5 );  
  directive_failed( status, "rate_monotonic_period restart" );

  status = task_wake_after( 1 * TICKS_PER_SECOND );
  directive_failed( status, "task_wake_after" );

  status = rate_monotonic_period( Period_id[ 1 ], 5 );
  fatal_directive_status( 
    status,
    TIMEOUT,
    "rate_monotonic_period"
  );
  puts( "TA1 - rate_monotonic_period - 5 ticks - TIMEOUT" );

  status = task_start( Task_id[ 4 ], Task_4, 0 );
  directive_failed( status, "task_start of TA4" );

  puts( "TA1 - yielding to TA4" );
  status = task_wake_after( YIELD_PROCESSOR );

  status = rate_monotonic_delete( 100 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "rate_monotonic_delete with illegal id"
  );
  puts( "TA1 - rate_monotonic_delete - unknown INVALID_ID" );

  status = rate_monotonic_delete( 0x10100 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "rate_monotonic_delete with illegal id"
  );
  puts( "TA1 - rate_monotonic_delete - local INVALID_ID" );

  status = rate_monotonic_delete( Period_id[ 1 ] );
  directive_failed( status, "rate_monotonic_delete" );
  puts( "TA1 - rate_monotonic_delete - SUCCESSFUL" );
}
