/*  Screen11
 *
 *  This routine generates error screen 11 for test 9.
 *
 *  Input parameters:  NONE
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

void Screen11()
{
  void              *buffer_address_1;
  void              *buffer_address_2;
  void              *buffer_address_3;
  rtems_status_code  status;

  status = partition_create( 
    0, 
    Partition_good_area,
    128, 
    40, 
    DEFAULT_ATTRIBUTES, 
    &Junk_id
  );
  fatal_directive_status( 
    status,
    INVALID_NAME,
    "partition_create with illegal name"
  );
  puts( "TA1 - partition_create - INVALID_NAME" );

  status = partition_create( 
    Partition_name[ 1 ], 
    Partition_good_area,
    0, 
    80, 
    DEFAULT_ATTRIBUTES, 
    &Junk_id
  );
  fatal_directive_status( 
    status,
    INVALID_SIZE,
    "partition_create with illegal length"
  );
  puts( "TA1 - partition_create - length - INVALID_SIZE" );

  status = partition_create( 
    Partition_name[ 1 ], 
    Partition_good_area,
    128, 
    0, 
    DEFAULT_ATTRIBUTES, 
    &Junk_id
  );
  fatal_directive_status( 
    status,
    INVALID_SIZE,
    "partition_create with illegal buffer size"
  );
  puts( "TA1 - partition_create - buffer size - INVALID_SIZE" );

  status = partition_create( 
    Partition_name[ 1 ], 
    Partition_good_area,
    128, 
    256, 
    DEFAULT_ATTRIBUTES, 
    &Junk_id
  );
  fatal_directive_status( 
    status,
    INVALID_SIZE,
    "partition_create with buffer_size > length"
  );
  puts( 
    "TA1 - partition_create - length < buffer size - INVALID_SIZE" 
  );

  status = partition_create( 
    Partition_name[ 1 ], 
    Partition_good_area,
    128, 
    64, 
    GLOBAL, 
    &Junk_id
  );
  fatal_directive_status( 
    status,
    MP_NOT_CONFIGURED,
    "partition_create of global"
  );
  puts( "TA1 - partition_create - MP_NOT_CONFIGURED" );

  status = partition_create( 
    Partition_name[ 1 ], 
    Partition_bad_area,
    128, 
    64, 
    GLOBAL, 
    &Junk_id
  );
  fatal_directive_status( 
    status,
    INVALID_ADDRESS,
    "partition_create with bad address"
  );
  puts( "TA1 - partition_create - INVALID_ADDRESS" );

  status = partition_create( 
    Partition_name[ 1 ], 
    Partition_good_area,
    128, 
    34, 
    DEFAULT_ATTRIBUTES, 
    &Junk_id
  );
  fatal_directive_status( 
    status,
    INVALID_SIZE,
    "partition_create with unaligned buffer_size"
  );
  puts( "TA1 - partition_create - INVALID_SIZE" );

  status = partition_delete( 100 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "partition_delete with illegal id"
  );
  puts( "TA1 - partition_delete - unknown INVALID_ID" );

  status = partition_delete( 0x10100 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "partition_delete with illegal id"
  );
  puts( "TA1 - partition_delete - local INVALID_ID" );

  status = partition_get_buffer( 100, &buffer_address_1 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "partition_get_buffer with illegal id"
  );
  puts( "TA1 - partition_get_buffer - INVALID_ID" );

  status = partition_ident( 0, SEARCH_ALL_NODES, &Junk_id );
  fatal_directive_status( 
    status,
    INVALID_NAME,
    "partition_ident with illegal name"
  );
  puts( "TA1 - partition_ident - INVALID_NAME" );

  status = partition_return_buffer( 100, buffer_address_1 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "partition_return_buffer with illegal id"
  );
  puts( "TA1 - partition_return_buffer - INVALID_ID" );

  status = partition_create( 
    Partition_name[ 1 ], 
    Partition_good_area,
    128, 
    64, 
    DEFAULT_ATTRIBUTES, 
    &Partition_id[ 1 ] 
  );
  directive_failed( status, "partition_create" );
  puts( "TA1 - partition_create - SUCCESSFUL" );

  status = partition_create( 
    Partition_name[ 1 ], 
    Partition_good_area,
    128, 
    32, 
    DEFAULT_ATTRIBUTES, 
    &Junk_id
  );
  fatal_directive_status( 
    status,
    TOO_MANY,
    "partition_create of too many"
  );
  puts( "TA1 - partition_create - TOO_MANY" );

  status = partition_get_buffer( Partition_id[ 1 ], &buffer_address_1 );
  directive_failed( status, "partition_get_buffer");
  puts( "TA1 - partition_get_buffer - SUCCESSFUL" );

  status = partition_get_buffer( Partition_id[ 1 ], &buffer_address_2 );
  directive_failed( status, "partition_get_buffer" );
  puts( "TA1 - partition_get_buffer - SUCCESSFUL" );

  status = partition_get_buffer( Partition_id[ 1 ], &buffer_address_3 );
  fatal_directive_status( 
    status,
    UNSATISFIED,
    "partition_get_buffer unsatisfied"
  );
  puts( "TA1 - partition_get_buffer - UNSATISFIED" );

  status = partition_delete( Partition_id[ 1 ] );
  fatal_directive_status( 
    status,
    RESOURCE_IN_USE,
    "partition_delete with buffers in use"
  );
  puts( "TA1 - partition_delete - RESOURCE_IN_USE" );

  status = partition_return_buffer( 
    Partition_id[ 1 ], 
    Region_good_area                  /* NOTE: Region Memory */
  );
  fatal_directive_status( 
    status,
    INVALID_ADDRESS,
    "partition_return_buffer with buffer address out of partition"
  );
  puts( 
    "TA1 - partition_return_buffer - INVALID_ADDRESS - out of range"
  );

  status = partition_return_buffer( 
    Partition_id[ 1 ],
    &Partition_good_area[ 7 ] 
  );
  fatal_directive_status( 
    status,
    INVALID_ADDRESS,
    "partition_return_buffer with buffer address not on boundary"
  );
  puts_nocr( "TA1 - partition_return_buffer - " );
  puts     ( "INVALID_ADDRESS - not on boundary");
}
