/*  Screen12
 *
 *  This routine generates error screen 12 for test 9.
 *
 *  Input parameters:  NONE
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

void Screen12()
{
  void              *segment_address_1;
  void              *segment_address_2;
  void              *segment_address_3;
  rtems_unsigned32   offset;
  rtems_unsigned32   good_front_flag;
  rtems_unsigned32   good_back_flag;
  rtems_status_code  status;

  status = region_create( 
    0, 
    Region_good_area,
    128, 
    32, 
    DEFAULT_ATTRIBUTES, 
    &Junk_id
  );
  fatal_directive_status( 
    status,
    INVALID_NAME,
    "region_create with illegal name"
  );
  puts( "TA1 - region_create - INVALID_NAME" );

  status = region_create( 
    Region_name[ 1 ], 
    Region_bad_area,
    128, 
    32, 
    DEFAULT_ATTRIBUTES, 
    &Junk_id
  );
  fatal_directive_status( 
    status,
    INVALID_ADDRESS,
    "region_create with illegal address"
  );
  puts( "TA1 - region_create - INVALID_ADDRESS" );

  status = region_create( 
    Region_name[ 1 ], 
    Region_good_area,
    128, 
    34, 
    DEFAULT_ATTRIBUTES, 
    &Junk_id
  );
  fatal_directive_status( 
    status,
    INVALID_SIZE,
    "region_create with illegal size"
  );
  puts( "TA1 - region_create - INVALID_SIZE" );

  status = region_create( 
    Region_name[ 1 ], 
    Region_good_area,
    512, 
    128, 
    DEFAULT_ATTRIBUTES, 
    &Region_id[ 1 ]
  );
  directive_failed( status, "region_create" );
  puts( "TA1 - region_create - SUCCESSFUL" );

  status = region_create(
    Region_name[ 1 ], 
    Region_good_area,
    512, 
    128, 
    DEFAULT_ATTRIBUTES, 
    &Junk_id
  );
  fatal_directive_status( 
    status,
    TOO_MANY,
    "region_create of too many"
  );
  puts( "TA1 - region_create - TOO_MANY" );

  status = region_delete( 100 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "region_delete with illegal id"
  );
  puts( "TA1 - region_delete - unknown INVALID_ID" );

  status = region_delete( 0x10100 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "region_delete with illegal id"
  );
  puts( "TA1 - region_delete - local INVALID_ID" );

  status = region_ident( 0, &Junk_id );
  fatal_directive_status( 
    status,
    INVALID_NAME,
    "region_ident with illegal name"
  );
  puts( "TA1 - region_ident - INVALID_NAME" );

  status = region_get_segment( 
    100, 
    128, 
    DEFAULT_OPTIONS, 
    NO_TIMEOUT, 
    &segment_address_1 
  );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "region_get_segment with illegal id"
  );
  puts( "TA1 - region_get_segment - INVALID_ID" );

  status = region_get_segment( 
     Region_id[ 1 ],
     sizeof( Region_good_area ) * 2,
     DEFAULT_OPTIONS,
     NO_TIMEOUT,
     &segment_address_1
  );
  fatal_directive_status( 
    status,
    INVALID_SIZE,
    "region_get_segment with illegal size"
  );
  puts( "TA1 - region_get_segment - INVALID_SIZE" );

  status = region_get_segment(
     Region_id[ 1 ],
     384,
     DEFAULT_OPTIONS,
     NO_TIMEOUT,
     &segment_address_1
  );
  directive_failed( status, "region_get_segment" );
  puts( "TA1 - region_get_segment - SUCCESSFUL" );

  status = region_get_segment(
     Region_id[ 1 ],
     384,
     NO_WAIT,
     NO_TIMEOUT,
     &segment_address_2
  );
  fatal_directive_status( 
    status,
    UNSATISFIED,
    "region_get_segment unsatisfied"
  );
  puts( "TA1 - region_get_segment - UNSATISFIED" );

  puts( "TA1 - region_get_segment - timeout in 3 seconds" );
  status = region_get_segment( 
    Region_id[ 1 ], 
    128, 
    DEFAULT_OPTIONS, 
    3 * TICKS_PER_SECOND, 
    &segment_address_3 
  );
  fatal_directive_status( 
    status,
    TIMEOUT,
    "region_get_segment timeout"
  );
  puts( "TA1 - region_get_segment - woke up with TIMEOUT" );

  status = region_delete( Region_id[ 1 ] );
  fatal_directive_status( 
    status,
    RESOURCE_IN_USE,
    "region_delete with buffers in use"
  );
  puts( "TA1 - region_delete - RESOURCE_IN_USE" );

  status = region_return_segment( 100, segment_address_1 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "region_return_segment with illegal id"
  );
  puts( "TA1 - region_return_segment - INVALID_ID" );

  status = region_return_segment( Region_id[ 1 ], Region_good_area );
  fatal_directive_status( 
    status,
    INVALID_ADDRESS,
    "region_return_segment with illegal segment"
  );
  puts( "TA1 - region_return_segment - INVALID_ADDRESS" );

/* 
 *  The following generate internal heap errors.  Thus this code
 *  is subject to change if the heap code changes.
 */

  offset = (segment_address_1 - (void *)Region_good_area) / 4;

/* bad FRONT_FLAG error */

  good_front_flag = Region_good_area[ offset - 1 ];
  Region_good_area[ offset - 1 ] = good_front_flag + 2;

  status = region_return_segment( Region_id[ 1 ], segment_address_1 );
  fatal_directive_status( 
    status,
    INVALID_ADDRESS,
    "region_return_segment with back_flag != front_flag"
  );
  puts( "TA1 - region_return_segment - INVALID_ADDRESS" );

  Region_good_area[ offset - 1 ] = good_front_flag;
  
/* bad FRONT_FLAG error */

  good_back_flag = Region_good_area[ offset - 2 ];
  Region_good_area[ offset - 2 ] = 1024;

  status = region_return_segment( Region_id[ 1 ], segment_address_1 );
  fatal_directive_status( 
    status,
    INVALID_ADDRESS,
    "region_return_segment with back_flag != front_flag"
  );
  puts( "TA1 - region_return_segment - INVALID_ADDRESS" );

  Region_good_area[ offset - 2 ] = good_back_flag;
}
