/*  Screen14
 *
 *  This routine generates error screen 14 for test 9.
 *
 *  Input parameters:  NONE
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

void Screen14()
{
  rtems_status_code  status;
  rtems_time_of_day  time;

  status = timer_create( 0, &Junk_id );
  fatal_directive_status( 
    status,
    INVALID_NAME,
    "timer_create with illegal name"
  );
  puts( "TA1 - timer_create - INVALID_NAME" );

  status = timer_create( Timer_name[ 1 ], &Timer_id[ 1 ] );
  directive_failed( status, "timer_create" );
  puts( "TA1 - timer_create - 1 - SUCCESSFUL" );

  status = timer_create( 2, &Junk_id );
  fatal_directive_status( 
    status,
    TOO_MANY,
    "timer_create for too many"
  );
  puts( "TA1 - timer_create - 2 - TOO_MANY" );

  status = timer_delete( 100 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "timer_delete with illegal id"
  );
  puts( "TA1 - timer_delete - local INVALID_ID" );

  status = timer_delete( 0x010100 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "timer_delete with illegal id"
  );
  puts( "TA1 - timer_delete - global INVALID_ID" );

  status = timer_ident( 0, &Junk_id );
  fatal_directive_status( 
    status,
    INVALID_NAME,
    "timer_ident with illegal name"
  );
  puts( "TA1 - timer_ident - INVALID_NAME" );

  status = timer_cancel( 0x010100 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "timer_cancel with illegal id"
  );
  puts( "TA1 - timer_cancel - INVALID_ID" );

  status = timer_cancel( Timer_id[ 1 ] );
  fatal_directive_status( 
    status,
    INCORRECT_STATE,
    "timer_cancel before initiated"
  );
  puts( "TA1 - timer_cancel - INCORRECT_STATE" );

  status = timer_reset( 0x010100 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "timer_reset with illegal id"
  );
  puts( "TA1 - timer_reset - INVALID_ID" );

  status = timer_reset( Timer_id[ 1 ] );
  fatal_directive_status( 
    status,
    NOT_DEFINED,
    "timer_reset before initiated"
  );
  puts( "TA1 - timer_reset - NOT_DEFINED" );

  status = timer_fire_after( 
    0x010100, 
    5 * TICKS_PER_SECOND, 
    Delayed_routine,
    NULL
  );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "timer_fire_after illegal id"
  );
  puts( "TA1 - timer_fire_after - INVALID_ID" );

  build_time( &time, 12, 31, 1994, 9, 0, 0, 0 );
  status = timer_fire_when( 0x010100, &time, Delayed_routine, NULL );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "timer_fire_when with illegal id"
  );
  puts( "TA1 - timer_fire_when - INVALID_ID" );

  status = timer_fire_after( Timer_id[ 1 ], 0, Delayed_routine, NULL );
  fatal_directive_status( 
    status,
    INVALID_NUMBER,
    "timer_fire_after with 0 ticks"
  );
  puts( "TA1 - timer_fire_after - INVALID_NUMBER" );
  
  build_time( &time, 2, 5, 1987, 8, 30, 45, 0 );
  status = timer_fire_when( Timer_id[ 1 ], &time, Delayed_routine, NULL );
  fatal_directive_status( 
    status,
    INVALID_TIME_OF_DAY,
    "timer_fire_when with illegal time"
  );
  print_time( 
    "TA1 - timer_fire_when - ", 
    &time, 
    " - invalid INVALID_TIME_OF_DAY\n" 
  );

  status = clock_get( &time );
  directive_failed( status, "clock_set" );
  print_time( "TA1 - clock_get       - ", &time, "\n" );

  build_time( &time, 2, 5, 1990, 8, 30, 45, 0 );
  status = timer_fire_when( Timer_id[ 1 ], &time, Delayed_routine, NULL );
  fatal_directive_status( 
    status,
    INVALID_TIME_OF_DAY,
    "timer_fire_when before current time"
  );
  print_time( 
    "TA1 - timer_fire_when - ", 
    &time, 
    " - invalid INVALID_TIME_OF_DAY\n" 
  );
}
