/*  Screen2
 *
 *  This routine generates error screen 2 for test 9.
 *
 *  Input parameters:  NONE
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

void Screen2()
{
  rtems_time_of_day time;
  rtems_status_code status;

/* errors before clock is set */

  status = clock_get( &time );
  fatal_directive_status( 
    status,
    NOT_DEFINED,
    "clock_get before clock is set"
  );
  puts( "TA1 - clock_get - NOT_DEFINED" );

  status = task_wake_when( &time );
  fatal_directive_status( 
    status,
    NOT_DEFINED,
    "task_wake_when before clock is set"
  );
  puts( "TA1 - task_wake_when - NOT_DEFINED" );

  status = timer_fire_when( 0, &time, Delayed_routine, NULL );
  fatal_directive_status( 
    status,
    NOT_DEFINED,
    "task_fire_when before clock is set"
  );
  puts( "TA1 - timer_fire_when - NOT_DEFINED" );

  build_time( &time, 2, 5, 1987, 8, 30, 45, 0 );
  print_time( "TA1 - clock_set - ", &time, "" );
  status = clock_set( &time );
  fatal_directive_status( 
    status,
    INVALID_TIME_OF_DAY,
    "clock_set with invalid year"
  );
  puts( " - INVALID_TIME_OF_DAY" );

  build_time( &time, 15, 5, 1988, 8, 30, 45, 0 );
  print_time( "TA1 - clock_set - ", &time, "" );
  status = clock_set( &time );
  fatal_directive_status( 
    status,
    INVALID_TIME_OF_DAY,
    "clock_set with invalid month"
  );
  puts( " - INVALID_TIME_OF_DAY" );

  build_time( &time, 2, 32, 1988, 8, 30, 45, 0 );
  print_time( "TA1 - clock_set - ", &time, "" );
  status = clock_set( &time );
  fatal_directive_status( 
    status,
    INVALID_TIME_OF_DAY,
    "clock_set with invalid day"
  );
  puts( " - INVALID_TIME_OF_DAY" );

  build_time( &time, 2, 5, 1988, 25, 30, 45, 0 );
  print_time( "TA1 - clock_set - ", &time, "" );
  status = clock_set( &time );
  fatal_directive_status( 
    status,
    INVALID_TIME_OF_DAY,
    "clock_set with invalid hour"
  );
  puts( " - INVALID_TIME_OF_DAY" );

  build_time( &time, 2, 5, 1988, 8, 61, 45, 0 );
  print_time( "TA1 - clock_set - ", &time, "" );
  status = clock_set( &time );
  fatal_directive_status( 
    status,
    INVALID_TIME_OF_DAY,
    "clock_set with invalid minute"
  );
  puts( " - INVALID_TIME_OF_DAY" );

  build_time( &time, 2, 5, 1988, 8, 30, 61, 0 );
  print_time( "TA1 - clock_set - ", &time, "" );
  status = clock_set( &time );
  fatal_directive_status( 
    status,
    INVALID_TIME_OF_DAY,
    "clock_set with invalid second"
  );
  puts( " - INVALID_TIME_OF_DAY" );

  build_time( &time, 2, 5, 1988, 8, 30, 45, TICKS_PER_SECOND + 1 );
  print_time( "TA1 - clock_set - ", &time, "" );
  status = clock_set( &time );
  fatal_directive_status( 
    status,
    INVALID_TIME_OF_DAY,
    "clock_set with invalid ticks per second"
  );
  puts( " - INVALID_TIME_OF_DAY" );

  build_time( &time, 2, 5, 1988, 8, 30, 45, 0 );
  print_time( "TA1 - clock_set - ", &time, "" );
  status = clock_set( &time );
  directive_failed( status, "clock_set successful" );
  puts( " - SUCCESSFUL" );

/* task_wake_when */

  build_time( &time, 2, 5, 1988, 8, 30, 48, TICKS_PER_SECOND + 1 );
  time.second += 3;
  puts( "TA1 - task_wake_when - TICKINVALID - sleep about 3 seconds" );

  status = task_wake_when( &time );
  directive_failed( 
    status, 
    "task_wake_when with invalid ticks per second" 
  );
  puts( "TA1 - task_wake_when - TICKINVALID - woke up SUCCESSFUL" );

  build_time( &time, 2, 5, 1961, 8, 30, 48, 0 );
  print_time( "TA1 - task_wake_when - ", &time, "" );
  status = task_wake_when( &time );
  fatal_directive_status( 
    status,
    INVALID_TIME_OF_DAY,
    "task_wake_when with invalid year"
  );
  puts( " - INVALID_TIME_OF_DAY" );

  build_time( &time, 2, 5, 1988, 25, 30, 48, 0 );
  print_time( "TA1 - task_wake_when - ", &time, "" );
  status = task_wake_when( &time );
  fatal_directive_status( 
    status,
    INVALID_TIME_OF_DAY,
    "task_wake_when with invalid hour"
  );
  puts( " - INVALID_TIME_OF_DAY" );

  clock_get( &time );
  print_time( "TA1 - current time - ", &time, "\n" );

  time.month = 1;
  print_time( "TA1 - task_wake_when - ", &time, "" );
  status = task_wake_when( &time );
  fatal_directive_status( 
    status,
    INVALID_TIME_OF_DAY,
    "task_wake_when before current time"
  );
  puts( " -  INVALID_TIME_OF_DAY" );
}
