/*  Screen3
 *
 *  This routine generates error screen 3 for test 9.
 *
 *  Input parameters:  NONE
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

extern rtems_configuration_table Configuration;

void Screen3()
{
  rtems_name        task_name;
  rtems_status_code status;

  task_name = 1;
  status = task_create( 
    0, 
    1, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Junk_id 
  );
  fatal_directive_status( 
    status,
    INVALID_NAME,
    "task_create with illegal name"
  );
  puts( "TA1 - task_create - INVALID_NAME" );

  status = task_create(
    task_name, 
    1, 
    Configuration.work_space_size, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Junk_id
  );
  fatal_directive_status( 
    status,
    UNSATISFIED,
    "task_create with a stack size larger than the workspace"
  );
  puts( "TA1 - task_create - stack size - UNSATISFIED" );

  status = task_create( 
    Task_name[ 2 ], 
    4, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 2 ]
  );
  directive_failed( status, "task_create of TA2" );
  puts( "TA1 - task_create - TA2 created - SUCCESSFUL" );

  status = task_suspend( Task_id[ 2 ] );
  directive_failed( status, "task_suspend of TA2" );
  puts( "TA1 - task_suspend - suspend TA2 - SUCCESSFUL" );

  status = task_suspend( Task_id[ 2 ] );
  fatal_directive_status( 
    status,
    ALREADY_SUSPENDED,
    "task_suspend of suspended TA2"
  );
  puts( "TA1 - task_suspend - suspend TA2 - ALREADY_SUSPENDED" );

  status = task_resume( Task_id[ 2 ] );
  directive_failed( status, "task_resume of TA2" );
  puts( "TA1 - task_resume - TA2 resumed - SUCCESSFUL" );

  status = task_create( 
    Task_name[ 3 ], 
    4, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 3 ]
  );
  directive_failed( status, "task_create of TA3" );
  puts( "TA1 - task_create - TA3 created - SUCCESSFUL" );

  status = task_create(
    Task_name[ 4 ], 
    4, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 4 ]
  );
  directive_failed( status, "task_create of TA4" );
  puts( "TA1 - task_create - 4 created - SUCCESSFUL" );

  status = task_create( 
    Task_name[ 5 ], 
    4, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 5 ]
  );
  directive_failed( status, "task_create of TA5" );
  puts( "TA1 - task_create - 5 created - SUCCESSFUL" );

  status = task_create( 
    Task_name[ 6 ], 
    4, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 6 ]
  );
  directive_failed( status, "task_create of TA6" );
  puts( "TA1 - task_create - 6 created - SUCCESSFUL" );

  status = task_create( 
    Task_name[ 7 ], 
    4, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 7 ]
  );
  directive_failed( status, "task_create of TA7" );
  puts( "TA1 - task_create - 7 created - SUCCESSFUL" );

  status = task_create( 
    Task_name[ 8 ], 
    4, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 8 ]
  );
  directive_failed( status, "task_create of TA8" );
  puts( "TA1 - task_create - 8 created - SUCCESSFUL" );

  status = task_create(
    Task_name[ 9 ], 
    4, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 9 ]
  );
  directive_failed( status, "task_create of TA9" );
  puts( "TA1 - task_create - 9 created - SUCCESSFUL" );

  status = task_create( 
    Task_name[ 10 ], 
    4, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 10 ]
  );
  directive_failed( status, "task_create of TA10" );
  puts( "TA1 - task_create - 10 created - SUCCESSFUL" );

  status = task_create( 
    task_name, 
    4, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Junk_id
  );
  fatal_directive_status( 
    status, 
    TOO_MANY,
    "task_create for too many tasks"
  );
  puts( "TA1 - task_create - 11 - TOO_MANY" );

  status = task_create( 
    task_name, 
    4, 
    2048, 
    DEFAULT_MODES, 
    GLOBAL, 
    &Junk_id 
  );
  fatal_directive_status( 
    status,
    MP_NOT_CONFIGURED,
    "task_create of global task in a single cpu system"
  );
  puts( "TA1 - task_create - MP_NOT_CONFIGURED" );
}
