/*  Screen5
 *
 *  This routine generates error screen 5 for test 9.
 *
 *  Input parameters:  NONE
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

void Screen5()
{
  rtems_status_code status;

  status = semaphore_create( 0, 1, DEFAULT_ATTRIBUTES, &Junk_id );
  fatal_directive_status( 
    status,
    INVALID_NAME,
    "semaphore_create with illegal name"
  );
  puts( "TA1 - semaphore_create - INVALID_NAME" );

  status = semaphore_create( 
    Semaphore_name[ 1 ], 
    1, 
    DEFAULT_ATTRIBUTES, 
    &Semaphore_id[ 1 ] 
  );
  directive_failed( status, "semaphore_create" );
  puts( "TA1 - semaphore_create - 1 - SUCCESSFUL" );

  status = semaphore_create( 
    Semaphore_name[ 2 ], 
    1, 
    BINARY_SEMAPHORE, 
    &Semaphore_id[ 2 ] 
  );
  directive_failed( status, "semaphore_create" );
  puts( "TA1 - semaphore_create - 2 - SUCCESSFUL" );

  status = semaphore_create( 
     Semaphore_name[ 3 ], 
     1, 
     DEFAULT_ATTRIBUTES, 
     &Junk_id 
  );
  fatal_directive_status( 
    status,
    TOO_MANY,
    "semaphore_create of too many"
  );
  puts( "TA1 - semaphore_create - 3 - TOO_MANY" );

  status = semaphore_create( 
    Semaphore_name[ 1 ], 
    1, 
    INHERIT_PRIORITY | BINARY_SEMAPHORE | FIFO, 
    &Junk_id 
  );
  fatal_directive_status( 
    status,
    NOT_DEFINED,
    "semaphore_create of FIFO INHERIT_PRIORITY"
  );
  puts( "TA1 - semaphore_create - NOT_DEFINED" );

  status = semaphore_create( 
    Semaphore_name[ 1 ], 
    1, 
    INHERIT_PRIORITY | COUNTING_SEMAPHORE | PRIORITY, 
    &Junk_id 
  );
  fatal_directive_status( 
    status,
    NOT_DEFINED,
    "semaphore_create of COUNTING_SEMAPHORE INHERIT_PRIORITY"
  );
  puts( "TA1 - semaphore_create - NOT_DEFINED" );

  status = semaphore_create( 
    Semaphore_name[ 1 ], 
    2, 
    BINARY_SEMAPHORE, 
    &Junk_id 
  );
  fatal_directive_status( 
    status,
    INVALID_NUMBER,
    "semaphore_create of binary semaphore with count > 1"
  );
  puts( "TA1 - semaphore_create - INVALID_NUMBER" );

  status = semaphore_create( Semaphore_name[ 3 ], 1, GLOBAL, &Junk_id );
  fatal_directive_status( 
    status,
    MP_NOT_CONFIGURED,
    "semaphore_create of mp not configured"
  );
  puts( "TA1 - semaphore_create - MP_NOT_CONFIGURED" );

  status = semaphore_delete( 100 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "semaphore_delete with illegal id"
  );
  puts( "TA1 - semaphore_delete - unknown INVALID_ID" );

  status = semaphore_delete( 0x010100 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "semaphore_delete with local illegal id"
  );
  puts( "TA1 - semaphore_delete - local INVALID_ID" );

  status = semaphore_ident( 100, SEARCH_ALL_NODES, &Junk_id );
  fatal_directive_status( 
    status,
    INVALID_NAME,
    "semaphore_ident will illegal name (local)"
  );
  puts( "TA1 - semaphore_ident - global INVALID_NAME" );

  status = semaphore_ident( 100, 1, &Junk_id );
  fatal_directive_status( 
    status,
    INVALID_NAME,
    "semaphore_ident will illegal name (global)"
  );
  puts( "TA1 - semaphore_ident - local INVALID_NAME" );
}
