/*  Screen6
 *
 *  This routine generates error screen 6 for test 9.
 *
 *  Input parameters:  NONE
 *
 *  Output parameters:  NONE
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"

void Screen6()
{
  rtems_status_code status;

  status = semaphore_obtain( 100, DEFAULT_OPTIONS, NO_TIMEOUT );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "semaphore_obtain with illegal id"
  );
  puts( "TA1 - semaphore_obtain - INVALID_ID" );

  status = semaphore_obtain( 
    Semaphore_id[ 1 ], 
    DEFAULT_OPTIONS, 
    NO_TIMEOUT 
  );
  directive_failed( status, "semaphore_obtain successful" );
  puts( "TA1 - semaphore_obtain - got sem 1 - SUCCESSFUL" );

  status = semaphore_obtain( 
    Semaphore_id[ 1 ], 
    NO_WAIT, 
    NO_TIMEOUT 
  );
  fatal_directive_status( 
    status,
    UNSATISFIED,
    "semaphore_obtain not available"
  );
  puts( "TA1 - semaphore_obtain - UNSATISFIED" );

  puts( "TA1 - semaphore_obtain - timeout in 3 seconds" );
  status = semaphore_obtain( 
    Semaphore_id[ 1 ], 
    DEFAULT_OPTIONS, 
    3 * TICKS_PER_SECOND 
  );
  fatal_directive_status( 
    status,
    TIMEOUT,
    "semaphore_obtain timeout"
  );
  puts( "TA1 - semaphore_obtain - woke up with TIMEOUT" );

  status = semaphore_release( Semaphore_id[ 2 ] );
  fatal_directive_status( 
    status,
    NOT_OWNER_OF_RESOURCE,
    "semaphore_release and not owner"
  );
  puts( "TA1 - semaphore_release - NOT_OWNER_OF_RESOURCE" );

  status = semaphore_release( 100 );
  fatal_directive_status( 
    status,
    INVALID_ID,
    "semaphore_release with illegal id"
  );
  puts( "TA1 - semaphore_release - INVALID_ID" );

  puts( "TA1 - task_start - start TA2 - SUCCESSFUL" );
  status = task_start( Task_id[ 2 ], Task_2, 0 );
  directive_failed( status, "task_start of TA2" );

  puts( "TA1 - task_wake_after - yield processor - SUCCESSFUL" );
  status = task_wake_after( YIELD_PROCESSOR );
  directive_failed( status, "task_wake_after (yield)" );

  puts( "TA1 - semaphore_delete - delete sem 1 - SUCCESSFUL" );
  status = semaphore_delete( Semaphore_id[ 1 ] );
  directive_failed( status, "semaphore_delete of SM1" );

  puts( "TA1 - semaphore_obtain - binary semaphore" );
  status = semaphore_obtain( 
    Semaphore_id[ 2 ], 
    DEFAULT_OPTIONS, 
    NO_TIMEOUT
  );
  directive_failed( status, "semaphore_obtain");

  puts( "TA1 - semaphore_delete - delete sem 2 - RESOURCE_IN_USE" );
  status = semaphore_delete( Semaphore_id[ 2 ] );
  fatal_directive_status( 
     status, 
     RESOURCE_IN_USE,
     "semaphore_delete of SM2" 
  );

  puts( "TA1 - task_wake_after - yield processor - SUCCESSFUL" );
  status = task_wake_after( YIELD_PROCESSOR );
  directive_failed( status, "task_wake_after (yield)" );

  status = task_delete( Task_id[ 2 ] );
  fatal_directive_status(
    status,
    INVALID_ID,
    "task_delete after the task has been deleted"
  );
}
