/*
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#undef EXTERN
#define EXTERN
#include "conftbl.h"
#include "gvar.h"

rtems_unsigned32 time_set, eventout;

rtems_task High_tasks(
  rtems_task_argument argument
);
 
rtems_task Low_task(
  rtems_task_argument argument
);

void test_init();

rtems_task Init(
  rtems_task_argument argument
)
{
  rtems_status_code status;

  puts( "\n\n*** TIME TEST 15 ***" );

  test_init();

  status = task_delete( SELF );
  directive_failed( status, "task_delete of SELF" );
}

void test_init()
{
  rtems_id          id;
  rtems_unsigned32  index;
  rtems_event_set   event_out;
  rtems_status_code status;

  time_set = FALSE;

  status = task_create( 
    build_name( 'L', 'O', 'W', ' ' ),
    10, 
    1024, 
    NO_PREEMPT, 
    DEFAULT_ATTRIBUTES, 
    &id 
  );
  directive_failed( status, "task_create LOW" );

  status = task_start( id, Low_task, 0 );
  directive_failed( status, "task_start LOW" );

  for ( index = 1 ; index <= OPERATION_COUNT ; index++ ) {
    status = task_create( 
      build_name( 'H', 'I', 'G', 'H' ),
      5, 
      1024, 
      DEFAULT_MODES, 
      DEFAULT_ATTRIBUTES, 
      &Task_id[ index ] 
    );
    directive_failed( status, "task_create LOOP" );

    status = task_start( Task_id[ index ], High_tasks, 0 );
    directive_failed( status, "task_start LOOP" );
  }

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) Empty_function();
  overhead = Read_timer();

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) event_receive( 
               PENDING_EVENTS, 
               DEFAULT_OPTIONS, 
               NO_TIMEOUT, 
               &event_out 
             );
  end_time = Read_timer();

  put_time(
    "EVENT_RECEIVE (current)",
    end_time,
    OPERATION_COUNT,
    overhead,
    CALLING_OVERHEAD_EVENT_RECEIVE
  );


  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) event_receive( 
               ALL_EVENTS, 
               NO_WAIT, 
               NO_TIMEOUT, 
               &event_out
             );
  end_time = Read_timer();

  put_time(
    "EVENT_RECEIVE (NO_WAIT)",
    end_time,
    OPERATION_COUNT,
    overhead,
    CALLING_OVERHEAD_EVENT_RECEIVE
  );
}

rtems_task Low_task(
  rtems_task_argument argument
) 
{
  rtems_unsigned32  index;
  rtems_event_set   event_out;

  end_time = Read_timer();

  put_time(
    "EVENT_RECEIVE (blocking)",
    end_time,
    OPERATION_COUNT,
    0,
    CALLING_OVERHEAD_EVENT_RECEIVE
  );

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) Empty_function();
  overhead = Read_timer();

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) event_send( SELF, EVENT_16 );
  end_time = Read_timer();

  put_time(
    "EVENT_SEND (non-blocking)",
    end_time,
    OPERATION_COUNT,
    overhead,
    CALLING_OVERHEAD_EVENT_SEND
  );

  Timer_initialize();            
    (void) event_receive( 
             EVENT_16, 
             DEFAULT_OPTIONS, 
             NO_TIMEOUT, 
             &event_out 
           );
  end_time = Read_timer();

  put_time(
    "EVENT_RECEIVE (available)",
    end_time,
    1,
    0,
    CALLING_OVERHEAD_EVENT_RECEIVE
  );

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) event_send( Task_id[ index ], EVENT_16 );
  end_time = Read_timer();

  put_time(
    "EVENT_SEND (readying)",
    end_time,
    OPERATION_COUNT,
    overhead,
    CALLING_OVERHEAD_EVENT_SEND
  );

  exit( 0 );
}

rtems_task High_tasks(
  rtems_task_argument argument
) 
{
  rtems_status_code status;

  if ( time_set ) 
    status = event_receive( EVENT_16, DEFAULT_OPTIONS, NO_TIMEOUT, &eventout );
  else {
    time_set = 1;
    Timer_initialize();            /* start blocking event_receive time */
    status = event_receive( EVENT_16, DEFAULT_OPTIONS, NO_TIMEOUT, &eventout );
  }
}
