/*
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#undef EXTERN
#define EXTERN
#include "conftbl.h"
#include "gvar.h"

rtems_device_major_number _STUB_major = 0;

rtems_id         Region_id;
rtems_name       Region_name;
rtems_unsigned8  Region_area[ 2048 ];

rtems_id         Partition_id;
rtems_name       Partition_name;
rtems_unsigned8  Partition_area[ 2048 ];

void            *Buffer_address_1;
void            *Buffer_address_2;
void            *Buffer_address_3;
void            *Buffer_address_4;

rtems_unsigned32 buffer_count;

void            *Buffer_addresses[ OPERATION_COUNT+1 ];

rtems_task Task_1(
  rtems_task_argument argument
);
 
rtems_task Task_2(
  rtems_task_argument argument
);

rtems_task Init(
  rtems_task_argument argument
)
{
  rtems_status_code status;

  puts( "\n\n*** TIME TEST 20 ***" );

  status = task_create( 
    build_name( 'T', 'I', 'M', '1' ),
    128, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 1 ] 
  );
  directive_failed( status, "task_create of TASK1" );

  status = task_start( Task_id[ 1 ], Task_1, 0 );
  directive_failed( status, "task_start of TASK1" );

  status = task_create( 
    build_name( 'T', 'I', 'M', '2' ),
    129, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &Task_id[ 2 ] 
  );
  directive_failed( status, "task_create of TASK2" );

  status = task_start( Task_id[ 2 ], Task_2, 0 );
  directive_failed( status, "task_start of TASK2" );

  status = task_delete( SELF );
  directive_failed( status, "task_delete of SELF" );
}

rtems_task Task_1(
  rtems_task_argument argument
)
{
  rtems_unsigned32    index;
  rtems_mode          previous_mode;
  rtems_task_priority previous_priority;
  rtems_unsigned32    return_value;
  rtems_status_code   status;

  Partition_name = build_name( 'P', 'A', 'R', 'T' );

  Timer_initialize();
    partition_create( 
      Partition_name, 
      Partition_area,
      2048, 
      128, 
      DEFAULT_ATTRIBUTES, 
      &Partition_id 
    );
  end_time = Read_timer();

  put_time(
    "PARTITION_CREATE",
    end_time,
    1,
    0,
    CALLING_OVERHEAD_PARTITION_CREATE
  );

  Region_name = build_name( 'R', 'E', 'G', 'N' );

  Timer_initialize();
    region_create( 
      Region_name, 
      Region_area,
      2048, 
      16, 
      DEFAULT_ATTRIBUTES, 
      &Region_id 
    );
  end_time = Read_timer();

  put_time(
    "REGION_CREATE",
    end_time,
    1,
    0,
    CALLING_OVERHEAD_REGION_CREATE
  );

  Timer_initialize();
    (void) partition_get_buffer( Partition_id, &Buffer_address_1 );
  end_time = Read_timer();

  put_time(
    "PARTITION_GET_BUFFER (available)",
    end_time,
    1,
    0,
    CALLING_OVERHEAD_PARTITION_GET_BUFFER
  );

  buffer_count = 0;
  while ( FOREVER ) {

    status = partition_get_buffer( 
               Partition_id, 
               &Buffer_addresses[ buffer_count ]
            );

    if ( status == UNSATISFIED ) break;

    buffer_count++;
  }

  Timer_initialize();
    (void) partition_get_buffer( Partition_id, &Buffer_address_2 );
  end_time = Read_timer();

  put_time(
    "PARTITION_GET_BUFFER (not available)",
    end_time,
    1,
    0,
    CALLING_OVERHEAD_PARTITION_GET_BUFFER
  );

  Timer_initialize();
    (void) partition_return_buffer( Partition_id, Buffer_address_1 );
  end_time = Read_timer();

  put_time(
    "PARTITION_RETURN_BUFFER",
    end_time,
    1,
    0,
    CALLING_OVERHEAD_PARTITION_RETURN_BUFFER
  );

  for ( index = 0 ; index < buffer_count ; index++ ) {
    
    status = partition_return_buffer( 
               Partition_id, 
               Buffer_addresses[ index ] 
             );
    directive_failed( status, "partition_return_buffer" );

  }

  Timer_initialize();
    (void) partition_delete( Partition_id );
  end_time = Read_timer();

  put_time(
    "PARTITION_DELETE",
    end_time,
    1,
    0,
    CALLING_OVERHEAD_PARTITION_DELETE
  );

  status = region_get_segment( 
             Region_id, 
             400, 
             DEFAULT_OPTIONS, 
             NO_TIMEOUT, 
             &Buffer_address_2 
           );
  directive_failed( status, "region_get_semgent" );

  Timer_initialize();
    (void) region_get_segment( 
      Region_id, 
      400, 
      DEFAULT_OPTIONS, 
      NO_TIMEOUT, 
      &Buffer_address_3 
    );
  end_time = Read_timer();

  put_time(
    "REGION_GET_SEGMENT (available)",
    end_time,
    1,
    0,
    CALLING_OVERHEAD_REGION_GET_SEGMENT
  );

  Timer_initialize();
    (void) region_get_segment( 
      Region_id, 
      1998, 
      NO_WAIT, 
      NO_TIMEOUT, 
      &Buffer_address_4 
    );
  end_time = Read_timer();

  put_time(
    "REGION_GET_SEGMENT (NO_WAIT)",
    end_time,
    1,
    0,
    CALLING_OVERHEAD_REGION_GET_SEGMENT
  );

  status = region_return_segment( Region_id, Buffer_address_3 );
  directive_failed( status, "region_return_segment" );

  Timer_initialize();
    (void) region_return_segment( Region_id, Buffer_address_2 );
  end_time = Read_timer();

  put_time(
    "REGION_RETURN_SEGMENT (no tasks waiting)",
    end_time,
    1,
    0,
    CALLING_OVERHEAD_REGION_RETURN_SEGMENT
  );

  status = region_get_segment( 
    Region_id, 
    400, 
    DEFAULT_OPTIONS, 
    NO_TIMEOUT, 
    &Buffer_address_1 
  );
  directive_failed( status, "region_get_segment" );

  Timer_initialize();
    (void) region_get_segment( 
      Region_id, 
      1998, 
      DEFAULT_OPTIONS, 
      NO_TIMEOUT, 
      &Buffer_address_2 
    );

  /* execute Task_2 */

  end_time = Read_timer();

  put_time(
    "REGION_RETURN_SEGMENT (preempt)",
    end_time,
    1,
    0,
    CALLING_OVERHEAD_REGION_RETURN_SEGMENT
  );

  status = region_return_segment( Region_id, Buffer_address_2 );
  directive_failed( status, "region_return_segment" );

  status = task_mode( NO_PREEMPT, PREEMPT_MASK, &previous_mode );
  directive_failed( status, "task_mode" );

  status = task_set_priority( SELF, 254, &previous_priority );  
  directive_failed( status, "task_set_priority" );

  status = region_get_segment( 
    Region_id, 
    400, 
    DEFAULT_OPTIONS, 
    NO_TIMEOUT, 
    &Buffer_address_1 
  );
  directive_failed( status, "region_return_segment" );

  status = region_get_segment( 
    Region_id, 
    1998, 
    DEFAULT_OPTIONS, 
    NO_TIMEOUT, 
    &Buffer_address_2 
  );
  directive_failed( status, "region_get_segment" );

  /* execute Task_2 */

  status = region_return_segment( Region_id, Buffer_address_2 );
  directive_failed( status, "region_return_segment" );

  Timer_initialize();
    (void) region_delete( Region_id );
  end_time = Read_timer();

  put_time(
    "REGION_DELETE",
    end_time,
    1,
    0,
    CALLING_OVERHEAD_REGION_DELETE
  );

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) Empty_function();
  overhead = Read_timer();

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) io_initialize( 0, 0, NULL, &return_value );
  end_time = Read_timer();

  put_time(
    "IO_INITIALIZE",
    end_time,
    OPERATION_COUNT,
    overhead,
    CALLING_OVERHEAD_IO_INITIALIZE
  );

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) io_open( 0, 0, NULL, &return_value );
  end_time = Read_timer();

  put_time(
    "IO_OPEN",
    end_time,
    OPERATION_COUNT,
    overhead,
    CALLING_OVERHEAD_IO_OPEN
  );

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) io_close( 0, 0, NULL, &return_value );
  end_time = Read_timer();

  put_time(
    "IO_CLOSE",
    end_time,
    OPERATION_COUNT,
    overhead,
    CALLING_OVERHEAD_IO_CLOSE
  );

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) io_read( 0, 0, NULL, &return_value );
  end_time = Read_timer();

  put_time(
    "IO_READ",
    end_time,
    OPERATION_COUNT,
    overhead,
    CALLING_OVERHEAD_IO_READ
  );

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) io_write( 0, 0, NULL, &return_value );
  end_time = Read_timer();

  put_time(
    "IO_WRITE",
    end_time,
    OPERATION_COUNT,
    overhead,
    CALLING_OVERHEAD_IO_WRITE
  );

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) io_control( 0, 0, NULL, &return_value );
  end_time = Read_timer();

  put_time(
    "IO_CONTROL",
    end_time,
    OPERATION_COUNT,
    overhead,
    CALLING_OVERHEAD_IO_CONTROL
  );

  exit( 0 );
}

rtems_task Task_2(
  rtems_task_argument argument
)
{
  rtems_status_code status;

  end_time = Read_timer();

  put_time(
    "REGION_GET_SEGMENT (blocking)",
    end_time,
    1,
    0,
    CALLING_OVERHEAD_REGION_GET_SEGMENT
  );

  Timer_initialize();
    (void) region_return_segment( Region_id, Buffer_address_1 );

  /* preempt back to Task_1 */

  Timer_initialize();
    (void) region_return_segment( Region_id, Buffer_address_1 );
  end_time = Read_timer();

  put_time(
    "REGION_RETURN_SEGMENT (ready -- return)",
    end_time,
    1,
    0,
    CALLING_OVERHEAD_REGION_RETURN_SEGMENT
  );

  status = task_delete( SELF );
  directive_failed( status, "task_delete of SELF" );
}
