/*
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#undef EXTERN
#define EXTERN
#include "conftbl.h"
#include "gvar.h"

rtems_id Queue_id;

long Buffer[4];

rtems_task Low_task(
  rtems_task_argument argument
);
 
rtems_task High_task(
  rtems_task_argument argument
);
 
rtems_task Preempt_task(
  rtems_task_argument argument
);

rtems_task Init(
  rtems_task_argument argument
)
{ 
  rtems_id          id;
  rtems_status_code status;

  puts( "\n\n*** TIME TEST 22 ***" );

  status = message_queue_create( 
    build_name( 'M', 'Q', '1', ' '),
    0,
    DEFAULT_ATTRIBUTES,
    &Queue_id
  );
  directive_failed( status, "message_queue_create" );

  status = task_create( 
    build_name( 'L', 'O', 'W', ' ' ),
    10, 
    2048, 
    NO_PREEMPT, 
    DEFAULT_ATTRIBUTES, 
    &id 
  ); 
  directive_failed( status, "task_create" );

  status = task_start( id, Low_task, 0 );
  directive_failed( status, "task_start LOW" );

  status = task_create( 
    1, 
    11, 
    1024, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &id 
  ); 
  directive_failed( status, "task_create PREEMPT" );

  status = task_start( id, Preempt_task, 0 );
  directive_failed( status, "task_start PREEMPT" );

  status = task_delete( SELF );
  directive_failed( status, "task_delete of SELF" );
}

rtems_task High_task(
  rtems_task_argument argument
) 
{
  rtems_unsigned32  count; 
  rtems_status_code status;
  
  Timer_initialize();
    (void) message_queue_broadcast( 
             Queue_id, 
             (long (*)[4]) Buffer, 
             &count 
           );
  end_time = Read_timer();

  put_time(
    "MESSAGE_QUEUE_BROADCAST (readying)",
    end_time,
    1,
    0,
    CALLING_OVERHEAD_MESSAGE_QUEUE_BROADCAST
  );

  status = task_suspend(SELF);
  directive_failed( status, "task_suspend" );
}

rtems_task Low_task(
  rtems_task_argument argument
) 
{ 
  rtems_id          id;
  rtems_unsigned32  index;
  rtems_unsigned32  count;
  rtems_status_code status;
   
  status = task_create( 
    build_name( 'H', 'I', 'G', 'H' ),
    5, 
    2048, 
    NO_PREEMPT, 
    DEFAULT_ATTRIBUTES, 
    &id 
  );
  directive_failed( status, "task_create" );

  status = task_start( id, High_task, 0 );
  directive_failed( status, "task_start HIGH" );

  status = message_queue_receive(
    Queue_id, 
    (long (*)[4]) Buffer, 
    DEFAULT_OPTIONS, 
    NO_TIMEOUT
  );
  directive_failed( status, "message_queu_receive" );

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) message_queue_broadcast(
               Queue_id, 
               (long (*)[4]) Buffer, 
               &count 
             );
  end_time = Read_timer();

  put_time(
    "MESSAGE_QUEUE_BROADCAST (no waiting tasks)",
    end_time,
    OPERATION_COUNT,
    1,
    CALLING_OVERHEAD_MESSAGE_QUEUE_BROADCAST
  );

  (void) message_queue_receive( 
           Queue_id, 
           (long (*)[4]) Buffer, 
           DEFAULT_OPTIONS, 
           NO_TIMEOUT 
         );

  /* should go to Preempt_task here */

  end_time = Read_timer();

  put_time(
    "MESSAGE_QUEUE_BROADCAST (preempt)",
    end_time,
    1,
    0,
    CALLING_OVERHEAD_MESSAGE_QUEUE_BROADCAST
  );

  exit( 0 );
}

rtems_task Preempt_task(
  rtems_task_argument argument
)
{
  rtems_unsigned32  count; 

  Timer_initialize();
    (void) message_queue_broadcast( 
             Queue_id, 
             (long (*)[4]) Buffer, 
             &count 
           );

 /* should be preempted by low task */
}
