/*
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

/*
#include "system.h"
#include "thread.h"
#include "sem.h"
#include "status.h"
#include "tasks.h"
#include "config.h"
#include "init.h"
*/

#include "system.h"
#include "config.h"
#include "clock.h"
#include "tasks.h"
#include "dpmem.h"
#include "event.h"
#include "fatal.h"
#include "heap.h"
#include "init.h"
#include "intthrd.h"
#include "isr.h"
#include "intr.h"
#include "io.h"
#include "object.h"
#include "message.h"
#include "mp.h"
#include "mpci.h"
#include "part.h"
#include "priority.h"
#include "ratemon.h"
#include "region.h"
#include "sem.h"
#include "signal.h"
#include "tasks.h"
#include "thread.h"
#include "timer.h"
#include "tod.h"
#include "userext.h"
#include "watchdog.h"
#include "wkspace.h"

#define DEFAULT_ATTRIBUTES   ATTRIBUTES_DEFAULTS
#define FLOATING_POINT       ATTRIBUTES_FLOATING_POINT
#define DEFAULT_MODES        MODES_DEFAULTS
#define NO_PREEMPT           MODES_NO_PREEMPT
#define SUCCESSFUL           STATUS_SUCCESSFUL

#define rtems_unsigned32     unsigned32
#define rtems_status_code    Status_Codes
#define rtems_task_argument  unsigned32
#define rtems_id             Objects_Id
#define rtems_name           Objects_Name
#define rtems_interval       Watchdog_Interval
#define rtems_device_driver  void

#define build_name           _Objects_Build_name
#define fatal_error_occurred _Fatal_Error_occurred
#define task_create          _RTEMS_tasks_Create
#define task_start           _RTEMS_tasks_Start
#define task_delete          _RTEMS_tasks_Delete
#define shutdown_executive   _Shutdown_Executive

typedef Configuration_Driver_address_table
                            rtems_driver_address_table;
typedef Configuration_Initialization_tasks_table
                            rtems_initialization_tasks_table;
typedef Configuration_Table rtems_configuration_table;

#define SELF 0

typedef void rtems_task;

/* #include "system.h" */
#include "coverhd.h"
#include "tmacros.h"
#include "timesys.h"
/* end of #include "system.h" */

#undef EXTERN
#define EXTERN
#include "gvar.h"
#include "conftbl.h"
#undef EXTERN
#define EXTERN extern

/* TEST DATA */
rtems_id Semaphore_id;

Objects_Locations location;

Thread_Control *Middle_tcb;

rtems_task High_task(
  rtems_task_argument argument
);

rtems_task Middle_task(
  rtems_task_argument argument
);

rtems_task Low_task(
  rtems_task_argument argument
);

rtems_task Floating_point_task_1(
  rtems_task_argument argument
);

rtems_task Floating_point_task_2(
  rtems_task_argument argument
);

int FP_use();

void complete_test( void );

rtems_task null_task(
  rtems_task_argument argument
)
{
}

rtems_task Init(
  rtems_task_argument argument
)
{
  rtems_unsigned32  index;
  rtems_id          task_id; 
  rtems_status_code status;

  puts( "\n\n*** TIME TEST 26 ***" );

  status = task_create( 
    build_name( 'F', 'P', '1', ' ' ),
    201, 
    2048, 
    DEFAULT_MODES, 
    FLOATING_POINT, 
    &task_id 
  );
  directive_failed( status, "task_create of FP1" );

  status = task_start( task_id, Floating_point_task_1, 0 );
  directive_failed( status, "task_start of FP1" );

  status = task_create( 
    build_name( 'F', 'P', '2', ' ' ),
    202, 
    2048, 
    DEFAULT_MODES, 
    FLOATING_POINT, 
    &task_id 
  );
  directive_failed( status, "task_create of FP2" );

  status = task_start( task_id, Floating_point_task_2, 0 );
  directive_failed( status, "task_start of FP2" );

  status = task_create( 
    build_name( 'L', 'O', 'W', ' ' ),
    200, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &task_id 
  );
  directive_failed( status, "task_create of LOW" );

  status = task_start( task_id, Low_task, 0 );
  directive_failed( status, "task_start of LOW" );

  status = task_create( 
    build_name( 'M', 'I', 'D', ' ' ),
    128, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &task_id 
  );
  directive_failed( status, "task_create of MIDDLE" );

  status = task_start( task_id, Middle_task, 0 );
  directive_failed( status, "task_start of MIDDLE" );

  status = task_create( 
    build_name( 'H', 'I', 'G', 'H' ),
    5, 
    2048, 
    DEFAULT_MODES, 
    DEFAULT_ATTRIBUTES, 
    &task_id 
  );
  directive_failed( status, "task_create of HIGH" );

  status = task_start( task_id, High_task, 0 );
  directive_failed( status, "task_start of HIGH" );

  status = _Semaphore_Create( 
    build_name( 'S', 'E', 'M', '1' ), 
    OPERATION_COUNT, 
    DEFAULT_ATTRIBUTES, 
    &Semaphore_id 
  );
  directive_failed( status, "semaphore_create" );

  for ( index = 1 ; index <= OPERATION_COUNT ; index++ ) {
    status = task_create( 
      build_name( 'N', 'U', 'L', 'L' ),
      254, 
      1024, 
      DEFAULT_MODES, 
      DEFAULT_ATTRIBUTES, 
      &task_id 
    );
    directive_failed( status, "task_create LOOP" );

    status = task_start( task_id, null_task, 0 );
    directive_failed( status, "task_start LOOP" );
  }

  status = task_delete( SELF );
  directive_failed( status, "task_delete of SELF" );
}

rtems_task High_task(
  rtems_task_argument argument
) 
{ 
  Timer_initialize();
    _Thread_Disable_dispatch();
  end_time = Read_timer();

  put_time(
    "THREAD_DISABLE_DISPATCH",
    end_time,
    1,
    0,
    0
  );


  Timer_initialize();
    _Thread_Enable_dispatch();
  end_time = Read_timer();

  put_time(
    "THREAD_ENABLE_DISPATCH",
    end_time,
    1,
    0,
    0
  );

  Timer_initialize();
    _Thread_Set_state( _Thread_Executing, STATES_SUSPENDED ); 
  end_time = Read_timer();

  put_time(
    "THREAD_SET_STATE",
    end_time,
    1,
    0,
    0
  );

  _Context_Switch_necessary = TRUE;

  Timer_initialize();
    _Thread_Dispatch();           /* dispatches Middle_task */
}

rtems_task Middle_task(
  rtems_task_argument argument
) 
{
  end_time = Read_timer();

  put_time(
    "THREAD_DISPATCH (NO FP)",
    end_time,
    1,
    0,
    0
  );

  _Thread_Set_state( _Thread_Executing, STATES_SUSPENDED ); 

  Middle_tcb   = _Thread_Executing;

  _Thread_Executing = 
        (Thread_Control *) _Thread_Ready_chain[200].first; 

  /* do not force context switch */

  _Context_Switch_necessary = FALSE;

  Timer_initialize(); 
    _Context_Switch( &Middle_tcb->Registers, &_Thread_Executing->Registers ); 
}

rtems_task Low_task(
  rtems_task_argument argument
)
{
  Thread_Control *executing;

  end_time = Read_timer();

  put_time(
    "CONTEXT_SWITCH (NO FP)",
    end_time,
    1,
    0,
    0
  );

  executing    = _Thread_Executing;

  _Thread_Executing = 
        (Thread_Control *) _Thread_Ready_chain[201].first; 

  /* do not force context switch */

  _Context_Switch_necessary = FALSE;

  Timer_initialize(); 
    _Context_Restore_fp( &_Thread_Executing->fp_context ); 
    _Context_Switch( &executing->Registers, &_Thread_Executing->Registers );
}

rtems_task Floating_point_task_1(
  rtems_task_argument argument
)
{
  Thread_Control *executing;

  end_time = Read_timer();

  put_time(
    "CONTEXT_SWITCH (restore 1st FP)",
    end_time,
    1,
    0,
    0
  );

  executing = _Thread_Executing;

  _Thread_Executing = 
        (Thread_Control *) _Thread_Ready_chain[202].first; 

  /* do not force context switch */

  _Context_Switch_necessary = FALSE;

  Timer_initialize(); 
    _Context_Save_fp( &executing->fp_context ); 
    _Context_Restore_fp( &_Thread_Executing->fp_context ); 
    _Context_Switch( &executing->Registers, &_Thread_Executing->Registers ); 
  /* switch to Floating_point_task_2 */

  end_time = Read_timer();

  put_time(
    "CONTEXT_SWITCH (used->init FP)",
    end_time,
    1,
    0,
    0
  );

  (void) FP_use();

  executing = _Thread_Executing;

  _Thread_Executing = 
       (Thread_Control *) _Thread_Ready_chain[202].first; 

  /* do not force context switch */

  _Context_Switch_necessary = FALSE;

  Timer_initialize(); 
    _Context_Save_fp( &executing->fp_context ); 
    _Context_Restore_fp( &_Thread_Executing->fp_context ); 
    _Context_Switch( &executing->Registers, &_Thread_Executing->Registers ); 
  /* switch to Floating_point_task_2 */
}

rtems_task Floating_point_task_2(
  rtems_task_argument argument
)
{
  Thread_Control *executing;

  end_time = Read_timer();

  put_time(
    "CONTEXT_SWITCH (init->init FP)",
    end_time,
    1,
    0,
    0
  );

  executing = _Thread_Executing;

  _Thread_Executing = 
       (Thread_Control *) _Thread_Ready_chain[201].first; 

  (void) FP_use();

  /* do not force context switch */

  _Context_Switch_necessary = FALSE;

  Timer_initialize(); 
    _Context_Save_fp( &executing->fp_context ); 
    _Context_Restore_fp( &_Thread_Executing->fp_context ); 
    _Context_Switch( &executing->Registers, &_Thread_Executing->Registers ); 
  /* switch to Floating_point_task_1 */

  end_time = Read_timer();

  put_time(
    "CONTEXT_SWITCH (used->used FP)",
    end_time,
    1,
    0,
    0
  );

  complete_test();
}

void complete_test( void )
{
  rtems_unsigned32  index;
  rtems_id          task_id;

  Timer_initialize();
    _Thread_Resume( Middle_tcb );
  end_time = Read_timer();

  put_time(
    "THREAD_RESUME",
    end_time,
    1,
    0,
    0
  );

  _Thread_Set_state( Middle_tcb, STATES_WAITING_FOR_MESSAGE );

  Timer_initialize();
    _Thread_Unblock( Middle_tcb );
  end_time = Read_timer();

  put_time(
    "THREAD_UNBLOCK",
    end_time,
    1,
    0,
    0
  );
  
  _Thread_Set_state( Middle_tcb, STATES_WAITING_FOR_MESSAGE );

  Timer_initialize();
    _Thread_Ready( Middle_tcb );
  end_time = Read_timer();

  put_time(
    "THREAD_READY",
    end_time,
    1,
    0,
    0
  );

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) Empty_function();
  overhead = Read_timer();

  task_id = Middle_tcb->Object.id;

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) _Thread_Get( task_id, &location );
  end_time = Read_timer();

  put_time(
    "THREAD_GET",
    end_time,
    OPERATION_COUNT,
    0,
    0
  );

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) _Semaphore_Get( Semaphore_id, &location );
  end_time = Read_timer();

  put_time(
    "SEMAPHORE_GET",
    end_time,
    OPERATION_COUNT,
    0,
    0
  );

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) _Thread_Get( 0x3, &location );
  end_time = Read_timer();

  put_time(
    "THREAD_GET invalid id",
    end_time,
    OPERATION_COUNT,
    0,
    0
  );
  exit( 0 );
}

