/*
 *
 *  COPYRIGHT (c) 1989, 1990, 1991, 1992, 1993, 1994.
 *  On-Line Applications Research Corporation (OAR).
 *  All rights assigned to U.S. Government, 1994.
 *
 *  This material may be reproduced by or for the U.S. Government pursuant
 *  to the copyright license under the clause at DFARS 252.227-7013.  This
 *  notice must appear in all copies of this file and its derivatives.
 *
 *  $Id$
 */

#include "system.h"
#undef EXTERN
#define EXTERN
#include "conftbl.h"
#include "gvar.h"

rtems_id Task_id[ OPERATION_COUNT + 1 ];

rtems_unsigned32 Task_restarted;

rtems_task null_task(
  rtems_task_argument argument
);

rtems_task Task_1(
  rtems_task_argument argument
);

void test_init( void );

rtems_task Init(
  rtems_task_argument argument
)
{
  rtems_status_code status;

  puts( "\n\n*** TIME TEST 6 ***" );

  test_init();

  status = task_delete( SELF );
  directive_failed( status, "task_delete of SELF" );
}

void test_init( void )
{
  rtems_status_code status;
  rtems_id id;

  Task_restarted = OPERATION_COUNT;

  status = task_create( 
    build_name( 'T', 'I', 'M', 'E' ), 
    128, 
    1024, 
    DEFAULT_MODES,
    DEFAULT_ATTRIBUTES, 
    &id 
  );
  directive_failed( status, "task_create" );

  status = task_start( id, Task_1, 0 );
  directive_failed( status, "task_start" );
}

rtems_task Task_1(
  rtems_task_argument argument
)
{
  rtems_status_code status;
  rtems_unsigned32  index;
    
  if ( Task_restarted == OPERATION_COUNT )
     Timer_initialize();

  Task_restarted--;

  if ( Task_restarted != 0 ) 
    (void) task_restart( SELF, 0 );

  end_time = Read_timer();
   
  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) Empty_function();
  overhead = Read_timer();

  put_time(
    "TASK_RESTART self",
    end_time,
    OPERATION_COUNT,
    overhead,
    CALLING_OVERHEAD_TASK_RESTART
  );

  for ( index=1 ; index <= OPERATION_COUNT ; index++ ) {
    status = task_create( 
      build_name( 'T', 'I', 'M', 'E' ), 
      254, 
      1024, 
      DEFAULT_MODES, 
      DEFAULT_ATTRIBUTES, 
      &Task_id[ index ] 
    );
    directive_failed( status, "task_create loop" );

    status = task_start( Task_id[ index ], null_task, 0 );
    directive_failed( status, "task_start loop" );
  }

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) task_suspend( Task_id[ index ] );
  end_time = Read_timer();

  put_time(
    "TASK_SUSPEND (no preempt)",
    end_time,
    OPERATION_COUNT,
    0,
    CALLING_OVERHEAD_TASK_SUSPEND
  );

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) task_resume( Task_id[ index ] );
  end_time = Read_timer();

  put_time(
    "TASK_RESUME (no preempt)",
    end_time,
    OPERATION_COUNT,
    0,
    CALLING_OVERHEAD_TASK_RESUME
  );

  Timer_initialize();
    for ( index=1 ; index <= OPERATION_COUNT ; index++ )
      (void) task_delete( Task_id[ index ] );
  end_time = Read_timer();

  put_time(
    "TASK_DELETE (others)",
    end_time,
    OPERATION_COUNT,
    0,
    CALLING_OVERHEAD_TASK_RESUME
  );

  exit( 0 );
}

rtems_task null_task(
  rtems_task_argument argument
)
{
  while ( FOREVER )
    ;
}
